-- mth34x16.vhd
--
--
-- Copyright 1993,1994,1995,1996 Cypress Semiconductor as an unpublished work.
--
-- Rev 1.0	09-20-93
-- Rev 1.1	12-06-93
-- Rev 1.2  01-29-94
--
--
-- This is a package that defines math components of various sizes,
-- namely 8,12,16,24.
-- The components are defined and entity architecture pairs
-- follow, suitable for implementation in 34X devices (ie.
-- the architectures render operations in terms of expanders and
-- macrocells). The synthesis_off attribute is used and use of
-- the -xor synthesis directive is expected.
--
-- Components:
-- adder: add_16
-- subtractor: sub_16
-- grt than(>): gt_16
-- less than(<): lt_16
-- gt or eq(>=): ge_16
-- lt or eq(<=): lt_16
--
--
package math34x16_pkg is

component add_16 port(
  a,b:in bit_vector(15 downto 0);
  s:out bit_vector(15 downto 0)
  );
end component;
component sub_16 port(
  a,b:in bit_vector(15 downto 0);
  s:out bit_vector(15 downto 0)
  );
end component;


component gt_16 port(
  a,b:in bit_vector(15 downto 0);
  a_gt_b:out bit
  );
end component;


component lt_16 port(
  a,b:in bit_vector(15 downto 0);
  a_lt_b:out bit
  );
end component;

component ge_16 port(
  a,b:in bit_vector(15 downto 0);
  a_ge_b:out bit
  );
end component;

component le_16 port(
  a,b:in bit_vector(15 downto 0);
  a_le_b:out bit
  );
end component;
end math34x16_pkg;

-- ***************************************************
-- entities and architectures for math components
-- ***************************************************

use work.cypress.all;
entity add_16 is port(
  a,b:in bit_vector(15 downto 0);
  s:out bit_vector(15 downto 0)
  );
end add_16;

--***********************************************
architecture ma34x of add_16 is
 signal x:bit_vector(1 to 31);
 attribute synthesis_off of x:signal is true;
 signal si:bit_vector(15 downto 0);
 attribute synthesis_off of si:signal is true;
 signal eq:bit_vector(1 to 28);
 signal car:bit_vector(0 to 3);
 attribute synthesis_off of car:signal is true;
 signal em:bit_vector(0 to 7);

begin
s<=si;

-- macrocell:
si(0)<=eq(2) xor eq(1);
eq(1)<=not(b(0)) and not(a(0)) and x(1);
eq(2)<=x(1);
   
-- expander:
x(1)<=not(a(0) and b(0));

-- macrocell:
si(1)<=eq(4) xor eq(3);
eq(3)<=x(2) and x(3);
eq(4)<=b(0) and a(0);
   
-- expanders:
x(2)<=not(not(a(1)) and not(b(1)));
x(3)<=not(a(1) and b(1));

-- macrocell:
si(2)<=eq(6) xor eq(5);
eq(5)<=(b(0) and a(0)and x(2))
     or (b(1) and a(1));
eq(6)<=x(4) and x(5);

-- expanders:
x(4)<=not(not(a(2)) and not(b(2)));
x(5)<=not(a(2) and b(2));

-- macrocell:
si(3)<=eq(8) xor eq(7);
eq(7)<=(b(0) and a(0) and x(2) and x(4))
     or (b(1) and a(1) and x(4))
     or (b(2) and a(2));
eq(8)<=x(6) and x(7);

-- expanders:
x(6)<=not(not(a(3)) and not(b(3)));
x(7)<=not(a(3) and b(3));

-- carry macrocell zero:
car(0)<=em(1) xor em(0);
em(0)<=(b(0) and a(0) and x(2) and x(4) and x(6) and x(7))
     or (b(1) and a(1) and x(4) and x(6) and x(7))
     or (b(2) and a(2) and x(6) and x(7));
em(1)<=a(3) and b(3);

-- macrocell:
si(4)<=eq(9) xor car(0);
eq(9)<=x(8) and x(9);

-- expanders:
x(8)<=not(not(a(4)) and not(b(4)));
x(9)<=not(a(4) and b(4));

-- macrocell:
si(5)<=eq(11) xor eq(10);
eq(10)<=(a(4) and b(4))
      or (car(0) and x(8));
eq(11)<=x(10) and x(11);

-- expanders:
x(11)<=not(not(a(5)) and not(b(5)));
x(10)<=not(a(5) and b(5));

-- macrocell:
si(6)<=eq(13) xor eq(12);
eq(12)<=(a(4) and b(4) and x(11))
      or (car(0) and x(8) and x(11))
      or (a(5) and b(5));
eq(13)<=x(12) and x(13);

-- expanders:
x(12)<=not(not(a(6)) and not(b(6)));
x(13)<=not(a(6) and b(6));

-- carry macrocell one:
car(1)<=em(3) xor em(2);
em(2)<=(b(4) and a(4) and x(11) and x(12) and x(13))
     or (car(0) and x(8) and x(11) and x(12) and x(13))
     or (b(5) and a(5) and x(12) and x(13));
em(3)<=a(6) and b(6);

-- macrocell:
si(7)<=eq(14) xor car(1);
eq(14)<=x(14) and x(15);

-- expanders:
x(14)<=not(not(a(7)) and not(b(7)));
x(15)<=not(a(7) and b(7));

-- macrocell:
si(8)<=eq(16) xor eq(15);
eq(15)<=(a(7) and b(7))
      or (car(1) and x(14));
eq(16)<=x(16) and x(17);

-- expanders:
x(16)<=not(not(a(8)) and not(b(8)));
x(17)<=not(a(8) and b(8));

-- macrocell:
si(9)<=eq(18) xor eq(17);
eq(17)<=(a(7) and b(7) and x(16))
      or (car(1) and x(14) and x(16))
      or (a(8) and b(8));
eq(18)<=x(18) and x(19);

-- expanders:
x(18)<=not(not(a(9)) and not(b(9)));
x(19)<=not(a(9) and b(9));

-- carry macrocell two:
car(2)<=em(5) xor em(4);
em(4)<=(b(7) and a(7) and x(16) and x(18) and x(19))
     or (car(1) and x(14) and x(16) and x(18) and x(19))
     or (b(8) and a(8) and x(18) and x(19));
em(5)<=a(9) and b(9);

-- macrocell:
si(10)<=car(2) xor eq(19);
eq(19)<=x(20) and x(21);

-- expanders:
x(20)<=not(a(10) and b(10));
x(21)<=not(not(a(10)) and not(b(10)));

-- macrocell:
si(11)<=eq(21) xor eq(20);
eq(20)<=(a(10) and b(10))
     or (car(2) and x(21));
eq(21)<=x(22) and x(23);

-- expanders:
x(22)<=not(not(a(11)) and not(b(11)));
x(23)<=not(a(11) and b(11));

-- macrocell:
si(12)<=eq(23) xor eq(22);
eq(22)<=(a(10) and b(10) and x(22))
     or (car(2) and x(21) and x(22))
     or (a(11) and b(11));
eq(23)<=x(24) and x(25);

-- expanders:
x(24)<=not(not(a(12)) and not(b(12)));
x(25)<=not(a(12) and b(12));

-- carry macrocell three:
car(3)<=em(7) xor em(6);
em(6)<=(b(10) and a(10) and x(22) and x(24) and x(25))
     or (car(2) and x(21) and x(22) and x(24) and x(25))
     or (b(11) and a(11) and x(24) and x(25));
em(7)<=a(12) and b(12);

-- macrocell:
si(13)<=car(3) xor eq(24);
eq(24)<=x(26) and x(27);

-- expanders:
x(26)<=not(a(13) and b(13));
x(27)<=not(not(a(13)) and not(b(13)));

-- macrocell:
si(14)<=eq(26) xor eq(25);
eq(25)<=(a(13) and b(13))
     or (car(3) and x(27));
eq(26)<=x(28) and x(29);

-- expanders:
x(28)<=not(not(a(14)) and not(b(14)));
x(29)<=not(a(14) and b(14));

-- macrocell:
si(15)<=eq(28) xor eq(27);
eq(27)<=(a(13) and b(13) and x(28))
     or (car(3) and x(27) and x(28))
     or (a(14) and b(14));
eq(28)<=x(30) and x(31);

-- expanders:
x(30)<=not(not(a(15)) and not(b(15)));
x(31)<=not(a(15) and b(15));

end ma34x;
---***********************************************
--   sub_n component ****************************
--***********************************************
--************************************************

use work.cypress.all;
entity sub_16 is port(
  a,b:in bit_vector(15 downto 0);
  s:out bit_vector(15 downto 0)
  );
end sub_16;

--************************************************
architecture ma34x of sub_16 is 
 signal x:bit_vector(1 to 31);
 attribute synthesis_off of x:signal is true;
 signal si:bit_vector(15 downto 0);
 attribute synthesis_off of si:signal is true;
 signal eq:bit_vector(1 to 28);
 signal car:bit_vector(0 to 3);
 attribute synthesis_off of car:signal is true;
 signal em:bit_vector(0 to 7);

begin
s<=si;

-- macrocell:
si(0)<=eq(2) xor eq(1);
eq(1)<=not(a(0)) or b(0);
eq(2)<=x(1);
   
-- expander:
x(1)<=not(not(a(0)) and b(0));

-- macrocell:
si(1)<=eq(4) xor eq(3);
eq(3)<=x(1);
eq(4)<=x(2) and x(3);
   
-- expanders:
x(2)<=not(not(a(1)) and b(1));
x(3)<=not(a(1) and not(b(1)));

-- macrocell:
si(2)<=eq(6) xor eq(5);
eq(5)<= (x(1) and x(2))
     or (not(b(1)) and a(1));
eq(6)<=x(4) and x(5);

-- expanders:
x(4)<=not(not(a(2)) and b(2));
x(5)<=not(a(2) and not(b(2)));

-- macrocell:
si(3)<=eq(8) xor eq(7);
eq(7)<=(x(1) and x(2) and x(4))
     or (not(b(1)) and a(1) and x(4))
     or (not(b(2)) and a(2));
eq(8)<=x(6) and x(7);

-- expanders:
x(6)<=not(not(a(3)) and b(3));
x(7)<=not(a(3) and not(b(3)));

-- carry macrocell zero:
car(0)<=em(1) xor em(0);
em(0)<=(x(1) and x(2) and x(4) and x(6) and x(7))
     or (not(b(1)) and a(1) and x(4) and x(6) and x(7))
     or (not(b(2)) and a(2) and x(6) and x(7));
em(1)<=a(3) and not(b(3));

-- macrocell:
si(4)<=eq(9) xor car(0);
eq(9)<=x(8) and x(9);

-- expanders:
x(8)<=not(not(a(4)) and b(4));
x(9)<=not(a(4) and not(b(4)));

-- macrocell:
si(5)<=eq(11) xor eq(10);
eq(10)<=(a(4) and not(b(4)))
      or (car(0) and x(8));
eq(11)<=x(10) and x(11);

-- expanders:
x(11)<=not(not(a(5)) and b(5));
x(10)<=not(a(5) and not(b(5)));

-- macrocell:
si(6)<=eq(13) xor eq(12);
eq(12)<=(a(4) and not(b(4)) and x(11))
      or (car(0) and x(8) and x(11))
      or (a(5) and not(b(5)));
eq(13)<=x(12) and x(13);

-- expanders:
x(12)<=not(not(a(6)) and b(6));
x(13)<=not(a(6) and not(b(6)));

-- carry macrocell one:
car(1)<=em(3) xor em(2);
em(2)<=(not(b(4)) and a(4) and x(11) and x(12) and x(13))
     or (car(0) and x(8) and x(11) and x(12) and x(13))
     or (not(b(5)) and a(5) and x(12) and x(13));
em(3)<=a(6) and not(b(6));

-- macrocell:
si(7)<=eq(14) xor car(1);
eq(14)<=x(14) and x(15);

-- expanders:
x(14)<=not(not(a(7)) and b(7));
x(15)<=not(a(7) and not(b(7)));

-- macrocell:
si(8)<=eq(16) xor eq(15);
eq(15)<=(a(7) and not(b(7)))
      or (car(1) and x(14));
eq(16)<=x(16) and x(17);

-- expanders:
x(16)<=not(not(a(8)) and b(8));
x(17)<=not(a(8) and not(b(8)));

-- macrocell:
si(9)<=eq(18) xor eq(17);
eq(17)<=(a(7) and not(b(7)) and x(16))
      or (car(1) and x(14) and x(16))
      or (a(8) and not(b(8)));
eq(18)<=x(18) and x(19);

-- expanders:
x(18)<=not(not(a(9)) and b(9));
x(19)<=not(a(9) and not(b(9)));

-- carry macrocell two:
car(2)<=em(5) xor em(4);
em(4)<=(not(b(7)) and a(7) and x(16) and x(18) and x(19))
     or (car(1) and x(14) and x(16) and x(18) and x(19))
     or (not(b(8)) and a(8) and x(18) and x(19));
em(5)<=a(9) and not(b(9));

-- macrocell:
si(10)<=car(2) xor eq(19);
eq(19)<=x(20) and x(21);

-- expanders:
x(20)<=not(a(10) and not(b(10)));
x(21)<=not(not(a(10)) and b(10));

-- macrocell:
si(11)<=eq(21) xor eq(20);
eq(20)<=(a(10) and not(b(10)))
     or (car(2) and x(21));
eq(21)<=x(22) and x(23);

-- expanders:
x(22)<=not(not(a(11)) and b(11));
x(23)<=not(a(11) and not(b(11)));

-- macrocell:
si(12)<=eq(23) xor eq(22);
eq(22)<=(a(10) and not(b(10)) and x(22))
     or (car(2) and x(21) and x(22))
     or (a(11) and not(b(11)));
eq(23)<=x(24) and x(25);

-- expanders:
x(24)<=not(not(a(12)) and b(12));
x(25)<=not(a(12) and not(b(12)));

-- carry macrocell three:
car(3)<=em(7) xor em(6);
em(6)<=(not(b(10)) and a(10) and x(22) and x(24) and x(25))
     or (car(2) and x(21) and x(22) and x(24) and x(25))
     or (not(b(11)) and a(11) and x(24) and x(25));
em(7)<=a(12) and not(b(12));

-- macrocell:
si(13)<=car(3) xor eq(24);
eq(24)<=x(26) and x(27);

-- expanders:
x(26)<=not(not(a(13)) and b(13));
x(27)<=not(a(13) and not(b(13)));

-- macrocell:
si(14)<=eq(26) xor eq(25);
eq(25)<=(a(13) and not(b(13)))
     or (car(3) and x(26));
eq(26)<=x(28) and x(29);

-- expanders:
x(28)<=not(not(a(14)) and b(14));
x(29)<=not(a(14) and not(b(14)));

-- macrocell:
si(15)<=eq(28) xor eq(27);
eq(27)<=(a(13) and not(b(13)) and x(28))
     or (car(3) and x(26) and x(28))
     or (a(14) and not(b(14)));
eq(28)<=x(30) and x(31);

-- expanders:
x(30)<=not(not(a(15)) and b(15));
x(31)<=not(a(15) and not(b(15)));

end ma34x;
--************************************************

--************************************************

use work.cypress.all;
entity gt_16 is port(
  a,b:in bit_vector(15 downto 0);
  a_gt_b: out bit
  );
end gt_16;


architecture ma34x of gt_16 is 
 signal x:bit_vector(1 to 31);
 attribute synthesis_off of x:signal is true;
 signal si:bit_vector(16 downto 0);
 attribute synthesis_off of si:signal is true;
 signal eq:bit_vector(1 to 30);
 signal car:bit_vector(0 to 3);
 attribute synthesis_off of car:signal is true;
 signal em:bit_vector(0 to 7);

begin
a_gt_b<=si(16);

-- macrocell:
si(0)<=eq(2) xor eq(1);
eq(1)<=b(0) and not(a(0)) and x(1);
eq(2)<=x(1);
   
-- expander:
x(1)<=not(not(a(0)) and b(0));

-- macrocell:
si(1)<=eq(4) xor eq(3);
eq(3)<=a(0) and not(b(0));
eq(4)<=x(2) and x(3);
   
-- expanders:
x(2)<=not(not(a(1)) and b(1));
x(3)<=not(a(1) and not(b(1)));

-- macrocell:
si(2)<=eq(6) xor eq(5);
eq(5)<= (not(b(0)) and a(0) and x(2))
     or (not(b(1)) and a(1));
eq(6)<=x(4) and x(5);

-- expanders:
x(4)<=not(not(a(2)) and b(2));
x(5)<=not(a(2) and not(b(2)));

-- macrocell:
si(3)<=eq(8) xor eq(7);
eq(7)<=(not(b(0)) and a(0) and x(2) and x(4))
     or (not(b(1)) and a(1) and x(4))
     or (not(b(2)) and a(2));
eq(8)<=x(6) and x(7);

-- expanders:
x(6)<=not(not(a(3)) and b(3));
x(7)<=not(a(3) and not(b(3)));

-- carry macrocell zero:
car(0)<=em(1) xor em(0);
em(0)<=(not(b(0)) and a(0) and x(2) and x(4) and x(6) and x(7))
     or (not(b(1)) and a(1) and x(4) and x(6) and x(7))
     or (not(b(2)) and a(2) and x(6) and x(7));
em(1)<=a(3) and not(b(3));

-- macrocell:
si(4)<=eq(9) xor car(0);
eq(9)<=x(8) and x(9);

-- expanders:
x(8)<=not(not(a(4)) and b(4));
x(9)<=not(a(4) and not(b(4)));

-- macrocell:
si(5)<=eq(11) xor eq(10);
eq(10)<=(a(4) and not(b(4)))
      or (car(0) and x(8));
eq(11)<=x(10) and x(11);

-- expanders:
x(11)<=not(not(a(5)) and b(5));
x(10)<=not(a(5) and not(b(5)));

-- macrocell:
si(6)<=eq(13) xor eq(12);
eq(12)<=(a(4) and not(b(4)) and x(11))
      or (car(0) and x(8) and x(11))
      or (a(5) and not(b(5)));
eq(13)<=x(12) and x(13);

-- expanders:
x(12)<=not(not(a(6)) and b(6));
x(13)<=not(a(6) and not(b(6)));

-- carry macrocell one:
car(1)<=em(3) xor em(2);
em(2)<=(not(b(4)) and a(4) and x(11) and x(12) and x(13))
     or (car(0) and x(8) and x(11) and x(12) and x(13))
     or (not(b(5)) and a(5) and x(12) and x(13));
em(3)<=a(6) and not(b(6));

-- macrocell:
si(7)<=eq(14) xor car(1);
eq(14)<=x(14) and x(15);

-- expanders:
x(14)<=not(not(a(7)) and b(7));
x(15)<=not(a(7) and not(b(7)));

-- macrocell:
si(8)<=eq(16) xor eq(15);
eq(15)<=(a(7) and not(b(7)))
      or (car(1) and x(14));
eq(16)<=x(16) and x(17);

-- expanders:
x(16)<=not(not(a(8)) and b(8));
x(17)<=not(a(8) and not(b(8)));

-- macrocell:
si(9)<=eq(18) xor eq(17);
eq(17)<=(a(7) and not(b(7)) and x(16))
      or (car(1) and x(14) and x(16))
      or (a(8) and not(b(8)));
eq(18)<=x(18) and x(19);

-- expanders:
x(18)<=not(not(a(9)) and b(9));
x(19)<=not(a(9) and not(b(9)));

-- carry macrocell two:
car(2)<=em(5) xor em(4);
em(4)<=(not(b(7)) and a(7) and x(16) and x(18) and x(19))
     or (car(1) and x(14) and x(16) and x(18) and x(19))
     or (not(b(8)) and a(8) and x(18) and x(19));
em(5)<=a(9) and not(b(9));

-- macrocell:
si(10)<=car(2) xor eq(19);
eq(19)<=x(20) and x(21);

-- expanders:
x(20)<=not(a(10) and not(b(10)));
x(21)<=not(not(a(10)) and b(10));

-- macrocell:
si(11)<=eq(21) xor eq(20);
eq(20)<=(a(10) and not(b(10)))
     or (car(2) and x(21));
eq(21)<=x(22) and x(23);

-- expanders:
x(22)<=not(not(a(11)) and b(11));
x(23)<=not(a(11) and not(b(11)));

-- macrocell:
si(12)<=eq(23) xor eq(22);
eq(22)<=(a(10) and not(b(10)) and x(22))
     or (car(2) and x(21) and x(22))
     or (a(11) and not(b(11)));
eq(23)<=x(24) and x(25);

-- expanders:
x(24)<=not(not(a(12)) and b(12));
x(25)<=not(a(12) and not(b(12)));

-- carry macrocell three:
car(3)<=em(7) xor em(6);
em(6)<=(not(b(10)) and a(10) and x(22) and x(24) and x(25))
     or (car(2) and x(21) and x(22) and x(24) and x(25))
     or (not(b(11)) and a(11) and x(24) and x(25));
em(7)<=a(12) and not(b(12));

-- macrocell:
si(13)<=car(3) xor eq(24);
eq(24)<=x(26) and x(27);

-- expanders:
x(26)<=not(not(a(13)) and b(13));
x(27)<=not(a(13) and not(b(13)));

-- macrocell:
si(14)<=eq(26) xor eq(25);
eq(25)<=(a(13) and not(b(13)))
     or (car(3) and x(26));
eq(26)<=x(28) and x(29);

-- expanders:
x(28)<=not(not(a(14)) and b(14));
x(29)<=not(a(14) and not(b(14)));

-- macrocell:
si(15)<=eq(28) xor eq(27);
eq(27)<=(a(13) and not(b(13)) and x(28))
     or (car(3) and x(26) and x(28))
     or (a(14) and not(b(14)));
eq(28)<=x(30) and x(31);

-- expanders:
x(30)<=not(not(a(15)) and b(15));
x(31)<=not(a(15) and not(b(15)));

-- macrocell:
si(16)<=eq(29) xor eq(30);
eq(29)<=(not(a(15)) and b(15))
     or (not(si(15)) and b(15));
eq(30)<=( not(si(15)) and not(a(15)) and not(b(15)) );


end ma34x; -- gt_16
--************************************************

--************************************************

use work.cypress.all;
entity lt_16 is port(
  a,b:in bit_vector(15 downto 0);
  a_lt_b: out bit
  );
end lt_16;

architecture ma34x of lt_16 is 
 signal x:bit_vector(1 to 31);
 attribute synthesis_off of x:signal is true;
 signal si:bit_vector(16 downto 0);
 attribute synthesis_off of si:signal is true;
 signal eq:bit_vector(1 to 30);
 signal car:bit_vector(0 to 3);
 attribute synthesis_off of car:signal is true;
 signal em:bit_vector(0 to 7);

begin
a_lt_b<=si(16);

-- macrocell:
si(0)<=eq(2) xor eq(1);
eq(1)<=not(a(0)) or b(0);
eq(2)<=x(1);
   
-- expander:
x(1)<=not(not(a(0)) and b(0));

-- macrocell:
si(1)<=eq(4) xor eq(3);
eq(3)<=x(1);
eq(4)<=x(2) and x(3);
   
-- expanders:
x(2)<=not(not(a(1)) and b(1));
x(3)<=not(a(1) and not(b(1)));

-- macrocell:
si(2)<=eq(6) xor eq(5);
eq(5)<= (x(1) and x(2))
     or (not(b(1)) and a(1));
eq(6)<=x(4) and x(5);

-- expanders:
x(4)<=not(not(a(2)) and b(2));
x(5)<=not(a(2) and not(b(2)));

-- macrocell:
si(3)<=eq(8) xor eq(7);
eq(7)<=(x(1) and x(2) and x(4))
     or (not(b(1)) and a(1) and x(4))
     or (not(b(2)) and a(2));
eq(8)<=x(6) and x(7);

-- expanders:
x(6)<=not(not(a(3)) and b(3));
x(7)<=not(a(3) and not(b(3)));

-- carry macrocell zero:
car(0)<=em(1) xor em(0);
em(0)<=(x(1) and x(2) and x(4) and x(6) and x(7))
     or (not(b(1)) and a(1) and x(4) and x(6) and x(7))
     or (not(b(2)) and a(2) and x(6) and x(7));
em(1)<=a(3) and not(b(3));

-- macrocell:
si(4)<=eq(9) xor car(0);
eq(9)<=x(8) and x(9);

-- expanders:
x(8)<=not(not(a(4)) and b(4));
x(9)<=not(a(4) and not(b(4)));

-- macrocell:
si(5)<=eq(11) xor eq(10);
eq(10)<=(a(4) and not(b(4)))
      or (car(0) and x(8));
eq(11)<=x(10) and x(11);

-- expanders:
x(11)<=not(not(a(5)) and b(5));
x(10)<=not(a(5) and not(b(5)));

-- macrocell:
si(6)<=eq(13) xor eq(12);
eq(12)<=(a(4) and not(b(4)) and x(11))
      or (car(0) and x(8) and x(11))
      or (a(5) and not(b(5)));
eq(13)<=x(12) and x(13);

-- expanders:
x(12)<=not(not(a(6)) and b(6));
x(13)<=not(a(6) and not(b(6)));

-- carry macrocell one:
car(1)<=em(3) xor em(2);
em(2)<=(not(b(4)) and a(4) and x(11) and x(12) and x(13))
     or (car(0) and x(8) and x(11) and x(12) and x(13))
     or (not(b(5)) and a(5) and x(12) and x(13));
em(3)<=a(6) and not(b(6));

-- macrocell:
si(7)<=eq(14) xor car(1);
eq(14)<=x(14) and x(15);

-- expanders:
x(14)<=not(not(a(7)) and b(7));
x(15)<=not(a(7) and not(b(7)));

-- macrocell:
si(8)<=eq(16) xor eq(15);
eq(15)<=(a(7) and not(b(7)))
      or (car(1) and x(14));
eq(16)<=x(16) and x(17);

-- expanders:
x(16)<=not(not(a(8)) and b(8));
x(17)<=not(a(8) and not(b(8)));

-- macrocell:
si(9)<=eq(18) xor eq(17);
eq(17)<=(a(7) and not(b(7)) and x(16))
      or (car(1) and x(14) and x(16))
      or (a(8) and not(b(8)));
eq(18)<=x(18) and x(19);

-- expanders:
x(18)<=not(not(a(9)) and b(9));
x(19)<=not(a(9) and not(b(9)));

-- carry macrocell two:
car(2)<=em(5) xor em(4);
em(4)<=(not(b(7)) and a(7) and x(16) and x(18) and x(19))
     or (car(1) and x(14) and x(16) and x(18) and x(19))
     or (not(b(8)) and a(8) and x(18) and x(19));
em(5)<=a(9) and not(b(9));

-- macrocell:
si(10)<=car(2) xor eq(19);
eq(19)<=x(20) and x(21);

-- expanders:
x(20)<=not(a(10) and not(b(10)));
x(21)<=not(not(a(10)) and b(10));

-- macrocell:
si(11)<=eq(21) xor eq(20);
eq(20)<=(a(10) and not(b(10)))
     or (car(2) and x(21));
eq(21)<=x(22) and x(23);

-- expanders:
x(22)<=not(not(a(11)) and b(11));
x(23)<=not(a(11) and not(b(11)));

-- macrocell:
si(12)<=eq(23) xor eq(22);
eq(22)<=(a(10) and not(b(10)) and x(22))
     or (car(2) and x(21) and x(22))
     or (a(11) and not(b(11)));
eq(23)<=x(24) and x(25);

-- expanders:
x(24)<=not(not(a(12)) and b(12));
x(25)<=not(a(12) and not(b(12)));

-- carry macrocell three:
car(3)<=em(7) xor em(6);
em(6)<=(not(b(10)) and a(10) and x(22) and x(24) and x(25))
     or (car(2) and x(21) and x(22) and x(24) and x(25))
     or (not(b(11)) and a(11) and x(24) and x(25));
em(7)<=a(12) and not(b(12));

-- macrocell:
si(13)<=car(3) xor eq(24);
eq(24)<=x(26) and x(27);

-- expanders:
x(26)<=not(not(a(13)) and b(13));
x(27)<=not(a(13) and not(b(13)));

-- macrocell:
si(14)<=eq(26) xor eq(25);
eq(25)<=(a(13) and not(b(13)))
     or (car(3) and x(26));
eq(26)<=x(28) and x(29);

-- expanders:
x(28)<=not(not(a(14)) and b(14));
x(29)<=not(a(14) and not(b(14)));

-- macrocell:
si(15)<=eq(28) xor eq(27);
eq(27)<=(a(13) and not(b(13)) and x(28))
     or (car(3) and x(26) and x(28))
     or (a(14) and not(b(14)));
eq(28)<=x(30) and x(31);

-- expanders:
x(30)<=not(not(a(15)) and b(15));
x(31)<=not(a(15) and not(b(15)));

-- macrocell:
si(16)<=eq(29) xor eq(30);
eq(29)<=(not(a(15)) and not(b(15)) and si(15))
     or (a(15) and not(b(15)));
eq(30)<=(a(15) and b(15) and si(15)); 

end ma34x; -- lt_16
--************************************************

use work.cypress.all;
entity ge_16 is port(
  a,b:in bit_vector(15 downto 0);
  a_ge_b: out bit
  );
end ge_16;

architecture ma34x of ge_16 is 
 signal x:bit_vector(1 to 31);
 attribute synthesis_off of x:signal is true;
 signal si:bit_vector(16 downto 0);
 attribute synthesis_off of si:signal is true;
 signal eq:bit_vector(1 to 30);
 signal car:bit_vector(0 to 3);
 attribute synthesis_off of car:signal is true;
 signal em:bit_vector(0 to 7);

begin
a_ge_b<=si(16);

-- macrocell:
si(0)<=eq(2) xor eq(1);
eq(1)<=not(a(0)) or b(0);
eq(2)<=x(1);
   
-- expander:
x(1)<=not(not(a(0)) and b(0));

-- macrocell:
si(1)<=eq(4) xor eq(3);
eq(3)<=x(1);
eq(4)<=x(2) and x(3);
   
-- expanders:
x(2)<=not(not(a(1)) and b(1));
x(3)<=not(a(1) and not(b(1)));

-- macrocell:
si(2)<=eq(6) xor eq(5);
eq(5)<= (x(1) and x(2))
     or (not(b(1)) and a(1));
eq(6)<=x(4) and x(5);

-- expanders:
x(4)<=not(not(a(2)) and b(2));
x(5)<=not(a(2) and not(b(2)));

-- macrocell:
si(3)<=eq(8) xor eq(7);
eq(7)<=(x(1) and x(2) and x(4))
     or (not(b(1)) and a(1) and x(4))
     or (not(b(2)) and a(2));
eq(8)<=x(6) and x(7);

-- expanders:
x(6)<=not(not(a(3)) and b(3));
x(7)<=not(a(3) and not(b(3)));

-- carry macrocell zero:
car(0)<=em(1) xor em(0);
em(0)<=(x(1) and x(2) and x(4) and x(6) and x(7))
     or (not(b(1)) and a(1) and x(4) and x(6) and x(7))
     or (not(b(2)) and a(2) and x(6) and x(7));
em(1)<=a(3) and not(b(3));

-- macrocell:
si(4)<=eq(9) xor car(0);
eq(9)<=x(8) and x(9);

-- expanders:
x(8)<=not(not(a(4)) and b(4));
x(9)<=not(a(4) and not(b(4)));

-- macrocell:
si(5)<=eq(11) xor eq(10);
eq(10)<=(a(4) and not(b(4)))
      or (car(0) and x(8));
eq(11)<=x(10) and x(11);

-- expanders:
x(11)<=not(not(a(5)) and b(5));
x(10)<=not(a(5) and not(b(5)));

-- macrocell:
si(6)<=eq(13) xor eq(12);
eq(12)<=(a(4) and not(b(4)) and x(11))
      or (car(0) and x(8) and x(11))
      or (a(5) and not(b(5)));
eq(13)<=x(12) and x(13);

-- expanders:
x(12)<=not(not(a(6)) and b(6));
x(13)<=not(a(6) and not(b(6)));

-- carry macrocell one:
car(1)<=em(3) xor em(2);
em(2)<=(not(b(4)) and a(4) and x(11) and x(12) and x(13))
     or (car(0) and x(8) and x(11) and x(12) and x(13))
     or (not(b(5)) and a(5) and x(12) and x(13));
em(3)<=a(6) and not(b(6));

-- macrocell:
si(7)<=eq(14) xor car(1);
eq(14)<=x(14) and x(15);

-- expanders:
x(14)<=not(not(a(7)) and b(7));
x(15)<=not(a(7) and not(b(7)));

-- macrocell:
si(8)<=eq(16) xor eq(15);
eq(15)<=(a(7) and not(b(7)))
      or (car(1) and x(14));
eq(16)<=x(16) and x(17);

-- expanders:
x(16)<=not(not(a(8)) and b(8));
x(17)<=not(a(8) and not(b(8)));

-- macrocell:
si(9)<=eq(18) xor eq(17);
eq(17)<=(a(7) and not(b(7)) and x(16))
      or (car(1) and x(14) and x(16))
      or (a(8) and not(b(8)));
eq(18)<=x(18) and x(19);

-- expanders:
x(18)<=not(not(a(9)) and b(9));
x(19)<=not(a(9) and not(b(9)));

-- carry macrocell two:
car(2)<=em(5) xor em(4);
em(4)<=(not(b(7)) and a(7) and x(16) and x(18) and x(19))
     or (car(1) and x(14) and x(16) and x(18) and x(19))
     or (not(b(8)) and a(8) and x(18) and x(19));
em(5)<=a(9) and not(b(9));

-- macrocell:
si(10)<=car(2) xor eq(19);
eq(19)<=x(20) and x(21);

-- expanders:
x(20)<=not(a(10) and not(b(10)));
x(21)<=not(not(a(10)) and b(10));

-- macrocell:
si(11)<=eq(21) xor eq(20);
eq(20)<=(a(10) and not(b(10)))
     or (car(2) and x(21));
eq(21)<=x(22) and x(23);

-- expanders:
x(22)<=not(not(a(11)) and b(11));
x(23)<=not(a(11) and not(b(11)));

-- macrocell:
si(12)<=eq(23) xor eq(22);
eq(22)<=(a(10) and not(b(10)) and x(22))
     or (car(2) and x(21) and x(22))
     or (a(11) and not(b(11)));
eq(23)<=x(24) and x(25);

-- expanders:
x(24)<=not(not(a(12)) and b(12));
x(25)<=not(a(12) and not(b(12)));

-- carry macrocell three:
car(3)<=em(7) xor em(6);
em(6)<=(not(b(10)) and a(10) and x(22) and x(24) and x(25))
     or (car(2) and x(21) and x(22) and x(24) and x(25))
     or (not(b(11)) and a(11) and x(24) and x(25));
em(7)<=a(12) and not(b(12));

-- macrocell:
si(13)<=car(3) xor eq(24);
eq(24)<=x(26) and x(27);

-- expanders:
x(26)<=not(not(a(13)) and b(13));
x(27)<=not(a(13) and not(b(13)));

-- macrocell:
si(14)<=eq(26) xor eq(25);
eq(25)<=(a(13) and not(b(13)))
     or (car(3) and x(26));
eq(26)<=x(28) and x(29);

-- expanders:
x(28)<=not(not(a(14)) and b(14));
x(29)<=not(a(14) and not(b(14)));

-- macrocell:
si(15)<=eq(28) xor eq(27);
eq(27)<=(a(13) and not(b(13)) and x(28))
     or (car(3) and x(26) and x(28))
     or (a(14) and not(b(14)));
eq(28)<=x(30) and x(31);

-- expanders:
x(30)<=not(not(a(15)) and b(15));
x(31)<=not(a(15) and not(b(15)));

-- macrocell:
si(16)<=eq(29) xor eq(30);
eq(29)<=(not(a(15)) and b(15))
     or (not(si(15)) and b(15));
eq(30)<=( not(si(15)) and not(a(15)) and not(b(15)) );

end ma34x; -- ge_16
--************************************************

--************************************************
--************************************************

use work.cypress.all;
entity le_16 is port(
  a,b:in bit_vector(15 downto 0);
  a_le_b: out bit
  );
end le_16;


architecture ma34x of le_16 is 
 signal x:bit_vector(1 to 31);
 attribute synthesis_off of x:signal is true;
 signal si:bit_vector(16 downto 0);
 attribute synthesis_off of si:signal is true;
 signal eq:bit_vector(1 to 30);
 signal car:bit_vector(0 to 3);
 attribute synthesis_off of car:signal is true;
 signal em:bit_vector(0 to 7);

begin
a_le_b<=si(16);

-- macrocell:
si(0)<=eq(2) xor eq(1);
eq(1)<=b(0) and not(a(0)) and x(1);
eq(2)<=x(1);
   
-- expander:
x(1)<=not(not(a(0)) and b(0));

-- macrocell:
si(1)<=eq(4) xor eq(3);
eq(3)<=a(0) and not(b(0));
eq(4)<=x(2) and x(3);
   
-- expanders:
x(2)<=not(not(a(1)) and b(1));
x(3)<=not(a(1) and not(b(1)));

-- macrocell:
si(2)<=eq(6) xor eq(5);
eq(5)<= (not(b(0)) and a(0) and x(2))
     or (not(b(1)) and a(1));
eq(6)<=x(4) and x(5);

-- expanders:
x(4)<=not(not(a(2)) and b(2));
x(5)<=not(a(2) and not(b(2)));

-- macrocell:
si(3)<=eq(8) xor eq(7);
eq(7)<=(not(b(0)) and a(0) and x(2) and x(4))
     or (not(b(1)) and a(1) and x(4))
     or (not(b(2)) and a(2));
eq(8)<=x(6) and x(7);

-- expanders:
x(6)<=not(not(a(3)) and b(3));
x(7)<=not(a(3) and not(b(3)));

-- carry macrocell zero:
car(0)<=em(1) xor em(0);
em(0)<=(not(b(0)) and a(0) and x(2) and x(4) and x(6) and x(7))
     or (not(b(1)) and a(1) and x(4) and x(6) and x(7))
     or (not(b(2)) and a(2) and x(6) and x(7));
em(1)<=a(3) and not(b(3));

-- macrocell:
si(4)<=eq(9) xor car(0);
eq(9)<=x(8) and x(9);

-- expanders:
x(8)<=not(not(a(4)) and b(4));
x(9)<=not(a(4) and not(b(4)));

-- macrocell:
si(5)<=eq(11) xor eq(10);
eq(10)<=(a(4) and not(b(4)))
      or (car(0) and x(8));
eq(11)<=x(10) and x(11);

-- expanders:
x(11)<=not(not(a(5)) and b(5));
x(10)<=not(a(5) and not(b(5)));

-- macrocell:
si(6)<=eq(13) xor eq(12);
eq(12)<=(a(4) and not(b(4)) and x(11))
      or (car(0) and x(8) and x(11))
      or (a(5) and not(b(5)));
eq(13)<=x(12) and x(13);

-- expanders:
x(12)<=not(not(a(6)) and b(6));
x(13)<=not(a(6) and not(b(6)));

-- carry macrocell one:
car(1)<=em(3) xor em(2);
em(2)<=(not(b(4)) and a(4) and x(11) and x(12) and x(13))
     or (car(0) and x(8) and x(11) and x(12) and x(13))
     or (not(b(5)) and a(5) and x(12) and x(13));
em(3)<=a(6) and not(b(6));

-- macrocell:
si(7)<=eq(14) xor car(1);
eq(14)<=x(14) and x(15);

-- expanders:
x(14)<=not(not(a(7)) and b(7));
x(15)<=not(a(7) and not(b(7)));

-- macrocell:
si(8)<=eq(16) xor eq(15);
eq(15)<=(a(7) and not(b(7)))
      or (car(1) and x(14));
eq(16)<=x(16) and x(17);

-- expanders:
x(16)<=not(not(a(8)) and b(8));
x(17)<=not(a(8) and not(b(8)));

-- macrocell:
si(9)<=eq(18) xor eq(17);
eq(17)<=(a(7) and not(b(7)) and x(16))
      or (car(1) and x(14) and x(16))
      or (a(8) and not(b(8)));
eq(18)<=x(18) and x(19);

-- expanders:
x(18)<=not(not(a(9)) and b(9));
x(19)<=not(a(9) and not(b(9)));

-- carry macrocell two:
car(2)<=em(5) xor em(4);
em(4)<=(not(b(7)) and a(7) and x(16) and x(18) and x(19))
     or (car(1) and x(14) and x(16) and x(18) and x(19))
     or (not(b(8)) and a(8) and x(18) and x(19));
em(5)<=a(9) and not(b(9));

-- macrocell:
si(10)<=car(2) xor eq(19);
eq(19)<=x(20) and x(21);

-- expanders:
x(20)<=not(a(10) and not(b(10)));
x(21)<=not(not(a(10)) and b(10));

-- macrocell:
si(11)<=eq(21) xor eq(20);
eq(20)<=(a(10) and not(b(10)))
     or (car(2) and x(21));
eq(21)<=x(22) and x(23);

-- expanders:
x(22)<=not(not(a(11)) and b(11));
x(23)<=not(a(11) and not(b(11)));

-- macrocell:
si(12)<=eq(23) xor eq(22);
eq(22)<=(a(10) and not(b(10)) and x(22))
     or (car(2) and x(21) and x(22))
     or (a(11) and not(b(11)));
eq(23)<=x(24) and x(25);

-- expanders:
x(24)<=not(not(a(12)) and b(12));
x(25)<=not(a(12) and not(b(12)));

-- carry macrocell three:
car(3)<=em(7) xor em(6);
em(6)<=(not(b(10)) and a(10) and x(22) and x(24) and x(25))
     or (car(2) and x(21) and x(22) and x(24) and x(25))
     or (not(b(11)) and a(11) and x(24) and x(25));
em(7)<=a(12) and not(b(12));

-- macrocell:
si(13)<=car(3) xor eq(24);
eq(24)<=x(26) and x(27);

-- expanders:
x(26)<=not(not(a(13)) and b(13));
x(27)<=not(a(13) and not(b(13)));

-- macrocell:
si(14)<=eq(26) xor eq(25);
eq(25)<=(a(13) and not(b(13)))
     or (car(3) and x(26));
eq(26)<=x(28) and x(29);

-- expanders:
x(28)<=not(not(a(14)) and b(14));
x(29)<=not(a(14) and not(b(14)));

-- macrocell:
si(15)<=eq(28) xor eq(27);
eq(27)<=(a(13) and not(b(13)) and x(28))
     or (car(3) and x(26) and x(28))
     or (a(14) and not(b(14)));
eq(28)<=x(30) and x(31);

-- expanders:
x(30)<=not(not(a(15)) and b(15));
x(31)<=not(a(15) and not(b(15)));

-- macrocell:
si(16)<=eq(29) xor eq(30);
eq(29)<=(not(a(15)) and not(b(15)) and si(15))
     or (a(15) and not(b(15)));
eq(30)<=(a(15) and b(15) and si(15)); 

end ma34x; --le_16
--************************************************
