-- mth34x24.vhd
--
--
-- Copyright 1993,1994,1995,1996 Cypress Semiconductor as an unpublished work.
-- Rev 1.0	09-20-93
-- Rev 1.1	12-06-93
-- Rev 1.2  12-18-93
-- Rev 1.3  01-19-94
--
--
-- This is a package that defines math components of various sizes,
-- namely 8,12,16,24.
-- The components are defined and entity architecture pairs
-- follow, suitable for implementation in 34X devices (ie.
-- the architectures render operations in terms of expanders and
-- macrocells). The synthesis_off attribute is used and use of
-- the -xor2 synthesis directive is expected.
--
-- Components:
-- adder: add_24
-- subtractor: sub_24
-- grt than(>): gt_24
-- less than(<): lt_24
-- gt or eq(>=): ge_24
-- lt or eq(<=): le_24   
--
--
package math34x24_pkg is

component add_24 port(
  a,b:in bit_vector(23 downto 0);
  s:out bit_vector(23 downto 0)
  );
end component;
component sub_24 port(
  a,b:in bit_vector(23 downto 0);
  s:out bit_vector(23 downto 0)
  );
end component;
component gt_24 port(
  a,b:in bit_vector(23 downto 0);
  a_gt_b:out bit
  );
end component;
component lt_24 port(
  a,b:in bit_vector(23 downto 0);
  a_lt_b:out bit
  );
end component;
component ge_24 port(
  a,b:in bit_vector(23 downto 0);
  a_ge_b:out bit
  );
end component;
component le_24 port(
  a,b:in bit_vector(23 downto 0);
  a_le_b:out bit
  );
end component;

end math34x24_pkg;

-- ***************************************************
-- entities and architectures for math components
-- ***************************************************
use work.cypress.all;
entity add_24 is port(
  a,b:in bit_vector(23 downto 0);
  s:out bit_vector(23 downto 0)
  );
end add_24;
--***********************************************
architecture ma34x of add_24 is 
 signal x:bit_vector(1 to 47);
 attribute synthesis_off of x:signal is true;
 signal si:bit_vector(23 downto 0);
 attribute synthesis_off of si:signal is true;
 signal eq:bit_vector(1 to 41);
 signal car:bit_vector(0 to 6);
 attribute synthesis_off of car:signal is true;
 signal em:bit_vector(0 to 13);

begin
s<=si;

-- macrocell:
si(0)<=eq(2) xor eq(1);
eq(1)<=not(b(0)) and not(a(0)) and x(1);
eq(2)<=x(1);
   
-- expander:
x(1)<=not(a(0) and b(0));

-- macrocell:
si(1)<=eq(4) xor eq(3);
eq(3)<=x(2) and x(3);
eq(4)<=b(0) and a(0);
   
-- expanders:
x(2)<=not(not(a(1)) and not(b(1)));
x(3)<=not(a(1) and b(1));

-- macrocell:
si(2)<=eq(6) xor eq(5);
eq(5)<=(b(0) and a(0)and x(2))
     or (b(1) and a(1));
eq(6)<=x(4) and x(5);

-- expanders:
x(4)<=not(not(a(2)) and not(b(2)));
x(5)<=not(a(2) and b(2));

-- macrocell:
si(3)<=eq(8) xor eq(7);
eq(7)<=(b(0) and a(0) and x(2) and x(4))
     or (b(1) and a(1) and x(4))
     or (b(2) and a(2));
eq(8)<=x(6) and x(7);

-- expanders:
x(6)<=not(not(a(3)) and not(b(3)));
x(7)<=not(a(3) and b(3));

-- carry macrocell zero:
car(0)<=em(1) xor em(0);
em(0)<=(b(0) and a(0) and x(2) and x(4) and x(6) and x(7))
     or (b(1) and a(1) and x(4) and x(6) and x(7))
     or (b(2) and a(2) and x(6) and x(7));
em(1)<=a(3) and b(3);

-- macrocell:
si(4)<=eq(9) xor car(0);
eq(9)<=x(8) and x(9);

-- expanders:
x(8)<=not(not(a(4)) and not(b(4)));
x(9)<=not(a(4) and b(4));

-- macrocell:
si(5)<=eq(11) xor eq(10);
eq(10)<=(a(4) and b(4))
      or (car(0) and x(8));
eq(11)<=x(10) and x(11);

-- expanders:
x(11)<=not(not(a(5)) and not(b(5)));
x(10)<=not(a(5) and b(5));

-- macrocell:
si(6)<=eq(13) xor eq(12);
eq(12)<=(a(4) and b(4) and x(11))
      or (car(0) and x(8) and x(11))
      or (a(5) and b(5));
eq(13)<=x(12) and x(13);

-- expanders:
x(12)<=not(not(a(6)) and not(b(6)));
x(13)<=not(a(6) and b(6));

-- carry macrocell one:
car(1)<=em(3) xor em(2);
em(2)<=(b(4) and a(4) and x(11) and x(12) and x(13))
     or (car(0) and x(8) and x(11) and x(12) and x(13))
     or (b(5) and a(5) and x(12) and x(13));
em(3)<=a(6) and b(6);

-- macrocell:
si(7)<=eq(14) xor car(1);
eq(14)<=x(14) and x(15);

-- expanders:
x(14)<=not(not(a(7)) and not(b(7)));
x(15)<=not(a(7) and b(7));

-- macrocell:
si(8)<=eq(16) xor eq(15);
eq(15)<=(a(7) and b(7))
      or (car(1) and x(14));
eq(16)<=x(16) and x(17);

-- expanders:
x(16)<=not(not(a(8)) and not(b(8)));
x(17)<=not(a(8) and b(8));

-- macrocell:
si(9)<=eq(18) xor eq(17);
eq(17)<=(a(7) and b(7) and x(16))
      or (car(1) and x(14) and x(16))
      or (a(8) and b(8));
eq(18)<=x(18) and x(19);

-- expanders:
x(18)<=not(not(a(9)) and not(b(9)));
x(19)<=not(a(9) and b(9));

-- carry macrocell two:
car(2)<=em(5) xor em(4);
em(4)<=(b(7) and a(7) and x(16) and x(18) and x(19))
     or (car(1) and x(14) and x(16) and x(18) and x(19))
     or (b(8) and a(8) and x(18) and x(19));
em(5)<=a(9) and b(9);

-- macrocell:
si(10)<=car(2) xor eq(19);
eq(19)<=x(20) and x(21);

-- expanders:
x(20)<=not(a(10) and b(10));
x(21)<=not(not(a(10)) and not(b(10)));

-- macrocell:
si(11)<=eq(21) xor eq(20);
eq(20)<=(a(10) and b(10))
     or (car(2) and x(21));
eq(21)<=x(22) and x(23);

-- expanders:
x(22)<=not(not(a(11)) and not(b(11)));
x(23)<=not(a(11) and b(11));

-- macrocell:
si(12)<=eq(23) xor eq(22);
eq(22)<=(a(10) and b(10) and x(22))
     or (car(2) and x(21) and x(22))
     or (a(11) and b(11));
eq(23)<=x(24) and x(25);

-- expanders:
x(24)<=not(not(a(12)) and not(b(12)));
x(25)<=not(a(12) and b(12));

-- carry macrocell three:
car(3)<=em(7) xor em(6);
em(6)<=(b(10) and a(10) and x(22) and x(24) and x(25))
     or (car(2) and x(21) and x(22) and x(24) and x(25))
     or (b(11) and a(11) and x(24) and x(25));
em(7)<=a(12) and b(12);

-- macrocell:
si(13)<=car(3) xor eq(24);
eq(24)<=x(26) and x(27);

-- expanders:
x(26)<=not(a(13) and b(13));
x(27)<=not(not(a(13)) and not(b(13)));

-- macrocell:
si(14)<=eq(26) xor eq(25);
eq(25)<=(a(13) and b(13))
     or (car(3) and x(27));
eq(26)<=x(28) and x(29);

-- expanders:
x(28)<=not(not(a(14)) and not(b(14)));
x(29)<=not(a(14) and b(14));

-- macrocell:
si(15)<=eq(28) xor eq(27);
eq(27)<=(a(13) and b(13) and x(28))
     or (car(3) and x(27) and x(28))
     or (a(14) and b(14));
eq(28)<=x(30) and x(31);

-- expanders:
x(30)<=not(not(a(15)) and not(b(15)));
x(31)<=not(a(15) and b(15));

-- carry macrocell four:
car(4)<=em(9) xor em(8);
em(9)<=(b(13) and a(13) and x(28) and x(30) and x(31))
     or (car(3) and x(27) and x(28) and x(30) and x(31))
     or (b(14) and a(14) and x(30) and x(31));
em(8)<=a(15) and b(15);

-- macrocell:
si(16)<=car(4) xor eq(29);
eq(29)<=x(32) and x(33);

-- expanders:
x(32)<=not(a(16) and b(16));
x(33)<=not(not(a(16)) and not(b(16)));

-- macrocell:
si(17)<=eq(30) xor eq(31);
eq(30)<=(a(16) and b(16))
     or (car(4) and x(33));
eq(31)<=x(34) and x(35);

-- expanders:
x(34)<=not(not(a(17)) and not(b(17)));
x(35)<=not(a(17) and b(17));

-- macrocell:
si(18)<=eq(32) xor eq(33);
eq(32)<=(a(16) and b(16) and x(34))
     or (car(4) and x(33) and x(34))
     or (a(17) and b(17));
eq(33)<=x(36) and x(37);

-- expanders:
x(36)<=not(not(a(18)) and not(b(18)));
x(37)<=not(a(18) and b(18));

-- carry macrocell five:
car(5)<=em(11) xor em(10);
em(11)<=(b(16) and a(16) and x(34) and x(36) and x(37))
     or (car(4) and x(33) and x(34) and x(36) and x(37))
     or (b(17) and a(17) and x(36) and x(37));
em(10)<=a(18) and b(18);

-- macrocell:
si(19)<=car(5) xor eq(34);
eq(34)<=x(38) and x(39);

-- expanders:
x(38)<=not(a(19) and b(19));
x(39)<=not(not(a(19)) and not(b(19)));

-- macrocell:
si(20)<=eq(35) xor eq(36);
eq(35)<=(a(19) and b(19))
     or (car(5) and x(39));
eq(36)<=x(40) and x(41);

-- expanders:
x(40)<=not(not(a(20)) and not(b(20)));
x(41)<=not(a(20) and b(20));

-- macrocell:
si(21)<=eq(37) xor eq(38);
eq(37)<=(a(19) and b(19) and x(40))
     or (car(5) and x(39) and x(40))
     or (a(20) and b(20));
eq(38)<=x(42) and x(43);

-- expanders:
x(42)<=not(not(a(21)) and not(b(21)));
x(43)<=not(a(21) and b(21));

-- carry macrocell six:
car(6)<=em(13) xor em(12);
em(13)<=(b(19) and a(19) and x(40) and x(42) and x(43))
     or (car(5) and x(39) and x(40) and x(42) and x(43))
     or (b(20) and a(20) and x(42) and x(43));
em(12)<=a(21) and b(21);

-- macrocell:
si(22)<=car(6) xor eq(39);
eq(39)<=x(44) and x(45);

-- expanders:
x(44)<=not(a(22) and b(22));
x(45)<=not(not(a(22)) and not(b(22)));

-- macrocell:
si(23)<=eq(40) xor eq(41);
eq(40)<=(a(22) and b(22))
     or (car(6) and x(45));
eq(41)<=x(46) and x(47);

-- expanders:
x(46)<=not(not(a(23)) and not(b(23)));
x(47)<=not(a(23) and b(23));

end ma34x;
--***********************************************
 
--***********************************************
--   sub_n component ****************************
--***********************************************
use work.cypress.all;
entity sub_24 is port(
  a,b:in bit_vector(23 downto 0);
  s:out bit_vector(23 downto 0)
  );
end sub_24;


architecture ma34x of sub_24 is 
 signal x:bit_vector(1 to 47);
 attribute synthesis_off of x:signal is true;
 signal si:bit_vector(23 downto 0);
 attribute synthesis_off of si:signal is true;
 signal eq:bit_vector(1 to 41);
 signal car:bit_vector(0 to 6);
 attribute synthesis_off of car:signal is true;
 signal em:bit_vector(0 to 13);

begin
s<=si;

-- macrocell:
si(0)<=eq(2) xor eq(1);
eq(1)<=not(a(0)) or b(0);
eq(2)<=x(1);
   
-- expander:
x(1)<=not(not(a(0)) and b(0));

-- macrocell:
si(1)<=eq(4) xor eq(3);
eq(3)<=x(1);
eq(4)<=x(2) and x(3);
   
-- expanders:
x(2)<=not(not(a(1)) and b(1));
x(3)<=not(a(1) and not(b(1)));

-- macrocell:
si(2)<=eq(6) xor eq(5);
eq(5)<= (x(1) and x(2))
     or (not(b(1)) and a(1));
eq(6)<=x(4) and x(5);

-- expanders:
x(4)<=not(not(a(2)) and b(2));
x(5)<=not(a(2) and not(b(2)));

-- macrocell:
si(3)<=eq(8) xor eq(7);
eq(7)<=(x(1) and x(2) and x(4))
     or (not(b(1)) and a(1) and x(4))
     or (not(b(2)) and a(2));
eq(8)<=x(6) and x(7);

-- expanders:
x(6)<=not(not(a(3)) and b(3));
x(7)<=not(a(3) and not(b(3)));

-- carry macrocell zero:
car(0)<=em(1) xor em(0);
em(0)<=(x(1) and x(2) and x(4) and x(6) and x(7))
     or (not(b(1)) and a(1) and x(4) and x(6) and x(7))
     or (not(b(2)) and a(2) and x(6) and x(7));
em(1)<=a(3) and not(b(3));

-- macrocell:
si(4)<=eq(9) xor car(0);
eq(9)<=x(8) and x(9);

-- expanders:
x(8)<=not(not(a(4)) and b(4));
x(9)<=not(a(4) and not(b(4)));

-- macrocell:
si(5)<=eq(11) xor eq(10);
eq(10)<=(a(4) and not(b(4)))
      or (car(0) and x(8));
eq(11)<=x(10) and x(11);

-- expanders:
x(11)<=not(not(a(5)) and b(5));
x(10)<=not(a(5) and not(b(5)));

-- macrocell:
si(6)<=eq(13) xor eq(12);
eq(12)<=(a(4) and not(b(4)) and x(11))
      or (car(0) and x(8) and x(11))
      or (a(5) and not(b(5)));
eq(13)<=x(12) and x(13);

-- expanders:
x(12)<=not(not(a(6)) and b(6));
x(13)<=not(a(6) and not(b(6)));

-- carry macrocell one:
car(1)<=em(3) xor em(2);
em(2)<=(not(b(4)) and a(4) and x(11) and x(12) and x(13))
     or (car(0) and x(8) and x(11) and x(12) and x(13))
     or (not(b(5)) and a(5) and x(12) and x(13));
em(3)<=a(6) and not(b(6));

-- macrocell:
si(7)<=eq(14) xor car(1);
eq(14)<=x(14) and x(15);

-- expanders:
x(14)<=not(not(a(7)) and b(7));
x(15)<=not(a(7) and not(b(7)));

-- macrocell:
si(8)<=eq(16) xor eq(15);
eq(15)<=(a(7) and not(b(7)))
      or (car(1) and x(14));
eq(16)<=x(16) and x(17);

-- expanders:
x(16)<=not(not(a(8)) and b(8));
x(17)<=not(a(8) and not(b(8)));

-- macrocell:
si(9)<=eq(18) xor eq(17);
eq(17)<=(a(7) and not(b(7)) and x(16))
      or (car(1) and x(14) and x(16))
      or (a(8) and not(b(8)));
eq(18)<=x(18) and x(19);

-- expanders:
x(18)<=not(not(a(9)) and b(9));
x(19)<=not(a(9) and not(b(9)));

-- carry macrocell two:
car(2)<=em(5) xor em(4);
em(4)<=(not(b(7)) and a(7) and x(16) and x(18) and x(19))
     or (car(1) and x(14) and x(16) and x(18) and x(19))
     or (not(b(8)) and a(8) and x(18) and x(19));
em(5)<=a(9) and not(b(9));

-- macrocell:
si(10)<=car(2) xor eq(19);
eq(19)<=x(20) and x(21);

-- expanders:
x(20)<=not(a(10) and not(b(10)));
x(21)<=not(not(a(10)) and b(10));

-- macrocell:
si(11)<=eq(21) xor eq(20);
eq(20)<=(a(10) and not(b(10)))
     or (car(2) and x(21));
eq(21)<=x(22) and x(23);

-- expanders:
x(22)<=not(not(a(11)) and b(11));
x(23)<=not(a(11) and not(b(11)));

-- macrocell:
si(12)<=eq(23) xor eq(22);
eq(22)<=(a(10) and not(b(10)) and x(22))
     or (car(2) and x(21) and x(22))
     or (a(11) and not(b(11)));
eq(23)<=x(24) and x(25);

-- expanders:
x(24)<=not(not(a(12)) and b(12));
x(25)<=not(a(12) and not(b(12)));

-- carry macrocell three:
car(3)<=em(7) xor em(6);
em(6)<=(not(b(10)) and a(10) and x(22) and x(24) and x(25))
     or (car(2) and x(21) and x(22) and x(24) and x(25))
     or (not(b(11)) and a(11) and x(24) and x(25));
em(7)<=a(12) and not(b(12));

-- macrocell:
si(13)<=car(3) xor eq(24);
eq(24)<=x(26) and x(27);

-- expanders:
x(26)<=not(not(a(13)) and b(13));
x(27)<=not(a(13) and not(b(13)));

-- macrocell:
si(14)<=eq(26) xor eq(25);
eq(25)<=(a(13) and not(b(13)))
     or (car(3) and x(26));
eq(26)<=x(28) and x(29);

-- expanders:
x(28)<=not(not(a(14)) and b(14));
x(29)<=not(a(14) and not(b(14)));

-- macrocell:
si(15)<=eq(28) xor eq(27);
eq(27)<=(a(13) and not(b(13)) and x(28))
     or (car(3) and x(26) and x(28))
     or (a(14) and not(b(14)));
eq(28)<=x(30) and x(31);

-- expanders:
x(30)<=not(not(a(15)) and b(15));
x(31)<=not(a(15) and not(b(15)));

-- carry macrocell four:
car(4)<=em(9) xor em(8);
em(9)<=(not(b(13)) and a(13) and x(28) and x(30) and x(31))
     or (car(3) and x(26) and x(28) and x(30) and x(31))
     or (not(b(14)) and a(14) and x(30) and x(31));
em(8)<=a(15) and not(b(15));

-- macrocell:
si(16)<=car(4) xor eq(29);
eq(29)<=x(32) and x(33);

-- expanders:
x(32)<=not(a(16) and not(b(16)));
x(33)<=not(not(a(16)) and b(16));

-- macrocell:
si(17)<=eq(30) xor eq(31);
eq(30)<=(a(16) and not(b(16)))
     or (car(4) and x(33));
eq(31)<=x(34) and x(35);

-- expanders:
x(34)<=not(not(a(17)) and b(17));
x(35)<=not(a(17) and not(b(17)));

-- macrocell:
si(18)<=eq(32) xor eq(33);
eq(32)<=(a(16) and not(b(16)) and x(34))
     or (car(4) and x(33) and x(34))
     or (a(17) and not(b(17)));
eq(33)<=x(36) and x(37);

-- expanders:
x(36)<=not(not(a(18)) and b(18));
x(37)<=not(a(18) and not(b(18)));

-- carry macrocell five:
car(5)<=em(11) xor em(10);
em(11)<=(not(b(16)) and a(16) and x(34) and x(36) and x(37))
     or (car(4) and x(33) and x(34) and x(36) and x(37))
     or (not(b(17)) and a(17) and x(36) and x(37));
em(10)<=a(18) and not(b(18));

-- macrocell:
si(19)<=car(5) xor eq(34);
eq(34)<=x(38) and x(39);

-- expanders:
x(38)<=not(a(19) and not(b(19)));
x(39)<=not(not(a(19)) and b(19));

-- macrocell:
si(20)<=eq(35) xor eq(36);
eq(35)<=(a(19) and not(b(19)))
     or (car(5) and x(39));
eq(36)<=x(40) and x(41);

-- expanders:
x(40)<=not(not(a(20)) and b(20));
x(41)<=not(a(20) and not(b(20)));

-- macrocell:
si(21)<=eq(37) xor eq(38);
eq(37)<=(a(19) and not(b(19)) and x(40))
     or (car(5) and x(39) and x(40))
     or (a(20) and not(b(20)));
eq(38)<=x(42) and x(43);

-- expanders:
x(42)<=not(not(a(21)) and b(21));
x(43)<=not(a(21) and not(b(21)));

-- carry macrocell six:
car(6)<=em(13) xor em(12);
em(13)<=(not(b(19)) and a(19) and x(40) and x(42) and x(43))
     or (car(5) and x(39) and x(40) and x(42) and x(43))
     or (not(b(20)) and a(20) and x(42) and x(43));
em(12)<=a(21) and not(b(21));

-- macrocell:
si(22)<=car(6) xor eq(39);
eq(39)<=x(44) and x(45);

-- expanders:
x(44)<=not(a(22) and not(b(22)));
x(45)<=not(not(a(22)) and b(22));

-- macrocell:
si(23)<=eq(40) xor eq(41);
eq(40)<=(a(22) and not(b(22)))
     or (car(6) and x(45));
eq(41)<=x(46) and x(47);

-- expanders:
x(46)<=not(not(a(23)) and b(23));
x(47)<=not(a(23) and not(b(23)));

end ma34x; -- sub_24
--************************************************

use work.cypress.all;
entity gt_24 is port(
  a,b:in bit_vector(23 downto 0);
  a_gt_b: out bit
  );
end gt_24;

architecture ma34x of gt_24 is 
 signal x:bit_vector(1 to 47);
 attribute synthesis_off of x:signal is true;
 signal si:bit_vector(24 downto 0);
 attribute synthesis_off of si:signal is true;
 signal eq:bit_vector(1 to 43);
 signal car:bit_vector(0 to 6);
 attribute synthesis_off of car:signal is true;
 signal em:bit_vector(0 to 13);

begin
a_gt_b<=si(24);

-- macrocell:
si(0)<=eq(2) xor eq(1);
eq(1)<=b(0) and not(a(0)) and x(1);
eq(2)<=x(1);
   
-- expander:
x(1)<=not(not(a(0)) and b(0));

-- macrocell:
si(1)<=eq(4) xor eq(3);
eq(3)<=a(0) and not(b(0));
eq(4)<=x(2) and x(3);
   
-- expanders:
x(2)<=not(not(a(1)) and b(1));
x(3)<=not(a(1) and not(b(1)));

-- macrocell:
si(2)<=eq(6) xor eq(5);
eq(5)<= (not(b(0)) and a(0) and x(2))
     or (not(b(1)) and a(1));
eq(6)<=x(4) and x(5);

-- expanders:
x(4)<=not(not(a(2)) and b(2));
x(5)<=not(a(2) and not(b(2)));

-- macrocell:
si(3)<=eq(8) xor eq(7);
eq(7)<=(not(b(0)) and a(0) and x(2) and x(4))
     or (not(b(1)) and a(1) and x(4))
     or (not(b(2)) and a(2));
eq(8)<=x(6) and x(7);

-- expanders:
x(6)<=not(not(a(3)) and b(3));
x(7)<=not(a(3) and not(b(3)));

-- carry macrocell zero:
car(0)<=em(1) xor em(0);
em(0)<=(not(b(0)) and a(0) and x(2) and x(4) and x(6) and x(7))
     or (not(b(1)) and a(1) and x(4) and x(6) and x(7))
     or (not(b(2)) and a(2) and x(6) and x(7));
em(1)<=a(3) and not(b(3));

-- macrocell:
si(4)<=eq(9) xor car(0);
eq(9)<=x(8) and x(9);

-- expanders:
x(8)<=not(not(a(4)) and b(4));
x(9)<=not(a(4) and not(b(4)));

-- macrocell:
si(5)<=eq(11) xor eq(10);
eq(10)<=(a(4) and not(b(4)))
      or (car(0) and x(8));
eq(11)<=x(10) and x(11);

-- expanders:
x(11)<=not(not(a(5)) and b(5));
x(10)<=not(a(5) and not(b(5)));

-- macrocell:
si(6)<=eq(13) xor eq(12);
eq(12)<=(a(4) and not(b(4)) and x(11))
      or (car(0) and x(8) and x(11))
      or (a(5) and not(b(5)));
eq(13)<=x(12) and x(13);

-- expanders:
x(12)<=not(not(a(6)) and b(6));
x(13)<=not(a(6) and not(b(6)));

-- carry macrocell one:
car(1)<=em(3) xor em(2);
em(2)<=(not(b(4)) and a(4) and x(11) and x(12) and x(13))
     or (car(0) and x(8) and x(11) and x(12) and x(13))
     or (not(b(5)) and a(5) and x(12) and x(13));
em(3)<=a(6) and not(b(6));

-- macrocell:
si(7)<=eq(14) xor car(1);
eq(14)<=x(14) and x(15);

-- expanders:
x(14)<=not(not(a(7)) and b(7));
x(15)<=not(a(7) and not(b(7)));

-- macrocell:
si(8)<=eq(16) xor eq(15);
eq(15)<=(a(7) and not(b(7)))
      or (car(1) and x(14));
eq(16)<=x(16) and x(17);

-- expanders:
x(16)<=not(not(a(8)) and b(8));
x(17)<=not(a(8) and not(b(8)));

-- macrocell:
si(9)<=eq(18) xor eq(17);
eq(17)<=(a(7) and not(b(7)) and x(16))
      or (car(1) and x(14) and x(16))
      or (a(8) and not(b(8)));
eq(18)<=x(18) and x(19);

-- expanders:
x(18)<=not(not(a(9)) and b(9));
x(19)<=not(a(9) and not(b(9)));

-- carry macrocell two:
car(2)<=em(5) xor em(4);
em(4)<=(not(b(7)) and a(7) and x(16) and x(18) and x(19))
     or (car(1) and x(14) and x(16) and x(18) and x(19))
     or (not(b(8)) and a(8) and x(18) and x(19));
em(5)<=a(9) and not(b(9));

-- macrocell:
si(10)<=car(2) xor eq(19);
eq(19)<=x(20) and x(21);

-- expanders:
x(20)<=not(a(10) and not(b(10)));
x(21)<=not(not(a(10)) and b(10));

-- macrocell:
si(11)<=eq(21) xor eq(20);
eq(20)<=(a(10) and not(b(10)))
     or (car(2) and x(21));
eq(21)<=x(22) and x(23);

-- expanders:
x(22)<=not(not(a(11)) and b(11));
x(23)<=not(a(11) and not(b(11)));

-- macrocell:
si(12)<=eq(23) xor eq(22);
eq(22)<=(a(10) and not(b(10)) and x(22))
     or (car(2) and x(21) and x(22))
     or (a(11) and not(b(11)));
eq(23)<=x(24) and x(25);

-- expanders:
x(24)<=not(not(a(12)) and b(12));
x(25)<=not(a(12) and not(b(12)));

-- carry macrocell three:
car(3)<=em(7) xor em(6);
em(6)<=(not(b(10)) and a(10) and x(22) and x(24) and x(25))
     or (car(2) and x(21) and x(22) and x(24) and x(25))
     or (not(b(11)) and a(11) and x(24) and x(25));
em(7)<=a(12) and not(b(12));

-- macrocell:
si(13)<=car(3) xor eq(24);
eq(24)<=x(26) and x(27);

-- expanders:
x(26)<=not(not(a(13)) and b(13));
x(27)<=not(a(13) and not(b(13)));

-- macrocell:
si(14)<=eq(26) xor eq(25);
eq(25)<=(a(13) and not(b(13)))
     or (car(3) and x(26));
eq(26)<=x(28) and x(29);

-- expanders:
x(28)<=not(not(a(14)) and b(14));
x(29)<=not(a(14) and not(b(14)));

-- macrocell:
si(15)<=eq(28) xor eq(27);
eq(27)<=(a(13) and not(b(13)) and x(28))
     or (car(3) and x(26) and x(28))
     or (a(14) and not(b(14)));
eq(28)<=x(30) and x(31);

-- expanders:
x(30)<=not(not(a(15)) and b(15));
x(31)<=not(a(15) and not(b(15)));

-- carry macrocell four:
car(4)<=em(9) xor em(8);
em(9)<=(not(b(13)) and a(13) and x(28) and x(30) and x(31))
     or (car(3) and x(26) and x(28) and x(30) and x(31))
     or (not(b(14)) and a(14) and x(30) and x(31));
em(8)<=a(15) and not(b(15));

-- macrocell:
si(16)<=car(4) xor eq(29);
eq(29)<=x(32) and x(33);

-- expanders:
x(32)<=not(a(16) and not(b(16)));
x(33)<=not(not(a(16)) and b(16));

-- macrocell:
si(17)<=eq(30) xor eq(31);
eq(30)<=(a(16) and not(b(16)))
     or (car(4) and x(33));
eq(31)<=x(34) and x(35);

-- expanders:
x(34)<=not(not(a(17)) and b(17));
x(35)<=not(a(17) and not(b(17)));

-- macrocell:
si(18)<=eq(32) xor eq(33);
eq(32)<=(a(16) and not(b(16)) and x(34))
     or (car(4) and x(33) and x(34))
     or (a(17) and not(b(17)));
eq(33)<=x(36) and x(37);

-- expanders:
x(36)<=not(not(a(18)) and b(18));
x(37)<=not(a(18) and not(b(18)));

-- carry macrocell five:
car(5)<=em(11) xor em(10);
em(11)<=(not(b(16)) and a(16) and x(34) and x(36) and x(37))
     or (car(4) and x(33) and x(34) and x(36) and x(37))
     or (not(b(17)) and a(17) and x(36) and x(37));
em(10)<=a(18) and not(b(18));

-- macrocell:
si(19)<=car(5) xor eq(34);
eq(34)<=x(38) and x(39);

-- expanders:
x(38)<=not(a(19) and not(b(19)));
x(39)<=not(not(a(19)) and b(19));

-- macrocell:
si(20)<=eq(35) xor eq(36);
eq(35)<=(a(19) and not(b(19)))
     or (car(5) and x(39));
eq(36)<=x(40) and x(41);

-- expanders:
x(40)<=not(not(a(20)) and b(20));
x(41)<=not(a(20) and not(b(20)));

-- macrocell:
si(21)<=eq(37) xor eq(38);
eq(37)<=(a(19) and not(b(19)) and x(40))
     or (car(5) and x(39) and x(40))
     or (a(20) and not(b(20)));
eq(38)<=x(42) and x(43);

-- expanders:
x(42)<=not(not(a(21)) and b(21));
x(43)<=not(a(21) and not(b(21)));

-- carry macrocell six:
car(6)<=em(13) xor em(12);
em(13)<=(not(b(19)) and a(19) and x(40) and x(42) and x(43))
     or (car(5) and x(39) and x(40) and x(42) and x(43))
     or (not(b(20)) and a(20) and x(42) and x(43));
em(12)<=a(21) and not(b(21));

-- macrocell:
si(22)<=car(6) xor eq(39);
eq(39)<=x(44) and x(45);

-- expanders:
x(44)<=not(a(22) and not(b(22)));
x(45)<=not(not(a(22)) and b(22));

-- macrocell:
si(23)<=eq(40) xor eq(41);
eq(40)<=(a(22) and not(b(22)))
     or (car(6) and x(45));
eq(41)<=x(46) and x(47);

-- expanders:
x(46)<=not(not(a(23)) and b(23));
x(47)<=not(a(23) and not(b(23)));

-- macrocell:
si(24)<=eq(42) xor eq(43);
eq(42)<=(not(a(23)) and b(23))
     or (not(si(23)) and b(23));
eq(43)<=( not(si(23)) and not(b(23)) and not(a(23)) ) ;

end ma34x; -- gt_24
--************************************************

use work.cypress.all;
entity lt_24 is port(
  a,b:in bit_vector(23 downto 0);
  a_lt_b: out bit
  );
end lt_24;


architecture ma34x of lt_24 is 
 signal x:bit_vector(1 to 47);
 attribute synthesis_off of x:signal is true;
 signal si:bit_vector(24 downto 0);
 attribute synthesis_off of si:signal is true;
 signal eq:bit_vector(1 to 43);
 signal car:bit_vector(0 to 6);
 attribute synthesis_off of car:signal is true;
 signal em:bit_vector(0 to 13);

begin
a_lt_b<=si(24);

-- macrocell:
si(0)<=eq(2) xor eq(1);
eq(1)<=not(a(0)) or b(0);
eq(2)<=x(1);
   
-- expander:
x(1)<=not(not(a(0)) and b(0));

-- macrocell:
si(1)<=eq(4) xor eq(3);
eq(3)<=x(1);
eq(4)<=x(2) and x(3);
   
-- expanders:
x(2)<=not(not(a(1)) and b(1));
x(3)<=not(a(1) and not(b(1)));

-- macrocell:
si(2)<=eq(6) xor eq(5);
eq(5)<= (x(1) and x(2))
     or (not(b(1)) and a(1));
eq(6)<=x(4) and x(5);

-- expanders:
x(4)<=not(not(a(2)) and b(2));
x(5)<=not(a(2) and not(b(2)));

-- macrocell:
si(3)<=eq(8) xor eq(7);
eq(7)<=(x(1) and x(2) and x(4))
     or (not(b(1)) and a(1) and x(4))
     or (not(b(2)) and a(2));
eq(8)<=x(6) and x(7);

-- expanders:
x(6)<=not(not(a(3)) and b(3));
x(7)<=not(a(3) and not(b(3)));

-- carry macrocell zero:
car(0)<=em(1) xor em(0);
em(0)<=(x(1) and x(2) and x(4) and x(6) and x(7))
     or (not(b(1)) and a(1) and x(4) and x(6) and x(7))
     or (not(b(2)) and a(2) and x(6) and x(7));
em(1)<=a(3) and not(b(3));

-- macrocell:
si(4)<=eq(9) xor car(0);
eq(9)<=x(8) and x(9);

-- expanders:
x(8)<=not(not(a(4)) and b(4));
x(9)<=not(a(4) and not(b(4)));

-- macrocell:
si(5)<=eq(11) xor eq(10);
eq(10)<=(a(4) and not(b(4)))
      or (car(0) and x(8));
eq(11)<=x(10) and x(11);

-- expanders:
x(11)<=not(not(a(5)) and b(5));
x(10)<=not(a(5) and not(b(5)));

-- macrocell:
si(6)<=eq(13) xor eq(12);
eq(12)<=(a(4) and not(b(4)) and x(11))
      or (car(0) and x(8) and x(11))
      or (a(5) and not(b(5)));
eq(13)<=x(12) and x(13);

-- expanders:
x(12)<=not(not(a(6)) and b(6));
x(13)<=not(a(6) and not(b(6)));

-- carry macrocell one:
car(1)<=em(3) xor em(2);
em(2)<=(not(b(4)) and a(4) and x(11) and x(12) and x(13))
     or (car(0) and x(8) and x(11) and x(12) and x(13))
     or (not(b(5)) and a(5) and x(12) and x(13));
em(3)<=a(6) and not(b(6));

-- macrocell:
si(7)<=eq(14) xor car(1);
eq(14)<=x(14) and x(15);

-- expanders:
x(14)<=not(not(a(7)) and b(7));
x(15)<=not(a(7) and not(b(7)));

-- macrocell:
si(8)<=eq(16) xor eq(15);
eq(15)<=(a(7) and not(b(7)))
      or (car(1) and x(14));
eq(16)<=x(16) and x(17);

-- expanders:
x(16)<=not(not(a(8)) and b(8));
x(17)<=not(a(8) and not(b(8)));

-- macrocell:
si(9)<=eq(18) xor eq(17);
eq(17)<=(a(7) and not(b(7)) and x(16))
      or (car(1) and x(14) and x(16))
      or (a(8) and not(b(8)));
eq(18)<=x(18) and x(19);

-- expanders:
x(18)<=not(not(a(9)) and b(9));
x(19)<=not(a(9) and not(b(9)));

-- carry macrocell two:
car(2)<=em(5) xor em(4);
em(4)<=(not(b(7)) and a(7) and x(16) and x(18) and x(19))
     or (car(1) and x(14) and x(16) and x(18) and x(19))
     or (not(b(8)) and a(8) and x(18) and x(19));
em(5)<=a(9) and not(b(9));

-- macrocell:
si(10)<=car(2) xor eq(19);
eq(19)<=x(20) and x(21);

-- expanders:
x(20)<=not(a(10) and not(b(10)));
x(21)<=not(not(a(10)) and b(10));

-- macrocell:
si(11)<=eq(21) xor eq(20);
eq(20)<=(a(10) and not(b(10)))
     or (car(2) and x(21));
eq(21)<=x(22) and x(23);

-- expanders:
x(22)<=not(not(a(11)) and b(11));
x(23)<=not(a(11) and not(b(11)));

-- macrocell:
si(12)<=eq(23) xor eq(22);
eq(22)<=(a(10) and not(b(10)) and x(22))
     or (car(2) and x(21) and x(22))
     or (a(11) and not(b(11)));
eq(23)<=x(24) and x(25);

-- expanders:
x(24)<=not(not(a(12)) and b(12));
x(25)<=not(a(12) and not(b(12)));

-- carry macrocell three:
car(3)<=em(7) xor em(6);
em(6)<=(not(b(10)) and a(10) and x(22) and x(24) and x(25))
     or (car(2) and x(21) and x(22) and x(24) and x(25))
     or (not(b(11)) and a(11) and x(24) and x(25));
em(7)<=a(12) and not(b(12));

-- macrocell:
si(13)<=car(3) xor eq(24);
eq(24)<=x(26) and x(27);

-- expanders:
x(26)<=not(not(a(13)) and b(13));
x(27)<=not(a(13) and not(b(13)));

-- macrocell:
si(14)<=eq(26) xor eq(25);
eq(25)<=(a(13) and not(b(13)))
     or (car(3) and x(26));
eq(26)<=x(28) and x(29);

-- expanders:
x(28)<=not(not(a(14)) and b(14));
x(29)<=not(a(14) and not(b(14)));

-- macrocell:
si(15)<=eq(28) xor eq(27);
eq(27)<=(a(13) and not(b(13)) and x(28))
     or (car(3) and x(26) and x(28))
     or (a(14) and not(b(14)));
eq(28)<=x(30) and x(31);

-- expanders:
x(30)<=not(not(a(15)) and b(15));
x(31)<=not(a(15) and not(b(15)));

-- carry macrocell four:
car(4)<=em(9) xor em(8);
em(9)<=(not(b(13)) and a(13) and x(28) and x(30) and x(31))
     or (car(3) and x(26) and x(28) and x(30) and x(31))
     or (not(b(14)) and a(14) and x(30) and x(31));
em(8)<=a(15) and not(b(15));

-- macrocell:
si(16)<=car(4) xor eq(29);
eq(29)<=x(32) and x(33);

-- expanders:
x(32)<=not(a(16) and not(b(16)));
x(33)<=not(not(a(16)) and b(16));

-- macrocell:
si(17)<=eq(30) xor eq(31);
eq(30)<=(a(16) and not(b(16)))
     or (car(4) and x(33));
eq(31)<=x(34) and x(35);

-- expanders:
x(34)<=not(not(a(17)) and b(17));
x(35)<=not(a(17) and not(b(17)));

-- macrocell:
si(18)<=eq(32) xor eq(33);
eq(32)<=(a(16) and not(b(16)) and x(34))
     or (car(4) and x(33) and x(34))
     or (a(17) and not(b(17)));
eq(33)<=x(36) and x(37);

-- expanders:
x(36)<=not(not(a(18)) and b(18));
x(37)<=not(a(18) and not(b(18)));

-- carry macrocell five:
car(5)<=em(11) xor em(10);
em(11)<=(not(b(16)) and a(16) and x(34) and x(36) and x(37))
     or (car(4) and x(33) and x(34) and x(36) and x(37))
     or (not(b(17)) and a(17) and x(36) and x(37));
em(10)<=a(18) and not(b(18));

-- macrocell:
si(19)<=car(5) xor eq(34);
eq(34)<=x(38) and x(39);

-- expanders:
x(38)<=not(a(19) and not(b(19)));
x(39)<=not(not(a(19)) and b(19));

-- macrocell:
si(20)<=eq(35) xor eq(36);
eq(35)<=(a(19) and not(b(19)))
     or (car(5) and x(39));
eq(36)<=x(40) and x(41);

-- expanders:
x(40)<=not(not(a(20)) and b(20));
x(41)<=not(a(20) and not(b(20)));

-- macrocell:
si(21)<=eq(37) xor eq(38);
eq(37)<=(a(19) and not(b(19)) and x(40))
     or (car(5) and x(39) and x(40))
     or (a(20) and not(b(20)));
eq(38)<=x(42) and x(43);

-- expanders:
x(42)<=not(not(a(21)) and b(21));
x(43)<=not(a(21) and not(b(21)));

-- carry macrocell six:
car(6)<=em(13) xor em(12);
em(13)<=(not(b(19)) and a(19) and x(40) and x(42) and x(43))
     or (car(5) and x(39) and x(40) and x(42) and x(43))
     or (not(b(20)) and a(20) and x(42) and x(43));
em(12)<=a(21) and not(b(21));

-- macrocell:
si(22)<=car(6) xor eq(39);
eq(39)<=x(44) and x(45);

-- expanders:
x(44)<=not(a(22) and not(b(22)));
x(45)<=not(not(a(22)) and b(22));

-- macrocell:
si(23)<=eq(40) xor eq(41);
eq(40)<=(a(22) and not(b(22)))
     or (car(6) and x(45));
eq(41)<=x(46) and x(47);

-- expanders:
x(46)<=not(not(a(23)) and b(23));
x(47)<=not(a(23) and not(b(23)));

-- macrocell:
si(24)<=eq(42) xor eq(43);
eq(42)<=(a(22) and not(b(22)) and x(46))
     or (car(6) and x(45) and x(46))
     or (a(23) and not(b(23)));
eq(43)<=x(46) and x(47); -- sign extend...

end ma34x; -- lt_24
--************************************************

use work.cypress.all;
entity ge_24 is port(
  a,b:in bit_vector(23 downto 0);
  a_ge_b: out bit
  );
end ge_24;


architecture ma34x of ge_24 is 
 signal x:bit_vector(1 to 47);
 attribute synthesis_off of x:signal is true;
 signal si:bit_vector(24 downto 0);
 attribute synthesis_off of si:signal is true;
 signal eq:bit_vector(1 to 43);
 signal car:bit_vector(0 to 6);
 attribute synthesis_off of car:signal is true;
 signal em:bit_vector(0 to 13);

begin
a_ge_b<=si(24);

-- macrocell:
si(0)<=eq(2) xor eq(1);
eq(1)<=not(a(0)) or b(0);
eq(2)<=x(1);
   
-- expander:
x(1)<=not(not(a(0)) and b(0));

-- macrocell:
si(1)<=eq(4) xor eq(3);
eq(3)<=x(1);
eq(4)<=x(2) and x(3);
   
-- expanders:
x(2)<=not(not(a(1)) and b(1));
x(3)<=not(a(1) and not(b(1)));

-- macrocell:
si(2)<=eq(6) xor eq(5);
eq(5)<= (x(1) and x(2))
     or (not(b(1)) and a(1));
eq(6)<=x(4) and x(5);

-- expanders:
x(4)<=not(not(a(2)) and b(2));
x(5)<=not(a(2) and not(b(2)));

-- macrocell:
si(3)<=eq(8) xor eq(7);
eq(7)<=(x(1) and x(2) and x(4))
     or (not(b(1)) and a(1) and x(4))
     or (not(b(2)) and a(2));
eq(8)<=x(6) and x(7);

-- expanders:
x(6)<=not(not(a(3)) and b(3));
x(7)<=not(a(3) and not(b(3)));

-- carry macrocell zero:
car(0)<=em(1) xor em(0);
em(0)<=(x(1) and x(2) and x(4) and x(6) and x(7))
     or (not(b(1)) and a(1) and x(4) and x(6) and x(7))
     or (not(b(2)) and a(2) and x(6) and x(7));
em(1)<=a(3) and not(b(3));

-- macrocell:
si(4)<=eq(9) xor car(0);
eq(9)<=x(8) and x(9);

-- expanders:
x(8)<=not(not(a(4)) and b(4));
x(9)<=not(a(4) and not(b(4)));

-- macrocell:
si(5)<=eq(11) xor eq(10);
eq(10)<=(a(4) and not(b(4)))
      or (car(0) and x(8));
eq(11)<=x(10) and x(11);

-- expanders:
x(11)<=not(not(a(5)) and b(5));
x(10)<=not(a(5) and not(b(5)));

-- macrocell:
si(6)<=eq(13) xor eq(12);
eq(12)<=(a(4) and not(b(4)) and x(11))
      or (car(0) and x(8) and x(11))
      or (a(5) and not(b(5)));
eq(13)<=x(12) and x(13);

-- expanders:
x(12)<=not(not(a(6)) and b(6));
x(13)<=not(a(6) and not(b(6)));

-- carry macrocell one:
car(1)<=em(3) xor em(2);
em(2)<=(not(b(4)) and a(4) and x(11) and x(12) and x(13))
     or (car(0) and x(8) and x(11) and x(12) and x(13))
     or (not(b(5)) and a(5) and x(12) and x(13));
em(3)<=a(6) and not(b(6));

-- macrocell:
si(7)<=eq(14) xor car(1);
eq(14)<=x(14) and x(15);

-- expanders:
x(14)<=not(not(a(7)) and b(7));
x(15)<=not(a(7) and not(b(7)));

-- macrocell:
si(8)<=eq(16) xor eq(15);
eq(15)<=(a(7) and not(b(7)))
      or (car(1) and x(14));
eq(16)<=x(16) and x(17);

-- expanders:
x(16)<=not(not(a(8)) and b(8));
x(17)<=not(a(8) and not(b(8)));

-- macrocell:
si(9)<=eq(18) xor eq(17);
eq(17)<=(a(7) and not(b(7)) and x(16))
      or (car(1) and x(14) and x(16))
      or (a(8) and not(b(8)));
eq(18)<=x(18) and x(19);

-- expanders:
x(18)<=not(not(a(9)) and b(9));
x(19)<=not(a(9) and not(b(9)));

-- carry macrocell two:
car(2)<=em(5) xor em(4);
em(4)<=(not(b(7)) and a(7) and x(16) and x(18) and x(19))
     or (car(1) and x(14) and x(16) and x(18) and x(19))
     or (not(b(8)) and a(8) and x(18) and x(19));
em(5)<=a(9) and not(b(9));

-- macrocell:
si(10)<=car(2) xor eq(19);
eq(19)<=x(20) and x(21);

-- expanders:
x(20)<=not(a(10) and not(b(10)));
x(21)<=not(not(a(10)) and b(10));

-- macrocell:
si(11)<=eq(21) xor eq(20);
eq(20)<=(a(10) and not(b(10)))
     or (car(2) and x(21));
eq(21)<=x(22) and x(23);

-- expanders:
x(22)<=not(not(a(11)) and b(11));
x(23)<=not(a(11) and not(b(11)));

-- macrocell:
si(12)<=eq(23) xor eq(22);
eq(22)<=(a(10) and not(b(10)) and x(22))
     or (car(2) and x(21) and x(22))
     or (a(11) and not(b(11)));
eq(23)<=x(24) and x(25);

-- expanders:
x(24)<=not(not(a(12)) and b(12));
x(25)<=not(a(12) and not(b(12)));

-- carry macrocell three:
car(3)<=em(7) xor em(6);
em(6)<=(not(b(10)) and a(10) and x(22) and x(24) and x(25))
     or (car(2) and x(21) and x(22) and x(24) and x(25))
     or (not(b(11)) and a(11) and x(24) and x(25));
em(7)<=a(12) and not(b(12));

-- macrocell:
si(13)<=car(3) xor eq(24);
eq(24)<=x(26) and x(27);

-- expanders:
x(26)<=not(not(a(13)) and b(13));
x(27)<=not(a(13) and not(b(13)));

-- macrocell:
si(14)<=eq(26) xor eq(25);
eq(25)<=(a(13) and not(b(13)))
     or (car(3) and x(26));
eq(26)<=x(28) and x(29);

-- expanders:
x(28)<=not(not(a(14)) and b(14));
x(29)<=not(a(14) and not(b(14)));

-- macrocell:
si(15)<=eq(28) xor eq(27);
eq(27)<=(a(13) and not(b(13)) and x(28))
     or (car(3) and x(26) and x(28))
     or (a(14) and not(b(14)));
eq(28)<=x(30) and x(31);

-- expanders:
x(30)<=not(not(a(15)) and b(15));
x(31)<=not(a(15) and not(b(15)));

-- carry macrocell four:
car(4)<=em(9) xor em(8);
em(9)<=(not(b(13)) and a(13) and x(28) and x(30) and x(31))
     or (car(3) and x(26) and x(28) and x(30) and x(31))
     or (not(b(14)) and a(14) and x(30) and x(31));
em(8)<=a(15) and not(b(15));

-- macrocell:
si(16)<=car(4) xor eq(29);
eq(29)<=x(32) and x(33);

-- expanders:
x(32)<=not(a(16) and not(b(16)));
x(33)<=not(not(a(16)) and b(16));

-- macrocell:
si(17)<=eq(30) xor eq(31);
eq(30)<=(a(16) and not(b(16)))
     or (car(4) and x(33));
eq(31)<=x(34) and x(35);

-- expanders:
x(34)<=not(not(a(17)) and b(17));
x(35)<=not(a(17) and not(b(17)));

-- macrocell:
si(18)<=eq(32) xor eq(33);
eq(32)<=(a(16) and not(b(16)) and x(34))
     or (car(4) and x(33) and x(34))
     or (a(17) and not(b(17)));
eq(33)<=x(36) and x(37);

-- expanders:
x(36)<=not(not(a(18)) and b(18));
x(37)<=not(a(18) and not(b(18)));

-- carry macrocell five:
car(5)<=em(11) xor em(10);
em(11)<=(not(b(16)) and a(16) and x(34) and x(36) and x(37))
     or (car(4) and x(33) and x(34) and x(36) and x(37))
     or (not(b(17)) and a(17) and x(36) and x(37));
em(10)<=a(18) and not(b(18));

-- macrocell:
si(19)<=car(5) xor eq(34);
eq(34)<=x(38) and x(39);

-- expanders:
x(38)<=not(a(19) and not(b(19)));
x(39)<=not(not(a(19)) and b(19));

-- macrocell:
si(20)<=eq(35) xor eq(36);
eq(35)<=(a(19) and not(b(19)))
     or (car(5) and x(39));
eq(36)<=x(40) and x(41);

-- expanders:
x(40)<=not(not(a(20)) and b(20));
x(41)<=not(a(20) and not(b(20)));

-- macrocell:
si(21)<=eq(37) xor eq(38);
eq(37)<=(a(19) and not(b(19)) and x(40))
     or (car(5) and x(39) and x(40))
     or (a(20) and not(b(20)));
eq(38)<=x(42) and x(43);

-- expanders:
x(42)<=not(not(a(21)) and b(21));
x(43)<=not(a(21) and not(b(21)));

-- carry macrocell six:
car(6)<=em(13) xor em(12);
em(13)<=(not(b(19)) and a(19) and x(40) and x(42) and x(43))
     or (car(5) and x(39) and x(40) and x(42) and x(43))
     or (not(b(20)) and a(20) and x(42) and x(43));
em(12)<=a(21) and not(b(21));

-- macrocell:
si(22)<=car(6) xor eq(39);
eq(39)<=x(44) and x(45);

-- expanders:
x(44)<=not(a(22) and not(b(22)));
x(45)<=not(not(a(22)) and b(22));

-- macrocell:
si(23)<=eq(40) xor eq(41);
eq(40)<=(a(22) and not(b(22)))
     or (car(6) and x(45));
eq(41)<=x(46) and x(47);

-- expanders:
x(46)<=not(not(a(23)) and b(23));
x(47)<=not(a(23) and not(b(23)));

-- macrocell:
si(24)<=eq(42) xor eq(43);
eq(42)<=(not(a(23)) and b(23))
     or (not(si(23)) and b(23));
eq(43)<=( not(si(23)) and not(b(23)) and not(a(23)) ) ;

end ma34x; -- ge_24
--************************************************


use work.cypress.all;
entity le_24 is port(
  a,b:in bit_vector(23 downto 0);
  a_le_b: out bit
  );
end le_24;

architecture ma34x of le_24 is 
 signal x:bit_vector(1 to 47);
 attribute synthesis_off of x:signal is true;
 signal si:bit_vector(24 downto 0);
 attribute synthesis_off of si:signal is true;
 signal eq:bit_vector(1 to 43);
 signal car:bit_vector(0 to 6);
 attribute synthesis_off of car:signal is true;
 signal em:bit_vector(0 to 13);

begin
a_le_b<=si(24);

-- macrocell:
si(0)<=eq(2) xor eq(1);
eq(1)<=b(0) and not(a(0)) and x(1);
eq(2)<=x(1);
   
-- expander:
x(1)<=not(not(a(0)) and b(0));

-- macrocell:
si(1)<=eq(4) xor eq(3);
eq(3)<=a(0) and not(b(0));
eq(4)<=x(2) and x(3);
   
-- expanders:
x(2)<=not(not(a(1)) and b(1));
x(3)<=not(a(1) and not(b(1)));

-- macrocell:
si(2)<=eq(6) xor eq(5);
eq(5)<= (not(b(0)) and a(0) and x(2))
     or (not(b(1)) and a(1));
eq(6)<=x(4) and x(5);

-- expanders:
x(4)<=not(not(a(2)) and b(2));
x(5)<=not(a(2) and not(b(2)));

-- macrocell:
si(3)<=eq(8) xor eq(7);
eq(7)<=(not(b(0)) and a(0) and x(2) and x(4))
     or (not(b(1)) and a(1) and x(4))
     or (not(b(2)) and a(2));
eq(8)<=x(6) and x(7);

-- expanders:
x(6)<=not(not(a(3)) and b(3));
x(7)<=not(a(3) and not(b(3)));

-- carry macrocell zero:
car(0)<=em(1) xor em(0);
em(0)<=(not(b(0)) and a(0) and x(2) and x(4) and x(6) and x(7))
     or (not(b(1)) and a(1) and x(4) and x(6) and x(7))
     or (not(b(2)) and a(2) and x(6) and x(7));
em(1)<=a(3) and not(b(3));

-- macrocell:
si(4)<=eq(9) xor car(0);
eq(9)<=x(8) and x(9);

-- expanders:
x(8)<=not(not(a(4)) and b(4));
x(9)<=not(a(4) and not(b(4)));

-- macrocell:
si(5)<=eq(11) xor eq(10);
eq(10)<=(a(4) and not(b(4)))
      or (car(0) and x(8));
eq(11)<=x(10) and x(11);

-- expanders:
x(11)<=not(not(a(5)) and b(5));
x(10)<=not(a(5) and not(b(5)));

-- macrocell:
si(6)<=eq(13) xor eq(12);
eq(12)<=(a(4) and not(b(4)) and x(11))
      or (car(0) and x(8) and x(11))
      or (a(5) and not(b(5)));
eq(13)<=x(12) and x(13);

-- expanders:
x(12)<=not(not(a(6)) and b(6));
x(13)<=not(a(6) and not(b(6)));

-- carry macrocell one:
car(1)<=em(3) xor em(2);
em(2)<=(not(b(4)) and a(4) and x(11) and x(12) and x(13))
     or (car(0) and x(8) and x(11) and x(12) and x(13))
     or (not(b(5)) and a(5) and x(12) and x(13));
em(3)<=a(6) and not(b(6));

-- macrocell:
si(7)<=eq(14) xor car(1);
eq(14)<=x(14) and x(15);

-- expanders:
x(14)<=not(not(a(7)) and b(7));
x(15)<=not(a(7) and not(b(7)));

-- macrocell:
si(8)<=eq(16) xor eq(15);
eq(15)<=(a(7) and not(b(7)))
      or (car(1) and x(14));
eq(16)<=x(16) and x(17);

-- expanders:
x(16)<=not(not(a(8)) and b(8));
x(17)<=not(a(8) and not(b(8)));

-- macrocell:
si(9)<=eq(18) xor eq(17);
eq(17)<=(a(7) and not(b(7)) and x(16))
      or (car(1) and x(14) and x(16))
      or (a(8) and not(b(8)));
eq(18)<=x(18) and x(19);

-- expanders:
x(18)<=not(not(a(9)) and b(9));
x(19)<=not(a(9) and not(b(9)));

-- carry macrocell two:
car(2)<=em(5) xor em(4);
em(4)<=(not(b(7)) and a(7) and x(16) and x(18) and x(19))
     or (car(1) and x(14) and x(16) and x(18) and x(19))
     or (not(b(8)) and a(8) and x(18) and x(19));
em(5)<=a(9) and not(b(9));

-- macrocell:
si(10)<=car(2) xor eq(19);
eq(19)<=x(20) and x(21);

-- expanders:
x(20)<=not(a(10) and not(b(10)));
x(21)<=not(not(a(10)) and b(10));

-- macrocell:
si(11)<=eq(21) xor eq(20);
eq(20)<=(a(10) and not(b(10)))
     or (car(2) and x(21));
eq(21)<=x(22) and x(23);

-- expanders:
x(22)<=not(not(a(11)) and b(11));
x(23)<=not(a(11) and not(b(11)));

-- macrocell:
si(12)<=eq(23) xor eq(22);
eq(22)<=(a(10) and not(b(10)) and x(22))
     or (car(2) and x(21) and x(22))
     or (a(11) and not(b(11)));
eq(23)<=x(24) and x(25);

-- expanders:
x(24)<=not(not(a(12)) and b(12));
x(25)<=not(a(12) and not(b(12)));

-- carry macrocell three:
car(3)<=em(7) xor em(6);
em(6)<=(not(b(10)) and a(10) and x(22) and x(24) and x(25))
     or (car(2) and x(21) and x(22) and x(24) and x(25))
     or (not(b(11)) and a(11) and x(24) and x(25));
em(7)<=a(12) and not(b(12));

-- macrocell:
si(13)<=car(3) xor eq(24);
eq(24)<=x(26) and x(27);

-- expanders:
x(26)<=not(not(a(13)) and b(13));
x(27)<=not(a(13) and not(b(13)));

-- macrocell:
si(14)<=eq(26) xor eq(25);
eq(25)<=(a(13) and not(b(13)))
     or (car(3) and x(26));
eq(26)<=x(28) and x(29);

-- expanders:
x(28)<=not(not(a(14)) and b(14));
x(29)<=not(a(14) and not(b(14)));

-- macrocell:
si(15)<=eq(28) xor eq(27);
eq(27)<=(a(13) and not(b(13)) and x(28))
     or (car(3) and x(26) and x(28))
     or (a(14) and not(b(14)));
eq(28)<=x(30) and x(31);

-- expanders:
x(30)<=not(not(a(15)) and b(15));
x(31)<=not(a(15) and not(b(15)));

-- carry macrocell four:
car(4)<=em(9) xor em(8);
em(9)<=(not(b(13)) and a(13) and x(28) and x(30) and x(31))
     or (car(3) and x(26) and x(28) and x(30) and x(31))
     or (not(b(14)) and a(14) and x(30) and x(31));
em(8)<=a(15) and not(b(15));

-- macrocell:
si(16)<=car(4) xor eq(29);
eq(29)<=x(32) and x(33);

-- expanders:
x(32)<=not(a(16) and not(b(16)));
x(33)<=not(not(a(16)) and b(16));

-- macrocell:
si(17)<=eq(30) xor eq(31);
eq(30)<=(a(16) and not(b(16)))
     or (car(4) and x(33));
eq(31)<=x(34) and x(35);

-- expanders:
x(34)<=not(not(a(17)) and b(17));
x(35)<=not(a(17) and not(b(17)));

-- macrocell:
si(18)<=eq(32) xor eq(33);
eq(32)<=(a(16) and not(b(16)) and x(34))
     or (car(4) and x(33) and x(34))
     or (a(17) and not(b(17)));
eq(33)<=x(36) and x(37);

-- expanders:
x(36)<=not(not(a(18)) and b(18));
x(37)<=not(a(18) and not(b(18)));

-- carry macrocell five:
car(5)<=em(11) xor em(10);
em(11)<=(not(b(16)) and a(16) and x(34) and x(36) and x(37))
     or (car(4) and x(33) and x(34) and x(36) and x(37))
     or (not(b(17)) and a(17) and x(36) and x(37));
em(10)<=a(18) and not(b(18));

-- macrocell:
si(19)<=car(5) xor eq(34);
eq(34)<=x(38) and x(39);

-- expanders:
x(38)<=not(a(19) and not(b(19)));
x(39)<=not(not(a(19)) and b(19));

-- macrocell:
si(20)<=eq(35) xor eq(36);
eq(35)<=(a(19) and not(b(19)))
     or (car(5) and x(39));
eq(36)<=x(40) and x(41);

-- expanders:
x(40)<=not(not(a(20)) and b(20));
x(41)<=not(a(20) and not(b(20)));

-- macrocell:
si(21)<=eq(37) xor eq(38);
eq(37)<=(a(19) and not(b(19)) and x(40))
     or (car(5) and x(39) and x(40))
     or (a(20) and not(b(20)));
eq(38)<=x(42) and x(43);

-- expanders:
x(42)<=not(not(a(21)) and b(21));
x(43)<=not(a(21) and not(b(21)));

-- carry macrocell six:
car(6)<=em(13) xor em(12);
em(13)<=(not(b(19)) and a(19) and x(40) and x(42) and x(43))
     or (car(5) and x(39) and x(40) and x(42) and x(43))
     or (not(b(20)) and a(20) and x(42) and x(43));
em(12)<=a(21) and not(b(21));

-- macrocell:
si(22)<=car(6) xor eq(39);
eq(39)<=x(44) and x(45);

-- expanders:
x(44)<=not(a(22) and not(b(22)));
x(45)<=not(not(a(22)) and b(22));

-- macrocell:
si(23)<=eq(40) xor eq(41);
eq(40)<=(a(22) and not(b(22)))
     or (car(6) and x(45));
eq(41)<=x(46) and x(47);

-- expanders:
x(46)<=not(not(a(23)) and b(23));
x(47)<=not(a(23) and not(b(23)));

-- macrocell:
si(24)<=eq(42) xor eq(43);
eq(42)<=(a(22) and not(b(22)) and x(46))
     or (car(6) and x(45) and x(46))
     or (a(23) and not(b(23)));
eq(43)<=x(46) and x(47); -- sign extend...

end ma34x; -- le_24
--************************************************


