--
--	Copyright Cypress Semiconductor Corporation, 1994,1995,1996
--	  as an unpublished work.
--

PACKAGE memorypkg IS

--------------------------------------------------------------------------
--		Components for Memory RTL.				--
--------------------------------------------------------------------------

    COMPONENT xdsrff		-- X type flip flop. (X1,X2,S,R and clk input)
	PORT (
	    x1	: IN BIT;		-- First Exclusive OR input.
	    x2	: IN BIT;		-- Second Exclusive OR input.
	    s	: IN BIT;		-- Asyncronous set.
	    r	: IN BIT;		-- Asyncronous reset.
	    clk	: IN BIT;		-- Positive edge Clock.
	    q	: OUT BIT		-- Qutput from register.
	);
    END COMPONENT;

    COMPONENT jksrff		-- JK type flip flop. ( J, K and clk input)
	PORT (
	    j	: IN BIT;		-- J input to J-K register.
	    k	: IN BIT;		-- K input to J-K register.
	    s	: IN BIT;		-- Asyncronous set.
	    r	: IN BIT;		-- Asyncronous reset.
	    clk	: IN BIT;		-- Positive edge Clock.
	    q	: OUT BIT		-- Qutput from register.
	);
    END COMPONENT;

    COMPONENT tsrff		-- T type flip flop with
				--  asynchronous set and reset.
	PORT (
	    t	: IN BIT;		-- T input to register.
	    s	: IN BIT;		-- Set input Async.
	    r	: IN BIT;		-- Reset input Async.
	    clk	: IN BIT;		-- Positive edge Clock.
	    q	: OUT BIT		-- Output from register.
	);
    END COMPONENT;

    component DSRLCH		-- Latch with Set and Reset
	port(
	     D  : IN bit;		-- Data input.
	     S  : IN bit;		-- Set input.
	     R  : IN bit;		-- Reset input.
	     E  : IN bit;		-- Enable input.
	     Q  : OUT bit);		-- Output from logic.
    end component;

END memorypkg;

package memorypkg_local is
COMPONENT PAlcell
	PORT (
	    qs	: IN bit;	-- Async Set on FF(Q).

	    a1	: IN bit;	-- Non-invert to AND A.
	    a2	: IN bit;	-- Invert to AND A.
	    a3	: IN bit;	-- Non-invert to AND A.
	    a4	: IN bit;	-- Invert to AND A.
	    a5	: IN bit;	-- Non-invert to AND A.
	    a6	: IN bit;	-- Invert to AND A.

	    b1	: IN bit;	-- Non-invert to AND B.
	    b2	: IN bit;	-- Invert to AND B.

	    c1	: IN bit;	-- Non-invert to AND C.
	    c2	: IN bit;	-- Invert to AND C.

	    d1	: IN bit;	-- Non-invert to AND D.
	    d2	: IN bit;	-- Invert to AND D.

	    e1	: IN bit;	-- Non-invert to AND E.
	    e2	: IN bit;	-- Invert to AND E.

	    f1	: IN bit;	-- Non-invert to AND F.
	    f2	: IN bit;	-- Invert to AND F.
	    f3	: IN bit;	-- Non-invert to AND F.
	    f4	: IN bit;	-- Invert to AND F.
	    f5	: IN bit;	-- Non-invert to AND F.
	    f6	: IN bit;	-- Invert to AND F.

	    qc	: IN bit;	-- Clock to FF(Q).
	    qr	: IN bit;	-- Async Reset on FF(Q).

	    az	: OUT bit;	-- Output of AND A.
	    oz	: OUT bit;	-- Output of MUX O.
	    qz	: OUT bit;	-- Output of FF  Q.
	    nz	: OUT bit;	-- Output of MUX N.
	    fz	: OUT bit	-- Output of AND F.
	);
    END COMPONENT;
    COMPONENT logico		-- Master Logic Cell.
	PORT (
	    a1	: IN bit;	-- Non-invert to AND A.
	    a2	: IN bit;	-- Invert to AND A.
	    a3	: IN bit;	-- Non-invert to AND A.
	    a4	: IN bit;	-- Invert to AND A.
	    a5	: IN bit;	-- Non-invert to AND A.
	    a6	: IN bit;	-- Invert to AND A.

	    b1	: IN bit;	-- Non-invert to AND B.
	    b2	: IN bit;	-- Invert to AND B.

	    c1	: IN bit;	-- Non-invert to AND C.
	    c2	: IN bit;	-- Invert to AND C.

	    d1	: IN bit;	-- Non-invert to AND D.
	    d2	: IN bit;	-- Invert to AND D.

	    e1	: IN bit;	-- Non-invert to AND E.
	    e2	: IN bit;	-- Invert to AND E.

	    f1	: IN bit;	-- Non-invert to AND F.
	    f2	: IN bit;	-- Invert to AND F.
	    f3	: IN bit;	-- Non-invert to AND F.
	    f4	: IN bit;	-- Invert to AND F.
	    f5	: IN bit;	-- Non-invert to AND F.
	    f6	: IN bit;	-- Invert to AND F.

	    oz	: OUT bit	-- Output of MUX O.
	);
    END COMPONENT;
end memorypkg_local ;
--------------------------------------------------------------------------
--		Enties for Memory RTL.					--
--------------------------------------------------------------------------

USE WORK.cypress.ALL;
USE WORK.memorypkg_local.ALL;
ENTITY xdsrff IS		-- X type flip flop. (X1,X2,S,R and clk input)
    PORT (
	x1	: IN BIT;		-- First Exclusive OR input.
	x2	: IN BIT;		-- Second Exclusive OR input.
	s	: IN BIT;		-- Asyncronous set.
	r	: IN BIT;		-- Asyncronous reset.
	clk	: IN BIT;		-- Positive edge Clock.
	q	: OUT BIT		-- Qutput from register.
    );
END xdsrff;

USE WORK.cypress.ALL;
USE WORK.memorypkg_local.ALL;
ENTITY jksrff IS		-- JKSR type flip flop. ( J,K,S,R,clk input)
    PORT (
	j	: IN BIT;		-- J input to J-K register.
	k	: IN BIT;		-- K input to J-K register.
	s	: IN BIT;		-- Asyncronous set.
	r	: IN BIT;		-- Asyncronous reset.
	clk	: IN BIT;		-- Positive edge Clock.
	q	: OUT BIT		-- Qutput from register.
    );
END jksrff;

USE WORK.cypress.ALL;
USE WORK.memorypkg_local.ALL;
ENTITY tsrff IS			-- T type flip flop. (T, S, R,  and clk input)
    PORT (
	t	: IN BIT;		-- Toggle input to T type register.
	s	: IN BIT;		-- Asyncronous set.
	r	: IN BIT;		-- Asyncronous reset.
	clk	: IN BIT;		-- Positive edge Clock.
	q	: OUT BIT		-- Output from register.
    );
END tsrff;

use work.cypress.all; 
use work.memorypkg_local.all; 

entity DSRLCH is
    port(
        D  : IN bit;
        S  : IN bit;
	R  : IN bit;
        E  : IN bit;
	Q  : OUT bit);
end DSRLCH;

--------------------------------------------------------------------------
--		Architectures for Generic RTL				--
--------------------------------------------------------------------------

ARCHITECTURE archxdsrff OF xdsrff IS

BEGIN

    U1:palcell port map(s,
			x1,zero,one,zero,one,zero,
			zero,one,one,zero,one,zero,zero,one,
			x2,zero,one,zero,one,zero,
			clk,r,open,open,q,open,open);

END archxdsrff;

ARCHITECTURE archtsrff OF tsrff IS

    signal qtmp : bit;

BEGIN

    U1:palcell port map(s,
			t,zero,one,zero,one,zero,
			zero,one,one,zero,one,zero,zero,one,
			qtmp,zero,one,zero,one,zero,
			clk,r,open,open,qtmp,open,open);

    q <= qtmp;

END archtsrff;

ARCHITECTURE archjksrff OF jksrff IS

    signal qtmp : bit;

BEGIN

    U1:palcell port map(s,
			j,zero,one,zero,one,zero,
			qtmp,zero,zero,one,one,zero,one,qtmp,
			k,zero,one,zero,one,zero,
			clk,r,open,open,qtmp,open,open);
    q <= qtmp;

END archjksrff;

architecture archDSRLCH of DSRLCH is

    signal qtmp : bit;
    attribute dont_touch of U1 : label is true;

begin

    U1:logico port map (one,s,one,zero,one,zero,
			one,r,one,r,qtmp,r,d,r,
			e,zero,one,zero,one,zero,qtmp);

    q <= qtmp;

end archDSRLCH;
