--
--      Copyright Cypress Semiconductor Corporation, 1994,1995,1996
--        as an unpublished work.
--

PACKAGE registerpkg IS 

component REG
    port(D  : IN bit;
         EN : IN bit;
         CLK: IN bit;
         R  : IN bit;
	 Q  : INOUT bit);
end component;

COMPONENT reg4
    PORT(D3 : IN bit;
         D2 : IN bit;
         D1 : IN bit;
         D0 : IN bit;
         EN : IN bit;
	 CLK: IN bit;
         R  : IN bit;
         Q3 : INOUT bit;
         Q2 : INOUT bit;
         Q1 : INOUT bit;
         Q0 : INOUT bit);
END COMPONENT;

COMPONENT reg8
    PORT(D7,D6,D5,D4,D3,D2,D1,D0: IN BIT;
	 EN: IN BIT;
	 CLK: IN BIT;
	 R: IN BIT;
	 Q7,Q6,Q5,Q4,Q3,Q2,Q1,Q0: INOUT BIT);
END COMPONENT;

COMPONENT reg16
    PORT(D15,D14,D13,D12,D11,D10,D9,D8: IN BIT;
	 D7,D6,D5,D4,D3,D2,D1,D0: IN BIT;
	 EN: IN BIT;
	 CLK: IN BIT;
	 R: IN BIT;
	 Q15,Q14,Q13,Q12,Q11,Q10,Q9,Q8: INOUT BIT;
	 Q7,Q6,Q5,Q4,Q3,Q2,Q1,Q0: INOUT BIT);
END COMPONENT;

END registerpkg;
------------------------------------------------
use work.cypress.all; 

entity REG is
    port(D  : IN bit;
         EN : IN bit;
         CLK: IN bit;
         R  : IN bit;
	 Q  : INOUT bit);
end REG;

architecture archREG of REG is
COMPONENT PAlcell
	PORT (
	    qs	: IN bit;	-- Async Set on FF(Q).

	    a1	: IN bit;	-- Non-invert to AND A.
	    a2	: IN bit;	-- Invert to AND A.
	    a3	: IN bit;	-- Non-invert to AND A.
	    a4	: IN bit;	-- Invert to AND A.
	    a5	: IN bit;	-- Non-invert to AND A.
	    a6	: IN bit;	-- Invert to AND A.

	    b1	: IN bit;	-- Non-invert to AND B.
	    b2	: IN bit;	-- Invert to AND B.

	    c1	: IN bit;	-- Non-invert to AND C.
	    c2	: IN bit;	-- Invert to AND C.

	    d1	: IN bit;	-- Non-invert to AND D.
	    d2	: IN bit;	-- Invert to AND D.

	    e1	: IN bit;	-- Non-invert to AND E.
	    e2	: IN bit;	-- Invert to AND E.

	    f1	: IN bit;	-- Non-invert to AND F.
	    f2	: IN bit;	-- Invert to AND F.
	    f3	: IN bit;	-- Non-invert to AND F.
	    f4	: IN bit;	-- Invert to AND F.
	    f5	: IN bit;	-- Non-invert to AND F.
	    f6	: IN bit;	-- Invert to AND F.

	    qc	: IN bit;	-- Clock to FF(Q).
	    qr	: IN bit;	-- Async Reset on FF(Q).

	    az	: OUT bit;	-- Output of AND A.
	    oz	: OUT bit;	-- Output of MUX O.
	    qz	: OUT bit;	-- Output of FF  Q.
	    nz	: OUT bit;	-- Output of MUX N.
	    fz	: OUT bit	-- Output of AND F.
	);
    END COMPONENT;
begin

    R0: palcell port map(zero,
			one,zero,one,zero,one,zero,
			one,zero,one,zero,Q,zero,D,zero,
			EN, zero,one,zero,one,zero,
			CLK,R,open,open,Q,open,open);

    
end archREG;

use work.cypress.all; 

entity REG4 is
    port(D3:  IN bit;
         D2:  IN bit;
         D1:  IN bit;
         D0:  IN bit;
         EN:  IN bit;
	 CLK: IN bit;
         R:   IN bit;
	 Q3:  INOUT bit;
	 Q2:  INOUT bit;
	 Q1:  INOUT bit;
	 Q0:  INOUT bit);
end REG4;

architecture archREG4 of REG4 is
begin

    r0: reg port map(d0, en, clk, r, q0);
    r1: reg port map(d1, en, clk, r, q1);
    r2: reg port map(d2, en, clk, r, q2);
    r3: reg port map(d3, en, clk, r, q3);
    
end archREG4;

ENTITY reg8 IS
    PORT(D7,D6,D5,D4,D3,D2,D1,D0: IN BIT;
	 EN: IN BIT;
	 CLK: IN BIT;
	 R: IN BIT;
	 Q7,Q6,Q5,Q4,Q3,Q2,Q1,Q0: INOUT BIT);
END reg8;

ARCHITECTURE archREG8 of reg8 IS
BEGIN

	r1:reg4 port map( d3, d2, d1, d0,en,clk,r, q3, q2, q1, q0);
	r2:reg4 port map( d7, d6, d5, d4,en,clk,r, q7, q6, q5, q4);

END archREG8;

ENTITY reg16 IS
    PORT(D15,D14,D13,D12,D11,D10,D9,D8: IN BIT;
	 D7,D6,D5,D4,D3,D2,D1,D0: IN BIT;
	 EN: IN BIT;
	 CLK: IN BIT;
	 R: IN BIT;
	 Q15,Q14,Q13,Q12,Q11,Q10,Q9,Q8: INOUT BIT;
	 Q7,Q6,Q5,Q4,Q3,Q2,Q1,Q0: INOUT BIT);
END reg16;

ARCHITECTURE archREG16 of reg16 IS
BEGIN

	r1:reg4 port map( d3, d2, d1, d0,en,clk,r, q3, q2, q1, q0);
	r2:reg4 port map( d7, d6, d5, d4,en,clk,r, q7, q6, q5, q4);
	r3:reg4 port map(d11,d10, d9, d8,en,clk,r,q11,q10, q9, q8);
	r4:reg4 port map(d15,d14,d13,d12,en,clk,r,q15,q14,q13,q12);

END archREG16;
