--
--	Copyright Cypress Semiconductor Corporation, 1994,1995,1996
--	  as an unpublished work.
--

library ieee;
use ieee.std_logic_1164.all;

PACKAGE c335p IS			-- C335 Specific package.
    
    TYPE reg_def IS (reg, comb, dblreg);	
    TYPE fb_type IS (xreg, xpin, adj);		-- Noreg generic.

    COMPONENT c335m			-- Device C335 macro cell.
	GENERIC (			-- Cell configuration:
	    registered	: reg_def	:= reg;		-- REGistered.
	    bta_sel	: fb_type	:= xreg;	-- REGistered.
	    Tpd		: TIME		:= 15 ns;	-- C335-83
            Twh		: TIME		:=  5 ns;
            Twl		: TIME		:=  5 ns;
            Tis		: TIME		:=  2 ns;
            Tih		: TIME		:=  2 ns;
            Tico	: TIME		:= 18 ns;
            Tpzx	: TIME		:= 12 ns;
            Tpxz	: TIME		:= 12 ns;
	    Ts		: TIME		:=  9 ns;
	    Th		: TIME		:=  0 ns;
	    Tco		: TIME		:= 10 ns;
	    Tco2	: TIME		:= 18 ns);
	PORT (
	    x1	 : IN    std_logic;		-- D input to register.
	    x2	 : IN    std_logic;		-- D input to register.
	    clk	 : IN    std_logic;		-- Positive edge clock.
	    iclk : IN    std_logic;		-- Positive edge clock.
	    ss	 : IN    std_logic;		-- Asynchronous Set (chip wide).
	    sr	 : IN    std_logic;		-- Asynchronous Reset (chip wide).
	    oe	 : IN    std_logic;		-- output enable (pin 14).
	    qfb	 : OUT   std_logic;		-- Shared Bit to array
	    fb	 : OUT   std_logic;		-- Bit To Array 
	    y	 : INOUT std_logic);	-- IO pin.
    END COMPONENT;			-- End component c335m.

    COMPONENT c335h			-- Device C335 hidden register
	GENERIC (			-- Cell configuration:
	    Tpd		: TIME		:= 15 ns;	-- C335-83
            Twh		: TIME		:=  5 ns;
            Twl		: TIME		:=  5 ns;
            Tis		: TIME		:=  2 ns;
            Tih		: TIME		:=  2 ns;
            Tico	: TIME		:= 18 ns;
            Tpzx	: TIME		:= 12 ns;
            Tpxz	: TIME		:= 12 ns;
	    Ts		: TIME		:=  9 ns;
	    Th		: TIME		:=  0 ns;
	    Tco		: TIME		:= 10 ns;
	    Tco2	: TIME		:= 18 ns);
	PORT (
	    x1	: IN  std_logic;
	    x2	: IN  std_logic;
	    clk	: IN  std_logic;
	    ss	: IN  std_logic;
	    sr	: IN  std_logic;
	    qfb	: OUT std_logic);					
    END COMPONENT;			-- End component c335h.

    COMPONENT c335ireg		-- Device C335 input register/mux
	GENERIC (
	    iregister	: reg_def	:= reg;		-- Registered.
	    Tpd		: TIME		:= 15 ns;	-- C335-83
            Twh		: TIME		:=  5 ns;
            Twl		: TIME		:=  5 ns;
            Tis		: TIME		:=  2 ns;
            Tih		: TIME		:=  2 ns;
            Tico	: TIME		:= 18 ns;
            Tpzx	: TIME		:= 12 ns;
            Tpxz	: TIME		:= 12 ns;
	    Ts		: TIME		:=  9 ns;
	    Th		: TIME		:=  0 ns;
	    Tco		: TIME		:= 10 ns;
	    Tco2	: TIME		:= 18 ns);
	PORT (
	    d	 : IN  std_logic;
	    iclk : IN  std_logic;
	    fb	 : OUT std_logic);
    END COMPONENT;			-- End component c335ireg.

    COMPONENT c335ckmx		-- Device C335 Clock Mux.
	GENERIC (
	    Tpd		: TIME		:= 15 ns;	-- C335-83
            Twh		: TIME		:=  5 ns;
            Twl		: TIME		:=  5 ns;
            Tis		: TIME		:=  2 ns;
            Tih		: TIME		:=  2 ns;
            Tico	: TIME		:= 18 ns;
            Tpzx	: TIME		:= 12 ns;
            Tpxz	: TIME		:= 12 ns;
	    Ts		: TIME		:=  9 ns;
	    Th		: TIME		:=  0 ns;
	    Tco		: TIME		:= 10 ns;
	    Tco2	: TIME		:= 18 ns);
	PORT (
	    clksrc	 : IN  std_logic;
	    clk		 : OUT std_logic);
    END COMPONENT;			-- End component c335ckmx.

END c335p;				-- End package c335p;

