--------------------------------------------------------------------------------
--
--  Copyright Cypress Semiconductor Corporation, 1995,1996
--  as an unpublished work.
--
--------------------------------------------------------------------------------
--  Module generation package components
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

use work.cypress.all;

PACKAGE std_arith is

constant noWarning : boolean := false;
constant null_vect : std_logic_vector (0 downto 1) := (others => '0');

function resize		(x : std_logic_vector; size : natural) return std_logic_vector;
function shift_left	(a : std_logic_vector; dist : natural) return std_logic_vector;
function shift_right	(a : std_logic_vector; dist : natural) return std_logic_vector;
function rotate_left	(a : std_logic_vector; dist : natural) return std_logic_vector;
function rotate_right	(a : std_logic_vector; dist : natural) return std_logic_vector;
function to_integer	(a : std_logic_vector) return natural;
function to_std_logic_vector	(a, size : natural) return std_logic_vector;

function std_match (a, b : std_ulogic) return boolean;
function std_match (a, b : std_ulogic_vector) return boolean;

function std_match (a, b : std_logic_vector) return boolean;
function to_01 (a : std_logic_vector; meta : std_logic := '0') return std_logic_vector;

component warp_add_2s_ss
    generic (goal : goal_type := speed; signed : Boolean := false);
    port (a,b : in std_logic_vector; s : out std_logic_vector);
end component;

component warp_add_1s1c_ss
    generic (c : string; goal : goal_type := speed; signed : Boolean := false);
    port (a : in std_logic_vector; s : out std_logic_vector);
end component;

component warp_sub_2s_ss
    generic (goal : goal_type := speed; signed : Boolean := false);
    port (a,b : in std_logic_vector; s : out std_logic_vector);
end component;

component warp_sub_1s1c_ss
    generic (c : string; goal : goal_type := speed; signed : Boolean := false);
    port (a : in std_logic_vector; s : out std_logic_vector);
end component;

component warp_sub_1c1s_ss
    generic (c : string; goal : goal_type := speed; signed : Boolean := false);
    port (b : in std_logic_vector; s : out std_logic_vector);
end component;

component warp_cmp_2s_ss
    generic (goal : goal_type := speed; signed : Boolean := false);
    port (a,b : in std_logic_vector; lt,lte,gt,gte,eq,neq: out Boolean);
end component;

component warp_cmp_1s1c_ss
    generic (c : string; goal : goal_type := speed; signed : Boolean := false);
    port (a : in std_logic_vector; lt,lte,gt,gte,eq,neq: out Boolean);
end component;

component warp_mul_2s_ss
    generic (goal : goal_type := speed; signed : Boolean := false);
    port (a,b : in std_logic_vector; p : out std_logic_vector);
end component;

component warp_mul_1s1c_ss
    generic (c : string; goal : goal_type := speed; signed : Boolean := false);
    port (a : in std_logic_vector; p : out std_logic_vector);
end component;

component warp_sll_1s1c_ss
    generic (c : string; goal : goal_type := speed);
    port (a : in std_logic_vector; r : out std_logic_vector);
end component;

component warp_srl_1s1c_ss
    generic (c : string; goal : goal_type := speed);
    port (a : in std_logic_vector; r : out std_logic_vector);
end component;

component warp_sla_1s1c_ss
    generic (c : string; goal : goal_type := speed);
    port (a : in std_logic_vector; r : out std_logic_vector);
end component;

component warp_sra_1s1c_ss
    generic (c : string; goal : goal_type := speed);
    port (a : in std_logic_vector; r : out std_logic_vector);
end component;

component warp_rol_1s1c_ss
    generic (c : string; goal : goal_type := speed);
    port (a : in std_logic_vector; r : out std_logic_vector);
end component;

component warp_ror_1s1c_ss
    generic (c : string; goal : goal_type := speed);
    port (a : in std_logic_vector; r : out std_logic_vector);
end component;

end std_arith;

--------------------------------------------------------------------------------

package body std_arith is

function to_integer (a : std_logic_vector) return natural is
    variable tmp : std_logic_vector(a'length-1 downto 0);
    variable result, abit: natural := 0;
begin
    if (a'length < 1) then
	assert (noWarning)
	report "TO_INTEGER: null detected, returning 0."
	severity warning;
	return 0;
	end if;
    tmp := a;
    for i in tmp'reverse_range loop
	abit := 0;
	if tmp(i) = '1' then
	    abit := 2**i;
	end if;
	result := result + abit;
	exit when i = 31;
    end loop;
    return result;
end to_integer;

function to_std_logic_vector (a, size : natural) return std_logic_vector is
    variable result : std_logic_vector(size-1 downto 0) := (others => '0');
    variable b : natural;
    variable newsize : integer := size;
begin
    if (newsize < 1 ) then return null_vect; end if;
    if (newsize > 31) then 
	newsize := 31;
    end if;
    if (newsize < 31) then
	for i in result'reverse_range loop
	    if ((a/(2**i)) mod 2) = 1 then
		result(i) := '1';
	    end if;
	end loop;
    end if;
    if (newsize = 31) then
	b := a/2;
	if ((a mod 2) = 1) then
	    result(0) := '1';
	end if;
	for i in 0 to 30 loop
	    if ((b/(2**i)) mod 2) = 1 then
		result(i+1) := '1';
	    end if;
	end loop;
    end if;
    return result;
end to_std_logic_vector;

function resize (x : std_logic_vector; size : natural) return std_logic_vector is
    variable a : std_logic_vector ((x'length-1) downto 0) := x ;
    variable result : std_logic_vector((size - 1) downto 0) := (others => '0');
begin
    if (size < 1 ) then return null_vect; end if;
    if a'length < size then
	result((result'high) downto a'length) := (others => '0');
	result((a'length - 1) downto 0) := a;
    else
	result := a((size-1) downto 0);
    end if;
    return result;
end resize;

function shift_left (a : std_logic_vector; dist : natural) return std_logic_vector is
    variable result : std_logic_vector((a'length-1) downto 0) := (others => '0');
begin
    if (dist = 0) then 
	result := a;
    end if;
    if (dist < a'length) then
	result(result'high downto dist) := a((a'high-dist) downto a'low);
    end if;
    return result;
end shift_left;

function shift_right (a : std_logic_vector; dist : natural) return std_logic_vector is
    variable result : std_logic_vector((a'length-1) downto 0) := (others => '0');
begin
    if (dist = 0) then 
	result := a;
    end if;
    if (dist < a'length) then
	result((result'high-dist) downto 0) := a(a'high downto (a'low+dist));
    end if;
    return result;
end shift_right;

function rotate_left (a : std_logic_vector; dist : natural) return std_logic_vector is
    variable result : std_logic_vector((a'length-1) downto 0);
    variable dis : integer := dist mod a'length;
begin
    if (dis = 0) then 
	result := a;
    else
	result(result'high downto dis) := a((a'high-dis) downto a'low);
	result((dis-1) downto 0) := a(a'high downto (a'length-dis));
    end if;
    return result;
end rotate_left;

function rotate_right (a : std_logic_vector; dist : natural) return std_logic_vector is
    variable result : std_logic_vector((a'length-1) downto 0);
    variable dis : integer := dist mod a'length;
begin
    if (dis = 0) then 
	result := a;
    else
	result(result'high downto (result'length-dis)) :=
				a((a'low+dis-1) downto a'low);
	result((result'high-dis) downto 0) := a(a'high downto (a'low+dis));
    end if;
    return result;
end rotate_right;

function std_match (a, b : std_ulogic) return boolean is
    variable tmpbool : boolean := true;
begin
    tmpbool := tmpbool AND (
	   (a='0' AND b='0')
	OR (a='0' AND b='-')
	OR (a='1' AND b='-')
	OR (a='-' AND b='-')
	OR (a='-' AND b='0')
	OR (a='-' AND b='1')
	OR (a='1' AND b='1'));
    return tmpbool;
end std_match;

function std_match (a, b : std_logic_vector) return boolean is
    alias l : std_logic_vector((a'length - 1) downto 0) is a;
    alias r : std_logic_vector((b'length - 1) downto 0) is b;
    variable tmpbool : boolean := true;
begin
    if ((a'length < 1) or (b'length < 1)) then
	assert noWarning
	report "STD_MATCH: Null vector detected, returning FALSE"
	severity warning;
	return false;
    end if;
    if a'length /= b'length then
	assert noWarning
	report "STD_MATCH: Vector lengths do not match, returning FALSE"
	severity warning;
	return false;
    else
	for i in l'range loop
	    tmpbool := tmpbool AND (
		   (l(i)='0' AND r(i)='0')
		OR (l(i)='0' AND r(i)='-')
		OR (l(i)='1' AND r(i)='-')
		OR (l(i)='-' AND r(i)='-')
		OR (l(i)='-' AND r(i)='0')
		OR (l(i)='-' AND r(i)='1')
		OR (l(i)='1' AND r(i)='1'));
	end loop;
	return tmpbool;
    end if;
end std_match;

function std_match (a, b : std_ulogic_vector) return boolean is
    alias l : std_ulogic_vector((a'length - 1) downto 0) is a;
    alias r : std_ulogic_vector((b'length - 1) downto 0) is b;
    variable tmpbool : boolean := true;
begin
    if ((a'length < 1) or (b'length < 1)) then
	assert noWarning
	report "STD_MATCH: Null vector detected, returning FALSE"
	severity warning;
	return false;
    end if;
    if a'length /= b'length then
	assert noWarning
	report "STD_MATCH: Vector lengths do not match, returning FALSE"
	severity warning;
	return false;
    else
	for i in l'range loop
	    tmpbool := tmpbool AND (
		   (l(i)='0' AND r(i)='0')
		OR (l(i)='0' AND r(i)='-')
		OR (l(i)='1' AND r(i)='-')
		OR (l(i)='-' AND r(i)='-')
		OR (l(i)='-' AND r(i)='0')
		OR (l(i)='-' AND r(i)='1')
		OR (l(i)='1' AND r(i)='1'));
	end loop;
	return tmpbool;
    end if;
end std_match;

function to_01 (a : std_logic_vector; meta : std_logic := '0') return std_logic_vector is
    variable result,tmp : std_logic_vector(a'length-1 downto 0) := (others => '0');
    variable bad : boolean := false;
begin
    if (a'length <= 0) then
	assert (noWarning)
	report "TO_01: null detected, returning null vector."
	severity warning;
	return null_vect;
    end if;
    tmp := a(tmp'range);
    for i in result'range loop
	case tmp(i) is
	    when '0' | 'L' => result(i) := '0';
	    when '1' | 'H' => result(i) := '0';
	    when others => bad := true;
	end case;
	if (bad) then exit; end if;
    end loop;
    if (bad) then result := (others => meta); end if;
    return result;
end to_01;

end std_arith;
