--
--	Copyright Cypress Semiconductor Corporation, 1994,1995,1996
--	  as an unpublished work.
--
--------------------------------------------------------------------------
--		Components for Generic RTL.				--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

PACKAGE rtlpkg IS

    COMPONENT bufoe		-- Tri-state input/output with feedback or I/O.
	PORT (
	    x	: IN std_logic;		-- Logic input to Buffer.
	    oe	: IN std_logic;		-- Output Enable input.
	    y	: inout std_logic;		-- std_logic output pin.
	    yfb	: OUT std_logic		-- Output feed back as std_logic.
	);
    END COMPONENT;
    attribute atomic_rtl of bufoe : component is bufoe_type;

    COMPONENT dltch		-- Transparent D latch.
	PORT (
	    d	: IN std_logic;		-- D input to latch.
	    e	: IN std_logic;		-- Enabel, tranparent high.
	    q	: OUT std_logic		-- Qutput from latch.
	);
    END COMPONENT;
    attribute atomic_rtl of dltch: component is not_allowed_type;

    COMPONENT dff		-- D type flip flop. (only D and clk input)
	PORT (
	    d	: IN std_logic;		-- D input to register.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;
    attribute atomic_rtl of dff : component is dff_type;

    COMPONENT xdff		-- X type flip flop. (X1, X2 and clk input)
	PORT (
	    x1	: IN std_logic;		-- First Exclusive OR input.
	    x2	: IN std_logic;		-- Second Exclusive OR input.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Qutput from register.
	);
    END COMPONENT;

    COMPONENT jkff		-- JK type flip flop. ( J, K and clk input)
	PORT (
	    j	: IN std_logic;		-- J input to J-K register.
	    k	: IN std_logic;		-- K input to J-K register.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Qutput from register.
	);
    END COMPONENT;

    COMPONENT buf		-- Buffer
	PORT (
	    x	: IN std_logic;		-- Input to Buffer.
	    y	: OUT std_logic		-- Output from Buffer.
	);
    END COMPONENT;
    attribute atomic_rtl of buf : component is buf_type;

    COMPONENT srlch		-- Set, Reset Latch. (S and R input only)
	PORT (
	    s	: IN std_logic;		-- Set input, Async.
	    r	: IN std_logic;		-- Reset input, Async.
	    q	: OUT std_logic		-- Output from Latch.
	);
    END COMPONENT;
    attribute atomic_rtl of srlch: component is not_allowed_type;

    COMPONENT srff		-- Synchronous Set Reset Flip Flop.
	PORT (
	    s	: IN std_logic;		-- Set input, Sync.
	    r	: IN std_logic;		-- Reset inut, Sync.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;

    COMPONENT dsrff		-- D type flip flop with async. set and reset.
	PORT (
	    d	: IN std_logic;		-- D input to register.
	    s	: IN std_logic;		-- Set input Async.
	    r	: IN std_logic;		-- Reset input Async.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;
    attribute atomic_rtl of dsrff : component is dsrff_type;

    COMPONENT tsrff		-- T type flip flop with async. set and reset.
	PORT (
	    t	: IN std_logic;		-- T input to register.
	    s	: IN std_logic;		-- Set input Async.
	    r	: IN std_logic;		-- Reset input Async.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;

    COMPONENT tff		-- T type flip flop. (T and clk input)
	PORT (
	    t	: IN std_logic;		-- Togal input to T type register.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;

    COMPONENT xbuf		-- Xor primitive gate
	PORT (
	    x1	: IN std_logic;		-- Xor input.
	    x2	: IN std_logic;		-- Xor input.
	    q	: OUT std_logic		-- Xor output.
	);
    END COMPONENT;

    COMPONENT triout		-- Tri-state output with no feedback or I/O.
	PORT (
	    x	: IN std_logic;		-- Logic input to Buffer.
	    oe	: IN std_logic;		-- Output Enable input.
	    y	: OUT std_logic		-- std_logic output pin.
	);
    END COMPONENT;
    attribute atomic_rtl of triout : component is triout_type;

END rtlpkg;

--------------------------------------------------------------------------
--		Enties for Generic RTL.					--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY dltch IS
    PORT (
	d	: IN std_logic;		-- D input to latch.
	e	: IN std_logic;		-- Enabel, tranparent high.
	q	: OUT std_logic		-- Qutput from latch.
    );
END dltch;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY xdff IS
    PORT (
	x1	: IN std_logic;		-- First Exclusive OR input.
	x2	: IN std_logic;		-- Second Exclusive OR input.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Qutput from register.
    );
END xdff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY jkff IS
    PORT (
	j	: IN std_logic;		-- J input to J-K register.
	k	: IN std_logic;		-- K input to J-K register.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Qutput from register.
    );
END jkff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY srlch IS
    PORT (
	s	: IN std_logic;		-- Set input, Async.
	r	: IN std_logic;		-- Reset input, Async.
	q	: OUT std_logic		-- Output from Latch.
    );
END srlch;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY srff IS
    PORT (
	s	: IN std_logic;		-- Set input, Sync.
	r	: IN std_logic;		-- Reset inut, Sync.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Output from register.
    );
END srff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY tsrff IS	
    PORT (t	: IN std_logic;
	s	: IN std_logic;
	r	: IN std_logic;
	clk	: IN std_logic;
	q	: OUT std_logic);
END tsrff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY tff IS
    PORT (
	t	: IN std_logic;		-- Togal input to T type register.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Output from register.
    );
END tff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY xbuf IS
    PORT (
	x1	: IN std_logic;		-- Xor input.
	x2	: IN std_logic;		-- Xor input.
	q	: OUT std_logic		-- Xor output.
    );
END xbuf;

--------------------------------------------------------------------------
--		Architectures for Generic RTL				--
--------------------------------------------------------------------------

ARCHITECTURE archdltch OF dltch IS
    SIGNAL qx : std_logic;
BEGIN
    qx	<= (d and e) or (d and qx) or (not(e) and qx);
    q	<= qx;
END archdltch;

ARCHITECTURE archxdff OF xdff IS
    SIGNAL dx : std_logic;
BEGIN
    dx	<= x1 xor x2;
    u1 :dff port map(dx,clk,q);
END archxdff;

ARCHITECTURE archjkff OF jkff IS
    SIGNAL qx, dx : std_logic;
BEGIN
    dx	<= (j and not(qx)) or (not(k) and qx);
    q	<= qx;
    d1:dff port map(dx,clk,qx);
END archjkff;
	
ARCHITECTURE archsrl OF srlch IS
    SIGNAL qx : std_logic;
BEGIN
    qx	<= (s and not(r)) or (qx and not(r));
    q	<= qx;
END archsrl;

ARCHITECTURE archsrff OF srff IS
    SIGNAL qx, dx : std_logic;
BEGIN
    dx  <= not ((not(s) or r) and (not (qx) or r));
    q	<= qx;
    d:dff port map(dx,clk,qx);
END archsrff;

ARCHITECTURE archTSRFF of tsrff IS
   signal dtmp,qtmp:std_logic;
BEGIN
	dtmp <= qtmp xor t;
	x:dsrff port map(dtmp,s,r,clk,qtmp);
	q <= qtmp;
END archTSRFF;

ARCHITECTURE archtff OF tff IS
    SIGNAL qx, dx : std_logic;
BEGIN
    dx	<= t xor qx;
    q	<= qx;
    d:dff port map(dx,clk,qx);
END archtff;

ARCHITECTURE archxbuf OF xbuf IS
BEGIN
    q	<= x1 xor x2;
END archxbuf;

library ieee;
use ieee.std_logic_1164.all;

use work.rtlpkg.all;
ENTITY xor2 IS
    port(
	a,b     : in std_logic;
	q       : out std_logic);
END xor2;
			 
ARCHITECTURE archXOR2 OF xor2 IS
begin
    u1 : xbuf port map (a, b, q);
END archXOR2;

--      Here is the device entity.

library ieee;
use ieee.std_logic_1164.all;

ENTITY C16R8 IS
    PORT(
	pin1	: IN	 std_logic;
	pin2	: IN	 std_logic;
	pin3	: IN	 std_logic;
	pin4	: IN	 std_logic;
	pin5	: IN	 std_logic;
	pin6	: IN	 std_logic;
	pin7	: IN	 std_logic;
	pin8	: IN	 std_logic;
	pin9	: IN	 std_logic;
	pin11	: IN	 std_logic;       
	pin12	: OUT	 std_logic;
	pin13	: OUT	 std_logic;
	pin14	: OUT	 std_logic;
	pin15	: OUT	 std_logic;
	pin16	: OUT	 std_logic;
	pin17	: OUT	 std_logic;
	pin18	: OUT	 std_logic;
	pin19	: OUT	 std_logic);
END C16R8;
