--
--	Copyright Cypress Semiconductor Corporation, 1994,1995,1996
--	  as an unpublished work.
--
--------------------------------------------------------------------------
--		Components for Generic RTL.				--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

PACKAGE rtlpkg IS

    COMPONENT bufoe		-- Tri-state input/output with feedback or I/O.
	PORT (
	    x	: IN std_logic;		-- Logic input to Buffer.
	    oe	: IN std_logic;		-- Output Enable input.
	    y	: inout std_logic;	-- std_logic output pin.
	    yfb	: OUT std_logic		-- Output feed back as std_logic.
	);
    END COMPONENT;
    attribute atomic_rtl of bufoe : component is bufoe_type;

    COMPONENT dltch		-- Transparent D latch.
	PORT (
	    d	: IN std_logic;		-- D input to latch.
	    e	: IN std_logic;		-- Enabel, tranparent high.
	    q	: OUT std_logic		-- Qutput from latch.
	);
    END COMPONENT;

    COMPONENT dff		-- D type flip flop. (only D and clk input)
	PORT (
	    d	: IN std_logic;		-- D input to register.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;
    attribute atomic_rtl of dff : component is dff_type;

    COMPONENT xdff		-- X type flip flop. (X1, X2 and clk input)
	PORT (
	    x1	: IN std_logic;		-- First Exclusive OR input.
	    x2	: IN std_logic;		-- Second Exclusive OR input.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Qutput from register.
	);
    END COMPONENT;
    attribute atomic_rtl of xdff : component is xdff_type;

    COMPONENT jkff		-- JK type flip flop. ( J, K and clk input)
	PORT (
	    j	: IN std_logic;		-- J input to J-K register.
	    k	: IN std_logic;		-- K input to J-K register.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Qutput from register.
	);
    END COMPONENT;

    COMPONENT buf		-- Buffer
	PORT (
	    x	: IN std_logic;		-- Input to Buffer.
	    y	: OUT std_logic		-- Output from Buffer.
	);
    END COMPONENT;
    attribute atomic_rtl of buf : component is buf_type;

    COMPONENT srlch		-- Set, Reset Latch. (S and R input only)
	PORT (
	    s	: IN std_logic;		-- Set input, Async.
	    r	: IN std_logic;		-- Reset input, Async.
	    q	: OUT std_logic		-- Output from Latch.
	);
    END COMPONENT;

    COMPONENT srff		-- Synchronous Set Reset Flip Flop.
	PORT (
	    s	: IN std_logic;		-- Set input, Sync.
	    r	: IN std_logic;		-- Reset inut, Sync.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;

    COMPONENT dsrff		-- D type flip flop with async. set and reset.
	PORT (
	    d	: IN std_logic;		-- D input to register.
	    s	: IN std_logic;		-- Set input Async.
	    r	: IN std_logic;		-- Reset input Async.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;
    attribute atomic_rtl of dsrff : component is dsrff_type;

    COMPONENT dpcff		-- D type flip flop with async. set and reset.
	PORT (
	    d	: IN std_logic;		-- D input to register.
	    s	: IN std_logic;		-- Set input Async.
	    r	: IN std_logic;		-- Reset input Async.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;
    attribute atomic_rtl of dpcff : component is dpcff_type;

    COMPONENT tsrff		-- T type flip flop with async. set and reset.
	PORT (
	    t	: IN std_logic;		-- T input to register.
	    s	: IN std_logic;		-- Set input Async.
	    r	: IN std_logic;		-- Reset input Async.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;

    COMPONENT tff		-- T type flip flop. (T and clk input)
	PORT (
	    t	: IN std_logic;		-- Togal input to T type register.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;

    COMPONENT xbuf		-- Xor primitive gate
	PORT (
	    x1	: IN std_logic;		-- Xor input.
	    x2	: IN std_logic;		-- Xor input.
	    q	: OUT std_logic		-- Xor output.
	);
    END COMPONENT;
    attribute atomic_rtl of xbuf : component is xbuf_type;

    COMPONENT triout		-- Tri-state output with no feedback or I/O.
	PORT (
	    x	: IN std_logic;		-- Logic input to Buffer.
	    oe	: IN std_logic;		-- Output Enable input.
	    y	: OUT std_logic		-- std_logic output pin.
	);
    END COMPONENT;
    attribute atomic_rtl of triout : component is triout_type;

-- c335 specials

    signal gsr	: std_logic;				-- Global sync. reset
    attribute node_num	of gsr : signal is 29;	-- is node 29.

    signal gss	: std_logic;				-- Global sync. set
    attribute node_num	of gss : signal is 30;	-- is node 30.

    component MC_C335
	generic(
	    has_output_register:boolean:= true
	);
	port(
	    x1	: in std_logic;		-- Exor Macro Input (Single).
	    x2	: in std_logic;		-- Exor Macro Input (Multiple).
	    clk	: in std_logic;		-- Positive edge, output reg, clock.
	    clk_i	: in std_logic;	-- Positive edge, input reg, clock.
	    oe	: in std_logic;		-- Output Enable.
	    q_bar	: out std_logic;	-- Output from output register.
	    q_i	: out std_logic;		-- Output from input register.
	    y	: inout std_logic;	-- Macro cell In/Out Pin.
	    yfb	: out std_logic		-- Feedback std_logic from Pin.
	);
    end component;
    attribute atomic_rtl of MC_C335 : component is MC_C335_type;

--  Schematic versions of macrocell.

    component MC335NR                   --  Non-registered.
	port(
	    x1	: in std_logic;		-- Exor Macro Input (Single).
	    x2	: in std_logic;		-- Exor Macro Input (Multiple).
	    clk	: in std_logic;		-- Positive edge, output reg, clock.
	    clk_i	: in std_logic;	-- Positive edge, input reg, clock.
	    oe	: in std_logic;		-- Output Enable.
	    q_bar	: out std_logic;	-- Output from output register.
	    q_i	: out std_logic;		-- Output from input register.
	    y	: inout std_logic;	-- Macro cell In/Out Pin.
	    yfb	: out std_logic		-- Feedback std_logic from Pin.
	);
    end component;

    component MC335RG                   --  Registered.
	port(
	    x1	: in std_logic;		-- Exor Macro Input (Single).
	    x2	: in std_logic;		-- Exor Macro Input (Multiple).
	    clk	: in std_logic;		-- Positive edge, output reg, clock.
	    clk_i	: in std_logic;	-- Positive edge, input reg, clock.
	    oe	: in std_logic;		-- Output Enable.
	    q_bar	: out std_logic;	-- Output from output register.
	    q_i	: out std_logic;		-- Output from input register.
	    y	: inout std_logic;	-- Macro cell In/Out Pin.
	    yfb	: out std_logic		-- Feedback std_logic from Pin.
	);
    end component;

    component ir_C335
	port(
	    d_i	: in std_logic;		-- D input to input register.
	    clk_i	: in std_logic;	-- Positive edge, input reg, clock.
	    q_i	: out std_logic		-- Output from input register.
	);
    end component;
    attribute atomic_rtl of ir_C335 : component is ir_C335_type;

END rtlpkg;

--------------------------------------------------------------------------
--		Enties for Generic RTL.					--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY dltch IS
    PORT (
	d	: IN std_logic;		-- D input to latch.
	e	: IN std_logic;		-- Enabel, tranparent high.
	q	: OUT std_logic		-- Qutput from latch.
    );
END dltch;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY jkff IS
    PORT (
	j	: IN std_logic;		-- J input to J-K register.
	k	: IN std_logic;		-- K input to J-K register.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Qutput from register.
    );
END jkff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY srlch IS
    PORT (
	s	: IN std_logic;		-- Set input, Async.
	r	: IN std_logic;		-- Reset input, Async.
	q	: OUT std_logic		-- Output from Latch.
    );
END srlch;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY srff IS
    PORT (
	s	: IN std_logic;		-- Set input, Sync.
	r	: IN std_logic;		-- Reset inut, Sync.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Output from register.
    );
END srff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY tsrff IS	
    PORT (t	: IN std_logic;
	s	: IN std_logic;
	r	: IN std_logic;
	clk	: IN std_logic;
	q	: OUT std_logic);
END tsrff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY tff IS
    PORT (
	t	: IN std_logic;		-- Togal input to T type register.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Output from register.
    );
END tff;

--------------------------------------------------------------------------
--		Architectures for Generic RTL				--
--------------------------------------------------------------------------

ARCHITECTURE archdltch OF dltch IS
    SIGNAL qx : std_logic;
BEGIN
    qx	<= (d and e) or (d and qx) or (not(e) and qx);
    q	<= qx;
END archdltch;

ARCHITECTURE archjkff OF jkff IS
    SIGNAL qx, dx : std_logic;
BEGIN
    dx	<= (j and not(qx)) or (not(k) and qx);
    q	<= qx;
    d1:dff port map(dx,clk,qx);
END archjkff;
	
ARCHITECTURE archsrl OF srlch IS
    SIGNAL qx : std_logic;
BEGIN
    qx	<= (s and not(r)) or (qx and not(r));
    q	<= qx;
END archsrl;

ARCHITECTURE archsrff OF srff IS
    SIGNAL qx, dx : std_logic;
BEGIN
    dx  <= not ((not(s) or r) and (not (qx) or r));
    q	<= qx;
    d:dff port map(dx,clk,qx);
END archsrff;

ARCHITECTURE archTSRFF of tsrff IS
   signal dtmp,qtmp:std_logic;
BEGIN
	dtmp <= qtmp xor t;
	x:dsrff port map(dtmp,s,r,clk,qtmp);
	q <= qtmp;
END archTSRFF;

ARCHITECTURE archtff OF tff IS
    SIGNAL qx, dx : std_logic;
BEGIN
    dx	<= t xor qx;
    q	<= qx;
    d:dff port map(dx,clk,qx);
END archtff;

library ieee;
use ieee.std_logic_1164.all;

use work.rtlpkg.all;
ENTITY xor2 IS
    port(
	a,b     : in std_logic;
	q       : out std_logic);
END xor2;
			 
ARCHITECTURE archXOR2 OF xor2 IS
begin
    u1 : xbuf port map (a, b, q);
END archXOR2;

--  Schematic versions of macrocell.

library ieee;
use ieee.std_logic_1164.all;

USE work.rtlpkg.all;

entity MC335NR is                       --  Non-registered.
    port(
	x1	: in std_logic;		-- Exor Macro Input (Single).
	x2	: in std_logic;		-- Exor Macro Input (Multiple).
	clk	: in std_logic;		-- Positive edge, output reg, clock.
	clk_i	: in std_logic;		-- Positive edge, input reg, clock.
	oe	: in std_logic;		-- Output Enable.
	q_bar	: out std_logic;		-- Output from output register.
	q_i	: out std_logic;		-- Output from input register.
	y	: inout std_logic;	-- Macro cell In/Out Pin.
	yfb	: out std_logic		-- Feedback std_logic from Pin.
    );
end MC335NR;

ARCHITECTURE archMC335NR OF MC335NR IS

	signal d,d_bar,q,yin : std_logic;

BEGIN
--  x1: MC_C335 generic map (has_output_register => false)
--                port map (x1, x2, clk, clk_i, oe, q_bar, q_i, y, yfb);

	U0: xbuf port map (x1,x2,d);
	d_bar <= not d;
	U1: bufoe port map (d_bar,oe,y,yin);
	yfb <= yin;
	U2: dff port map (d,clk,q);
	q_bar <= not q;
	U3: dff port map (yin,clk_i,q_i);

END archMC335NR;

library ieee;
use ieee.std_logic_1164.all;

USE work.rtlpkg.all;

entity MC335RG is                       --  Registered.
    port(
	x1	: in std_logic;		-- Exor Macro Input (Single).
	x2	: in std_logic;		-- Exor Macro Input (Multiple).
	clk	: in std_logic;		-- Positive edge, output reg, clock.
	clk_i	: in std_logic;		-- Positive edge, input reg, clock.
	oe	: in std_logic;		-- Output Enable.
	q_bar	: out std_logic;		-- Output from output register.
	q_i	: out std_logic;		-- Output from input register.
	y	: inout std_logic;		-- Macro cell In/Out Pin.
	yfb	: out std_logic		-- Feedback std_logic from Pin.
    );
end MC335RG;

ARCHITECTURE archMC335RG OF MC335RG IS

	signal q,qb,yin : std_logic;

BEGIN
--  x1: MC_C335 generic map (has_output_register => true)
--                port map (x1, x2, clk, clk_i, oe, q_bar, q_i, y, yfb);

	U1: xdff port map (x1,x2,clk,q);
	qb <= not q;
	q_bar <= not q;
	U2: bufoe port map (qb,oe,y,yin);
	yfb <= yin;
	U3: dff port map (yin,clk_i,q_i);

END archMC335RG;

--      Here is the device entity.

library ieee;
use ieee.std_logic_1164.all;

ENTITY C335 IS
PORT(
	pin1  :IN	std_logic;
	pin2  :IN	std_logic;
	pin3  :IN	std_logic;
	pin4  :in	std_logic;
	pin5  :in	std_logic;
	pin6  :in	std_logic;
	pin7  :in	std_logic;
	pin9  :in	std_logic;
	pin10 :in	std_logic;
	pin11 :IN	std_logic;
	pin13 :IN	std_logic;
	pin12 :IN	std_logic;
	pin14 :in	std_logic;
	pin15 :INOUT	std_logic;
	pin16 :INOUT	std_logic;
	pin17 :INOUT	std_logic;
	pin18 :INOUT	std_logic;
	pin19 :INOUT	std_logic;
	pin20 :INOUT	std_logic;
	pin23 :INOUT	std_logic;
	pin24 :INOUT	std_logic;
	pin25 :INOUT	std_logic;
	pin26 :INOUT	std_logic;
	pin27 :INOUT	std_logic;
	pin28 :INOUT	std_logic);
END C335;
