--
--	Copyright Cypress Semiconductor Corporation, 1994,1995,1996
--	  as an unpublished work.
--

-------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

PACKAGE rtlpkg IS

    SIGNAL one  : std_logic   := '1';
    SIGNAL zero : std_logic   := '0';

--------------------------------------------------------------------------
--		pASIC specific macro cell (rtl)				--
--------------------------------------------------------------------------

-- IO Master Cells:

    COMPONENT PAincell		-- Master Input Cell.
	PORT (
	    ip	: IN std_logic;	-- Input pad.
	    ini	: OUT std_logic;	-- Inverting output of cell.
	    iz	: OUT std_logic	-- Non-inverting output of cell.
	);
    END COMPONENT;
    ATTRIBUTE atomic_rtl	OF PAincell : COMPONENT IS PAincell_type;
        
    COMPONENT PAbicell		-- Master Bi-directional Cell.
	PORT (
	    ie	: IN std_logic;	-- Output Enable.
	    i1	: IN std_logic;	-- Inverting input.
	    i2	: IN std_logic;	-- Non-inverting input.
	    iz	: OUT std_logic;	-- Path into chip.
	    ip	: inout std_logic	-- I/O pad.
	);
    END COMPONENT;
    ATTRIBUTE atomic_rtl	OF PAbicell : COMPONENT IS PAbicell_type;
    	    
    COMPONENT PAckcell		-- Master Clock Cell.
	PORT (
	    ip	: IN std_logic;	-- Input pad.
	    ini	: OUT std_logic;	-- Inverted.
	    iz	: OUT std_logic;	-- Non-inverted.
	    ic	: OUT std_logic	-- To "Dedicated clock network".
	);
    END COMPONENT;
    ATTRIBUTE atomic_rtl	OF PAckcell : COMPONENT IS PAckcell_type;

    COMPONENT PAincel2		-- Master Input Cell.
	PORT (
	    ip	: IN std_logic;	-- Input pad.
	    ini	: OUT std_logic;	-- Inverting output of cell.
	    iz	: OUT std_logic;	-- Non-inverting output of cell.
	    iqe	: IN std_logic;	-- FF enable.
	    iqc	: IN std_logic;	-- FF clock.
	    ir	: IN std_logic;	-- FF reset.
	    iq	: OUT std_logic	-- FF output.
	);
    END COMPONENT;
    ATTRIBUTE atomic_rtl	OF PAincel2 : COMPONENT IS PAincell_type;
        
    COMPONENT PAbicel2		-- Master Bi-directional Cell.
	PORT (
	    ie	: IN std_logic;	-- Output Enable.
	    i1	: IN std_logic;	-- Inverting input.
	    i2	: IN std_logic;	-- Non-inverting input.
	    iz	: OUT std_logic;	-- Path into chip.
	    ip	: inout std_logic;	-- I/O pad.
	    iqe	: IN std_logic;	-- FF enable.
	    iqc	: IN std_logic;	-- FF clock.
	    ir	: IN std_logic;	-- FF reset.
	    iq	: OUT std_logic	-- FF output.
	);
    END COMPONENT;
    ATTRIBUTE atomic_rtl	OF PAbicel2 : COMPONENT IS PAbicell_type;
    	    
    COMPONENT PAckcel2		-- Master Clock Cell.
	PORT (
	    ip	: IN std_logic;	-- Input pad.
	    ini	: OUT std_logic;	-- Inverted.
	    iz	: OUT std_logic;	-- Non-inverted.
	    ic	: OUT std_logic;	-- To "Dedicated clock network".
	    iqe	: IN std_logic;	-- FF enable.
	    iqc	: IN std_logic;	-- FF clock.
	    ir	: IN std_logic;	-- FF reset.
	    iq	: OUT std_logic	-- FF output.
	);
    END COMPONENT;
    ATTRIBUTE atomic_rtl	OF PAckcel2 : COMPONENT IS PAckcell_type;

    COMPONENT clkbuf		-- Clock Buffer.
	PORT (
	    a	: IN std_logic;	-- Input.
	    z	: OUT std_logic	-- Output.
	);
    END COMPONENT;
    ATTRIBUTE atomic_rtl	OF clkbuf : COMPONENT IS PAclkbuf_type;

    
--		Fragment Cells:

    COMPONENT PAfrag_a
	PORT (
	    a1	: IN std_logic;	-- Non-invert to AND A.
	    a2	: IN std_logic;	-- Invert to AND A.
	    a3	: IN std_logic;	-- Non-invert to AND A.
	    a4	: IN std_logic;	-- Invert to AND A.
	    a5	: IN std_logic;	-- Non-invert to AND A.
	    a6	: IN std_logic;	-- Invert to AND A.

	    az	: OUT std_logic	-- Output of AND A.
	);
    END COMPONENT;
    ATTRIBUTE atomic_rtl	OF PAfrag_a : COMPONENT IS PAfrag_a_type;
    
    COMPONENT PAfrag_f		-- Fragment AND F.
	PORT (
	    f1	: IN std_logic;	-- Non-invert to AND F.
	    f2	: IN std_logic;	-- Invert to AND F.
	    f3	: IN std_logic;	-- Non-invert to AND F.
	    f4	: IN std_logic;	-- Invert to AND F.
	    f5	: IN std_logic;	-- Non-invert to AND F.
	    f6	: IN std_logic;	-- Invert to AND F.

	    fz	: OUT std_logic	-- Output of AND F.
	);
    END COMPONENT;
    ATTRIBUTE atomic_rtl	OF PAfrag_f : COMPONENT IS PAfrag_f_type;
        
    COMPONENT PAfrag_m		-- Fragment M.
	PORT (
	    os	: IN std_logic;	-- Select to MUX O.
	    nsi	: IN std_logic;	-- Select to MUX N.

	    b1	: IN std_logic;	-- Non-invert to AND B.
	    b2	: IN std_logic;	-- Invert to AND B.

	    c1	: IN std_logic;	-- Non-invert to AND C.
	    c2	: IN std_logic;	-- Invert to AND C.

	    d1	: IN std_logic;	-- Non-invert to AND D.
	    d2	: IN std_logic;	-- Invert to AND D.

	    e1	: IN std_logic;	-- Non-invert to AND E.
	    e2	: IN std_logic;	-- Invert to AND E.

	    oz	: OUT std_logic;	-- Output of MUX O.
	    nz	: OUT std_logic	-- Output of MUX N.
	);
    END COMPONENT;
    ATTRIBUTE atomic_rtl	OF PAfrag_m : COMPONENT IS PAfrag_m_type;
    
    COMPONENT PAfrag_q		-- Fragment Q (flip flop).
	PORT (
	    qs	: IN std_logic;	-- Async Set on FF(Q).
	    qc	: IN std_logic;	-- Clock to FF(Q).
	    qr	: IN std_logic;	-- Async Reset on FF(Q).

	    qd	: IN std_logic;	-- Data on FF(Q).

	    qz	: OUT std_logic	-- Output of FF  Q.
	);
    END COMPONENT;
    ATTRIBUTE atomic_rtl	OF PAfrag_q : COMPONENT IS PAfrag_q_type;
    
COMPONENT PAlcell
	PORT (
	    qs	: IN std_logic;	-- Async Set on FF(Q).

	    a1	: IN std_logic;	-- Non-invert to AND A.
	    a2	: IN std_logic;	-- Invert to AND A.
	    a3	: IN std_logic;	-- Non-invert to AND A.
	    a4	: IN std_logic;	-- Invert to AND A.
	    a5	: IN std_logic;	-- Non-invert to AND A.
	    a6	: IN std_logic;	-- Invert to AND A.

	    b1	: IN std_logic;	-- Non-invert to AND B.
	    b2	: IN std_logic;	-- Invert to AND B.

	    c1	: IN std_logic;	-- Non-invert to AND C.
	    c2	: IN std_logic;	-- Invert to AND C.

	    d1	: IN std_logic;	-- Non-invert to AND D.
	    d2	: IN std_logic;	-- Invert to AND D.

	    e1	: IN std_logic;	-- Non-invert to AND E.
	    e2	: IN std_logic;	-- Invert to AND E.

	    f1	: IN std_logic;	-- Non-invert to AND F.
	    f2	: IN std_logic;	-- Invert to AND F.
	    f3	: IN std_logic;	-- Non-invert to AND F.
	    f4	: IN std_logic;	-- Invert to AND F.
	    f5	: IN std_logic;	-- Non-invert to AND F.
	    f6	: IN std_logic;	-- Invert to AND F.

	    qc	: IN std_logic;	-- Clock to FF(Q).
	    qr	: IN std_logic;	-- Async Reset on FF(Q).

	    az	: OUT std_logic;	-- Output of AND A.
	    oz	: OUT std_logic;	-- Output of MUX O.
	    qz	: OUT std_logic;	-- Output of FF  Q.
	    nz	: OUT std_logic;	-- Output of MUX N.
	    fz	: OUT std_logic	-- Output of AND F.
	);
    END COMPONENT;
    ATTRIBUTE atomic_rtl	OF PAlcell : COMPONENT IS PAlcell_type;
    
COMPONENT PAlcell2
	PORT (
	    qs	: IN std_logic;	-- Async Set on FF(Q).

	    a1	: IN std_logic;	-- Non-invert to AND A.
	    a2	: IN std_logic;	-- Invert to AND A.
	    a3	: IN std_logic;	-- Non-invert to AND A.
	    a4	: IN std_logic;	-- Invert to AND A.
	    a5	: IN std_logic;	-- Non-invert to AND A.
	    a6	: IN std_logic;	-- Invert to AND A.

	    os	: IN std_logic;	-- Data input MUX O.
	    op	: IN std_logic;	-- Select input MUX O.

	    b1	: IN std_logic;	-- Non-invert to AND B.
	    b2	: IN std_logic;	-- Invert to AND B.

	    c1	: IN std_logic;	-- Non-invert to AND C.
	    c2	: IN std_logic;	-- Invert to AND C.

	    mp	: IN std_logic;	-- Select input MUX M.
	    ms	: IN std_logic;	-- Data input MUX M.

	    d1	: IN std_logic;	-- Non-invert to AND D.
	    d2	: IN std_logic;	-- Invert to AND D.

	    e1	: IN std_logic;	-- Non-invert to AND E.
	    e2	: IN std_logic;	-- Invert to AND E.

	    np	: IN std_logic;	-- Select input MUX N.
	    ns	: IN std_logic;	-- Data input MUX N.

	    f1	: IN std_logic;	-- Non-invert to AND F.
	    f2	: IN std_logic;	-- Invert to AND F.
	    f3	: IN std_logic;	-- Non-invert to AND F.
	    f4	: IN std_logic;	-- Invert to AND F.
	    f5	: IN std_logic;	-- Non-invert to AND F.
	    f6	: IN std_logic;	-- Invert to AND F.

	    qc	: IN std_logic;	-- Clock to FF(Q).
	    qr	: IN std_logic;	-- Async Reset on FF(Q).

	    az	: OUT std_logic;	-- Output of AND A.
	    oz	: OUT std_logic;	-- Output of MUX O.
	    qz	: OUT std_logic;	-- Output of FF  Q.
	    nz	: OUT std_logic;	-- Output of MUX N.
	    fz	: OUT std_logic	-- Output of AND F.
	);
    END COMPONENT;
    ATTRIBUTE atomic_rtl	OF PAlcell2 : COMPONENT IS PAlcell_type;
    
COMPONENT ppalcell
	PORT (
	    qs	: IN std_logic;	-- Async Set on FF(Q).

	    a1	: IN std_logic;	-- Non-invert to AND A.
	    a2	: IN std_logic;	-- Invert to AND A.
	    a3	: IN std_logic;	-- Non-invert to AND A.
	    a4	: IN std_logic;	-- Invert to AND A.
	    a5	: IN std_logic;	-- Non-invert to AND A.
	    a6	: IN std_logic;	-- Invert to AND A.

	    b1	: IN std_logic;	-- Non-invert to AND B.
	    b2	: IN std_logic;	-- Invert to AND B.

	    c1	: IN std_logic;	-- Non-invert to AND C.
	    c2	: IN std_logic;	-- Invert to AND C.

	    d1	: IN std_logic;	-- Non-invert to AND D.
	    d2	: IN std_logic;	-- Invert to AND D.

	    e1	: IN std_logic;	-- Non-invert to AND E.
	    e2	: IN std_logic;	-- Invert to AND E.

	    f1	: IN std_logic;	-- Non-invert to AND F.
	    f2	: IN std_logic;	-- Invert to AND F.
	    f3	: IN std_logic;	-- Non-invert to AND F.
	    f4	: IN std_logic;	-- Invert to AND F.
	    f5	: IN std_logic;	-- Non-invert to AND F.
	    f6	: IN std_logic;	-- Invert to AND F.

	    qc	: IN std_logic;	-- Clock to FF(Q).
	    qr	: IN std_logic;	-- Async Reset on FF(Q).

	    az	: OUT std_logic;	-- Output of AND A.
	    oz	: OUT std_logic;	-- Output of MUX O.
	    qz	: OUT std_logic;	-- Output of FF  Q.
	    nz	: OUT std_logic;	-- Output of MUX N.
	    fz	: OUT std_logic	-- Output of AND F.
	);
    END COMPONENT;

    ATTRIBUTE dont_touch of ppalcell:component is true ;
    ATTRIBUTE atomic_rtl	OF ppalcell : COMPONENT IS PAlcell_type;
    
    COMPONENT PAlogic		-- Master Logic Cell.
	PORT (
	    qs	: IN std_logic;	-- Async Set on FF(Q).

	    a1	: IN std_logic;	-- Non-invert to AND A.
	    a2	: IN std_logic;	-- Invert to AND A.
	    a3	: IN std_logic;	-- Non-invert to AND A.
	    a4	: IN std_logic;	-- Invert to AND A.
	    a5	: IN std_logic;	-- Non-invert to AND A.
	    a6	: IN std_logic;	-- Invert to AND A.

	    b1	: IN std_logic;	-- Non-invert to AND B.
	    b2	: IN std_logic;	-- Invert to AND B.

	    c1	: IN std_logic;	-- Non-invert to AND C.
	    c2	: IN std_logic;	-- Invert to AND C.

	    d1	: IN std_logic;	-- Non-invert to AND D.
	    d2	: IN std_logic;	-- Invert to AND D.

	    e1	: IN std_logic;	-- Non-invert to AND E.
	    e2	: IN std_logic;	-- Invert to AND E.

	    f1	: IN std_logic;	-- Non-invert to AND F.
	    f2	: IN std_logic;	-- Invert to AND F.
	    f3	: IN std_logic;	-- Non-invert to AND F.
	    f4	: IN std_logic;	-- Invert to AND F.
	    f5	: IN std_logic;	-- Non-invert to AND F.
	    f6	: IN std_logic;	-- Invert to AND F.

	    qc	: IN std_logic;	-- Clock to FF(Q).
	    qr	: IN std_logic;	-- Async Reset on FF(Q).

	    az	: OUT std_logic;	-- Output of AND A.
	    oz	: OUT std_logic;	-- Output of MUX O.
	    qz	: OUT std_logic;	-- Output of FF  Q.
	    nz	: OUT std_logic;	-- Output of MUX N.
	    fz	: OUT std_logic	-- Output of AND F.
	);
    END COMPONENT;
    COMPONENT logico		-- Master Logic Cell.
	PORT (
	    a1	: IN std_logic;	-- Non-invert to AND A.
	    a2	: IN std_logic;	-- Invert to AND A.
	    a3	: IN std_logic;	-- Non-invert to AND A.
	    a4	: IN std_logic;	-- Invert to AND A.
	    a5	: IN std_logic;	-- Non-invert to AND A.
	    a6	: IN std_logic;	-- Invert to AND A.

	    b1	: IN std_logic;	-- Non-invert to AND B.
	    b2	: IN std_logic;	-- Invert to AND B.

	    c1	: IN std_logic;	-- Non-invert to AND C.
	    c2	: IN std_logic;	-- Invert to AND C.

	    d1	: IN std_logic;	-- Non-invert to AND D.
	    d2	: IN std_logic;	-- Invert to AND D.

	    e1	: IN std_logic;	-- Non-invert to AND E.
	    e2	: IN std_logic;	-- Invert to AND E.

	    f1	: IN std_logic;	-- Non-invert to AND F.
	    f2	: IN std_logic;	-- Invert to AND F.
	    f3	: IN std_logic;	-- Non-invert to AND F.
	    f4	: IN std_logic;	-- Invert to AND F.
	    f5	: IN std_logic;	-- Non-invert to AND F.
	    f6	: IN std_logic;	-- Invert to AND F.

	    oz	: OUT std_logic	-- Output of MUX O.
	);
    END COMPONENT;
    ATTRIBUTE atomic_rtl	OF logico : COMPONENT IS PAmacro_type;

--------------------------------------------------------------------------
--		Components for Generic RTL.				--
--------------------------------------------------------------------------

component bufoe
    port(
	x	: in std_logic;		-- Logic input to Buffer.
	oe	: in std_logic;		-- Output Enable input.
	y	: inout std_logic;		-- Output pin.
	yfb	: out std_logic		-- Output feed back as std_logic.
    );
end component;
component triout
    port(
	x	: in std_logic;		-- Logic input to Buffer.
	oe	: in std_logic;		-- Output Enable input.
	y	: out std_logic		-- Output pin.
    );
end component;
component dltch
    port(
	d	: in std_logic;		-- D input to latch.
	e	: in std_logic;		-- Enabel, tranparent high.
	q	: out std_logic		-- Qutput from latch.
    );
end component;
component dff
    port(
	d	: in std_logic;		-- D input to register.
	clk	: in std_logic;		-- Positive edge Clock.
	q	: out std_logic		-- Output from register.
    );
end component;
component xdff
    port(
	x1	: in std_logic;		-- First Exclusive OR input.
	x2	: in std_logic;		-- Second Exclusive OR input.
	clk	: in std_logic;		-- Positive edge Clock.
	q	: out std_logic		-- Qutput from register.
    );
end component;
component jkff
    port(
	j	: in std_logic;		-- J input to J-K register.
	k	: in std_logic;		-- K input to J-K register.
	clk	: in std_logic;		-- Positive edge Clock.
	q	: out std_logic		-- Qutput from register.
    );
end component;
component buf
    port(
	x	: in std_logic;		-- Input to Buffer.
	y	: out std_logic		-- Output from Buffer.
    );
end component;
component srlch
    port(
	s	: in std_logic;		-- Set input, Async.
	r	: in std_logic;		-- Reset input, Async.
	q	: out std_logic		-- Output from Latch.
    );
end component;
component srff
    port(
	s	: in std_logic;		-- Set input, Sync.
	r	: in std_logic;		-- Reset inut, Sync.
	clk	: in std_logic;		-- Positive edge Clock.
	q	: out std_logic		-- Output from register.
    );
end component;
component dsrff
    port(
	d	: in std_logic;		-- D input to register.
	s	: in std_logic;		-- Set input Async.
	r	: in std_logic;		-- Reset input Async.
	clk	: in std_logic;		-- Positive edge Clock.
	q	: out std_logic		-- Output from register.
    );
end component;
component tsrff
    port(
	t	: in std_logic;		-- T input to register.
	s	: in std_logic;		-- Set input Async.
	r	: in std_logic;		-- Reset input Async.
	clk	: in std_logic;		-- Positive edge Clock.
	q	: out std_logic		-- Output from register.
    );
end component;
component tff
    port(
	t	: in std_logic;		-- Togal input to T type register.
	clk	: in std_logic;		-- Positive edge Clock.
	q	: out std_logic		-- Output from register.
    );
end component;
component xbuf                          -- Xor primitive gate
    port(
        x1      : in std_logic;               -- Xor input.
        x2      : in std_logic;               -- Xor input.
        q       : out std_logic               -- Xor output.
    );
end component;
component dsrlch 
    port (
	d	: in std_logic;
	s	: in std_logic;
	r	: in std_logic;
	e	: in std_logic;
	q	: out std_logic
    );
end component;

END rtlpkg;

--------------------------------------------------------------------------
--		Enties for Generic RTL.					--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY bufoe IS			-- Tri-state inptu/output with
				--  input/feedback path to std_logic.
    PORT (
	x	: IN std_logic;		-- Logic input to Buffer.
	oe	: IN std_logic;		-- Output Enable input.
	y	: inout std_logic;		-- std_logic output pin.
	yfb	: OUT std_logic		-- Output feed back as std_logic.
    );
END bufoe;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY triout IS		-- Tri-state output with no feedback or
				--  input path.
    PORT (
	x	: IN std_logic;		-- Logic input to Buffer.
	oe	: IN std_logic;		-- Output Enable input.
	y	: OUT std_logic		-- std_logic output pin.
    );
END triout;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY dltch IS			-- Transparent D latch.
    PORT (
	d	: IN std_logic;		-- D input to latch.
	e	: IN std_logic;		-- Enabel, tranparent high.
	q	: OUT std_logic		-- Qutput from latch.
    );
END dltch;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY dff IS			-- D type flip flop. (only D and clk input)
    PORT (
	d	: IN std_logic;		-- D input to register.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Output from register.
    );
END dff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY xdff IS			-- X type flip flop. (X1, X2 and clk input)
    PORT (
	x1	: IN std_logic;		-- First Exclusive OR input.
	x2	: IN std_logic;		-- Second Exclusive OR input.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Qutput from register.
    );
END xdff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY jkff IS			-- JK type flip flop. ( J, K and clk input)
    PORT (
	j	: IN std_logic;		-- J input to J-K register.
	k	: IN std_logic;		-- K input to J-K register.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Qutput from register.
    );
END jkff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY buf IS			-- Buffer
    PORT (
	x	: IN std_logic;		-- Input to Buffer.
	y	: OUT std_logic		-- Output from Buffer.
    );
END buf;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY srlch IS			-- Set, Reset Latch. (S and R input only)
    PORT (
	s	: IN std_logic;		-- Set input, Async.
	r	: IN std_logic;		-- Reset input, Async.
	q	: OUT std_logic		-- Output from Latch.
    );
END srlch;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY srff IS			-- Synchronous Set Reset Flip Flop.
				--  (S, R, and CLK input)
    PORT (
	s	: IN std_logic;		-- Set input, Sync.
	r	: IN std_logic;		-- Reset inut, Sync.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Output from register.
    );
END srff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY dsrff IS			-- D type flip flop with
				--  asynchronous set and reset.
    PORT (
	d	: IN std_logic;		-- D input to register.
	s	: IN std_logic;		-- Set input Async.
	r	: IN std_logic;		-- Reset input Async.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Output from register.
    );
END dsrff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY tsrff IS	
    PORT (t	: IN std_logic;
	s	: IN std_logic;
	r	: IN std_logic;
	clk	: IN std_logic;
	q	: OUT std_logic);
END tsrff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY tff IS			-- T type flip flop. (T and clk input)
    PORT (
	t	: IN std_logic;		-- Togal input to T type register.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Output from register.
    );
END tff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY xbuf IS		-- Xor primitive gate
    PORT (
	x1	: IN std_logic;		-- Xor input.
	x2	: IN std_logic;		-- Xor input.
	q	: OUT std_logic		-- Xor output.
    );
END xbuf;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
entity DSRLCH is
    port(
        D  : IN std_logic;
        S  : IN std_logic;
	R  : IN std_logic;
        E  : IN std_logic;
	Q  : OUT std_logic);
end DSRLCH;

--------------------------------------------------------------------------
--		pASIC specific macro cell (rtl)				--
--------------------------------------------------------------------------

-- IO Master Cells:

library ieee;
use ieee.std_logic_1164.all;

use work.rtlpkg.all;
ENTITY PAlogic IS
    PORT (
	qs	: IN std_logic;	-- Async Set on FF(Q).

	a1	: IN std_logic;	-- Non-invert to AND A.
	a2	: IN std_logic;	-- Invert to AND A.
	a3	: IN std_logic;	-- Non-invert to AND A.
	a4	: IN std_logic;	-- Invert to AND A.
	a5	: IN std_logic;	-- Non-invert to AND A.
	a6	: IN std_logic;	-- Invert to AND A.

	b1	: IN std_logic;	-- Non-invert to AND B.
	b2	: IN std_logic;	-- Invert to AND B.

	c1	: IN std_logic;	-- Non-invert to AND C.
	c2	: IN std_logic;	-- Invert to AND C.

	d1	: IN std_logic;	-- Non-invert to AND D.
	d2	: IN std_logic;	-- Invert to AND D.

	e1	: IN std_logic;	-- Non-invert to AND E.
	e2	: IN std_logic;	-- Invert to AND E.

	f1	: IN std_logic;	-- Non-invert to AND F.
	f2	: IN std_logic;	-- Invert to AND F.
	f3	: IN std_logic;	-- Non-invert to AND F.
	f4	: IN std_logic;	-- Invert to AND F.
	f5	: IN std_logic;	-- Non-invert to AND F.
	f6	: IN std_logic;	-- Invert to AND F.

	qc	: IN std_logic;	-- Clock to FF(Q).
	qr	: IN std_logic;	-- Async Reset on FF(Q).

	az	: OUT std_logic;	-- Output of AND A.
	oz	: OUT std_logic;	-- Output of MUX O.
	qz	: OUT std_logic;	-- Output of FF  Q.
	nz	: OUT std_logic;	-- Output of MUX N.
	fz	: OUT std_logic	-- Output of AND F.
    );
END PAlogic;

--------------------------------------------------------------------------
--		Synthesis architecture for pASIC specific components	--
--------------------------------------------------------------------------

ARCHITECTURE archPAlogic OF PAlogic IS
    SIGNAL azx, qdx, fzx, nzx : std_logic;
BEGIN

    u1: PAfrag_a PORT MAP (a1, a2, a3, a4, a5, a6, azx);
    u2: PAfrag_f PORT MAP (f1, f2, f3, f4, f5, f6, fzx);
    u3: PAfrag_m PORT MAP (azx, fzx, b1, b2, c1, c2, d1, d2, e1, e2, qdx, nzx);
    u4: PAfrag_q PORT MAP (qs, qc, qr, qdx, qz);

    az	<= azx;
    oz	<= qdx;
    fz	<= fzx;
    nz	<= nzx;

END archPAlogic;
	    
--------------------------------------------------------------------------
--		Architectures for Generic RTL				--
--------------------------------------------------------------------------

ARCHITECTURE archdltch OF dltch IS
    SIGNAL qx : std_logic;
BEGIN
    qx	<= (d and e) or (d and qx) or (not(e) and qx);
    q	<= qx;
END archdltch;

ARCHITECTURE archxdff OF xdff IS
    signal dx : std_logic;
BEGIN
    dx	<= x1 xor x2;

    u1	: PAfrag_q
	PORT MAP (
	    qs	=> zero,
	    qc	=> clk,		-- Clock.
	    qr	=> zero,
	    qd	=> dx,		-- xor of inputs.
	    qz	=> q		-- Output.
	);
END archxdff;

ARCHITECTURE archTSRFF of tsrff IS
   signal dtmp,qtmp:std_logic;
BEGIN
	dtmp <= qtmp xor t;
	x:dsrff port map(dtmp,s,r,clk,qtmp);
	q <= qtmp;
END archTSRFF;

ARCHITECTURE archtff OF tff IS
    SIGNAL dx,qtmp : std_logic;
BEGIN
    dx	<= t xor qtmp;

    u1	: PAfrag_q
	PORT MAP (
	    qs	=> zero,
	    qc	=> clk,		-- Clock.
	    qr	=> zero,
	    qd	=> dx,		-- xor of input and q.
	    qz	=> qtmp);	-- Output.
    q <= qtmp;
END archtff;

ARCHITECTURE archjkff OF jkff IS
    SIGNAL dx,qtmp : std_logic;
BEGIN
    dx	<= (j and not(qtmp)) or (not(k) and qtmp);

    u1	: PAfrag_q
	PORT MAP (
	    qs	=> zero,
	    qc	=> clk,		-- Clock.
	    qr	=> zero,
	    qd	=> dx,		-- d from jk.
	    qz	=> qtmp);	-- Output.
    q <= qtmp;
END archjkff;
	
ARCHITECTURE archsrl OF srlch IS
    SIGNAL qx : std_logic;
BEGIN
    qx	<= (s and not(r)) or (qx and not(r));
    q	<= qx;
END archsrl;

ARCHITECTURE archsrff OF srff IS
    SIGNAL qx, dx : std_logic;
BEGIN
    dx  <= not ((not(s) or r) and (not (qx) or r));
    q	<= qx;

    u1	: PAfrag_q
	PORT MAP (
	    qs	=> zero,
	    qc	=> clk,		-- Clock.
	    qr	=> zero,
	    qd	=> dx,		-- d from sr.
	    qz	=> qx		-- Output.
	);
END archsrff;

ARCHITECTURE archdff OF dff IS
BEGIN
    u1	: PAfrag_q
	PORT MAP (
	    qs	=> zero,
	    qc	=> clk,		-- Clock.
	    qr	=> zero,
	    qd	=> d,		-- Data input.
	    qz	=> q		-- Output.
	);
END archdff;

ARCHITECTURE archbuf OF buf IS
BEGIN
    u1	: PAfrag_a
	PORT MAP (
	    a1	=> x,		-- Input.
	    a2	=> zero,
	    a3	=> one,
	    a4	=> zero,
	    a5	=> one,
	    a6	=> zero,

	    az	=> y		-- Output.
	);
END archbuf;

ARCHITECTURE archdsrff OF dsrff IS
BEGIN
    u1	: PAfrag_q
	PORT MAP (
	    qs	=> s,		-- Set (Async).
	    qc	=> clk,		-- Clock.
	    qr	=> r,		-- Reset (Async).
	    qd	=> d,		-- Data input.
	    qz	=> q		-- Output.
	);
END archdsrff;

ARCHITECTURE archbufoe OF bufoe IS
BEGIN
    u1	: PAbicell
	PORT MAP (
	    ie	=> oe,		-- Output Enable input.
	    i1	=> one,
	    i2	=> x,		-- Input from array.
	    iz	=> yfb,		-- Input, feedback path.
	    ip	=> y		-- Pin.
	);
END archbufoe;

--use work.rtlpkg.bufoe;
architecture archtriout of triout is
    signal ytmp : std_logic;
begin
    u1	: bufoe port map (x, oe, ytmp, open);
    y <= ytmp;
end archtriout;

ARCHITECTURE archxbuf OF xbuf IS
BEGIN
    q	<= x1 xor x2;
END archxbuf;
 

library ieee;
use ieee.std_logic_1164.all;

use work.rtlpkg.all;
entity xor2 is
    port(
        a,b     : in std_logic;
        q       : out std_logic
    );
end xor2;
 
architecture archXOR2 of xor2 is
begin
    u1 : xbuf
        port map (
            x1 => a,
            x2 => b,
            q => q
        );
end archXOR2;
 
architecture archDSRLCH of DSRLCH is

    signal qtmp : std_logic;
    attribute dont_touch of U1 : label is true;

begin

    U1:logico port map (one,s,one,zero,one,zero,
			one,r,one,r,qtmp,r,d,r,
			e,zero,one,zero,one,zero,qtmp);

    q <= qtmp;

end archDSRLCH;
