--
--      Copyright Cypress Semiconductor Corporation, 1995,1996
--	as an unpublished work.
--

--
--	MADD_SUB
--

library ieee;
use ieee.std_logic_1164.all;

package Madd_sub_local is
    type bvm		is array (integer range <>,integer range <>) of std_logic;
    type Shift_Type	is (LPM_LOGICAL, LPM_ROTATE, LPM_ARITHMETIC);
    type Repre_Type	is (LPM_SIGNED, LPM_UNSIGNED, LPM_GRAY);
    type Truth_Type	is (LPM_F, LPM_FD, LPM_FR, LPM_FDR);
    type CtDir_Type	is (LPM_NO_DIR, LPM_UP, LPM_DOWN);
    type Arith_Type	is (LPM_NO_TYP, LPM_ADD, LPM_SUB);
    type Fflop_Type	is (LPM_DFF,LPM_TFF);
    type ShDir_Type	is (LPM_LEFT,LPM_RIGHT);
    type goal_type	is (speed,area,combinatorial);
    type port_state_type is (ps_tied_low,ps_tied_high,ps_open,ps_connected);

    signal zero : std_logic := '0' ;
    signal one : std_logic := '1' ;

    function port_state (arg : std_logic) return port_state_type;
    function connects (x : std_logic; i : integer) return integer;
    function max (a,b : natural) return natural;
    function vectorize (a, size : natural) RETURN std_logic_vector;
    function restring (a : string; w : positive) return std_logic_vector;
COMPONENT ppalcell
	PORT (
	    qs	: IN std_logic;	-- Async Set on FF(Q).

	    a1	: IN std_logic;	-- Non-invert to AND A.
	    a2	: IN std_logic;	-- Invert to AND A.
	    a3	: IN std_logic;	-- Non-invert to AND A.
	    a4	: IN std_logic;	-- Invert to AND A.
	    a5	: IN std_logic;	-- Non-invert to AND A.
	    a6	: IN std_logic;	-- Invert to AND A.

	    b1	: IN std_logic;	-- Non-invert to AND B.
	    b2	: IN std_logic;	-- Invert to AND B.

	    c1	: IN std_logic;	-- Non-invert to AND C.
	    c2	: IN std_logic;	-- Invert to AND C.

	    d1	: IN std_logic;	-- Non-invert to AND D.
	    d2	: IN std_logic;	-- Invert to AND D.

	    e1	: IN std_logic;	-- Non-invert to AND E.
	    e2	: IN std_logic;	-- Invert to AND E.

	    f1	: IN std_logic;	-- Non-invert to AND F.
	    f2	: IN std_logic;	-- Invert to AND F.
	    f3	: IN std_logic;	-- Non-invert to AND F.
	    f4	: IN std_logic;	-- Invert to AND F.
	    f5	: IN std_logic;	-- Non-invert to AND F.
	    f6	: IN std_logic;	-- Invert to AND F.

	    qc	: IN std_logic;	-- Clock to FF(Q).
	    qr	: IN std_logic;	-- Async Reset on FF(Q).

	    az	: OUT std_logic;	-- Output of AND A.
	    oz	: OUT std_logic;	-- Output of MUX O.
	    qz	: OUT std_logic;	-- Output of FF  Q.
	    nz	: OUT std_logic;	-- Output of MUX N.
	    fz	: OUT std_logic	-- Output of AND F.
	);
    END COMPONENT;

    ATTRIBUTE dont_touch of ppalcell:component is true ;
    COMPONENT PAfrag_a
	PORT (
	    a1	: IN std_logic;	-- Non-invert to AND A.
	    a2	: IN std_logic;	-- Invert to AND A.
	    a3	: IN std_logic;	-- Non-invert to AND A.
	    a4	: IN std_logic;	-- Invert to AND A.
	    a5	: IN std_logic;	-- Non-invert to AND A.
	    a6	: IN std_logic;	-- Invert to AND A.

	    az	: OUT std_logic	-- Output of AND A.
	);
    END COMPONENT;
    COMPONENT PAfrag_f		-- Fragment AND F.
	PORT (
	    f1	: IN std_logic;	-- Non-invert to AND F.
	    f2	: IN std_logic;	-- Invert to AND F.
	    f3	: IN std_logic;	-- Non-invert to AND F.
	    f4	: IN std_logic;	-- Invert to AND F.
	    f5	: IN std_logic;	-- Non-invert to AND F.
	    f6	: IN std_logic;	-- Invert to AND F.

	    fz	: OUT std_logic	-- Output of AND F.
	);
    END COMPONENT;
    COMPONENT logico		-- Master Logic Cell.
	PORT (
	    a1	: IN std_logic;	-- Non-invert to AND A.
	    a2	: IN std_logic;	-- Invert to AND A.
	    a3	: IN std_logic;	-- Non-invert to AND A.
	    a4	: IN std_logic;	-- Invert to AND A.
	    a5	: IN std_logic;	-- Non-invert to AND A.
	    a6	: IN std_logic;	-- Invert to AND A.

	    b1	: IN std_logic;	-- Non-invert to AND B.
	    b2	: IN std_logic;	-- Invert to AND B.

	    c1	: IN std_logic;	-- Non-invert to AND C.
	    c2	: IN std_logic;	-- Invert to AND C.

	    d1	: IN std_logic;	-- Non-invert to AND D.
	    d2	: IN std_logic;	-- Invert to AND D.

	    e1	: IN std_logic;	-- Non-invert to AND E.
	    e2	: IN std_logic;	-- Invert to AND E.

	    f1	: IN std_logic;	-- Non-invert to AND F.
	    f2	: IN std_logic;	-- Invert to AND F.
	    f3	: IN std_logic;	-- Non-invert to AND F.
	    f4	: IN std_logic;	-- Invert to AND F.
	    f5	: IN std_logic;	-- Non-invert to AND F.
	    f6	: IN std_logic;	-- Invert to AND F.

	    oz	: OUT std_logic	-- Output of MUX O.
	);
    END COMPONENT;
    component as32
	generic(w		: positive;
		dir		: arith_type);
	port(	dataA		: in	std_logic_vector((w-1) downto 0);
	    	dataB		: in	std_logic_vector((w-1) downto 0);
	    	cin		: in	std_logic;
	    	add_sub		: in	std_logic;
	    	result		: out	std_logic_vector((w-1) downto 0);
	    	cout		: out	std_logic;
		overflow	: out   std_logic);
    end component;
    component add2wc
	port(a0,b0,a1,b1,ci	: in    std_logic;
		 s0,s1,c0,c1	: inout std_logic);
    end component;
    component sub2wc
	port(a0,b0,a1,b1,ci	: in    std_logic;
		 s0,s1,c0,c1	: inout std_logic);
    end component;
end Madd_sub_local ;

package body Madd_sub_local is
    function port_state (arg : std_logic) return port_state_type is
	variable result : port_state_type := ps_connected;
    begin

	result := arg'port_state_att;

	return result;
    end port_state;

    function connects (x : std_logic; i : integer) return integer is
	variable result : integer := i;
    begin
	if (port_state(x) = ps_connected) then
	    result := result - 1;
	end if;
	return (result);
    end connects;

    function max (a,b : natural) return natural is
    begin
	if (a < b) then return b; else return a; end if;
    end max;

end Madd_sub_local ;

--
--	Add Sub 32
--

library ieee;
use ieee.std_logic_1164.all;

use work.Madd_sub_local.all;

entity as32 is
    generic(	w		: positive;
		dir		: arith_type := lpm_no_typ);
    port(	dataA		: in	std_logic_vector((w-1) downto 0);
	    	dataB		: in	std_logic_vector((w-1) downto 0);
	    	cin		: in	std_logic;
	    	add_sub		: in	std_logic;
	    	result		: out	std_logic_vector((w-1) downto 0);
	    	cout		: out	std_logic;
		overflow	: out   std_logic);
end as32;

architecture archlpmstd of as32 is

    signal a,b,s	: std_logic_vector(result'length downto 0);
    signal s0,s1	: std_logic_vector(result'range);
    signal ac0,ac1,sc0,sc1,ax1,sx1,as0,ss0,nc0,nc1	: std_logic;
    signal c1x,c3x,c7x,a1t				: std_logic;
    signal c0,c1,s2,c02,c12,s3,c3,c03,c13		: std_logic;
    signal c04,c14,s5,c05,c15,c06,c16,s7,c7,c07,c17	: std_logic;
    signal c08,c18,s9,c09,c19,c0a,c1a,sb,c0b,c1b	: std_logic;
    signal c0c,c1c,sd,c0d,c1d,se,c0e,c1e,sf,c0f,c1f	: std_logic;
    signal ya,yb,yc,yd,ye,yf,yg,yh,yi,yj,yk,yl,ym,yn,yo,yp	: std_logic;
    signal za,zb,zc,zd,ze,zf,zg,zh,zi,zj,zk,zl,zm,zn,zo,zp	: std_logic;
    signal zex,zfx,zix,zjx					: std_logic;
    signal uv,uw,ub					: std_logic;
    signal yq,zq,yr,zr,cf1,cf2,cf3,cf4	: std_logic;
    signal s16,c016,c116,y16,z16,v16,w16,z16x	: std_logic;
    signal s17,c017,c117,y17,z17,v17,w17,z17x	: std_logic;
    signal s18,c018,c118,y18,z18	: std_logic;
    signal s19,c019,c119,y19,z19	: std_logic;
    signal s20,c020,c120,y20,z20,v20,w20,z20x	: std_logic;
    signal s21,c021,c121,y21,z21,v21,w21,z21x	: std_logic;
    signal s22,c022,c122,y22,z22	: std_logic;
    signal s23,c023,c123,y23,z23	: std_logic;
    signal s24,c024,c124,y24,z24,v24,w24,z24x,z24y	: std_logic;
    signal s25,c025,c125,y25,z25,v25,w25,z25x,z25y	: std_logic;
    signal q25,q26,q28,q29,q30,q31,q32,q33,q34,q35,q36,q37	: std_logic;
    signal s26,c026,c126,y26,z26	: std_logic;
    signal s27,c027,c127,y27,z27	: std_logic;
    signal s28,c028,c128,y28,z28,v28,w28	: std_logic;
    signal s29,c029,c129,y29,z29,v29,w29	: std_logic;
    signal s30,c030,c130,y30,z30	: std_logic;
    signal s31,y31,z31	: std_logic;

begin

    a(result'high downto 0) <= dataa;
    G00: if NOT((port_state(add_sub) = ps_connected)
			    AND (dir = lpm_no_typ)) generate
	b(result'high downto 0) <= datab;
    end generate;

    G00B: if ((port_state(add_sub) = ps_connected)
			  AND (dir = lpm_no_typ)) generate
	b(result'high downto 0) <= datab when (add_sub = '1') else NOT datab;
    end generate;

    MAKE_EVEN: if ((result'length rem 2) /= 0) generate
	a(result'length) <= '0';
	b(result'length) <= '0';
    end generate;

    -- ADD and ADD/SUBTRACT

    G01:if ((((port_state(add_sub) = ps_tied_high)
			  AND (dir = lpm_no_typ))
			   OR (dir = lpm_add))
	  OR ((port_state(add_sub) = ps_connected)
			  AND (dir = lpm_no_typ))) generate
	GC:if (result'length >= 1) generate
	    GCI:if NOT(port_state(cin) = ps_tied_low) generate
		X0C: logico port map(one,a(0),one,zero,one,zero,
				    one,zero,cin,zero,cin,zero,zero,one,
				    one,b(0),one,zero,one,zero,c0);
		X1: ppalcell port map(zero, 
				    c0,zero,one,zero,one,zero,
				    zero,one,b(1),zero,b(1),zero,one,zero,
				    a(1),zero,one,zero,one,zero,
				    zero,zero,open,c1,open,open,a1t);
		X0S: logico port map(one,a(0),one,zero,one,zero,
				    cin,zero,one,cin,one,cin,cin,zero,
				    one ,b(0),one,zero,one,zero,s(0));
	    end generate;
	    GNC:if (port_state(cin) = ps_tied_low) generate
		AC: ppalcell port map(zero, 
				    a(0),zero,b(0),zero,one,zero,
				    zero,one,b(1),zero,b(1),zero,one,zero,
				    a(1),zero,one,zero,one,zero,
				    zero,zero,c0,c1,open,open,a1t);
		AS: logico port map(one,a(0),one,b(0),one,zero,
				    one,zero,zero,one,zero,one,zero,one,
				    c0,zero,one,zero,one,zero,s(0));
	    end generate;
	end generate;
	G1:if (result'length >= 2) generate
	    S1: logico port map(b(1),zero,one,zero,one,zero,
				c0,zero,one,c0,one,c0,c0,zero,
				a1t,zero,one,zero,one,zero,s(1));
	end generate;
	G2:if (result'length >= 3) generate
	    B1: pafrag_a port map (c1,zero,one,zero,one,zero,c1x);
	    X2: ppalcell port map(zero,
				a(2),zero,b(2),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(2),one,b(2),one,zero,
				zero,zero,c02,s2,open,open,c12);
	    GS2: logico port map(c1x,zero,one,zero,one,zero,
				zero,one,one,zero,one,zero,zero,one,
				s2,zero,one,zero,one,zero,s(2));
	end generate;
	G3:if (result'length >= 4) generate
	    X3: ppalcell port map(zero,
				a(3),zero,b(3),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(3),one,b(3),one,zero,
				zero,zero,c03,s3,open,open,c13);
	    GS3: logico port map(c1x,zero,one,zero,one,zero,
				c02,zero,one,c02,one,c12,c12,zero,
				s3,zero,one,zero,one,zero,s(3));
	end generate;
	G4:if (result'length >= 5) generate
	    GC3: ppalcell port map(zero,
				c02,zero,one,zero,one,zero,
				c03,zero,one,c13,one,c13,one,c13,
				c1,c12,one,zero,one,zero,
				zero,zero,open,c3,open,open,open);
	    B3: pafrag_f port map (c3,zero,one,zero,one,zero,c3x);
	    X5: ppalcell port map(zero,
				a(5),zero,b(5),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(5),one,b(5),one,zero,
				zero,zero,c05,s5,open,open,c15);
	    MA: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				c05,zero,one,c15,c05,zero,one,c15,
				a(4),zero,b(4),zero,one,zero,
				zero,zero,open,ya,open,za,c04);
	    MB: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,c15,c05,zero,one,c15,c05,zero,
				one,zero,one,a(4),one,b(4),
				zero,zero,open,yb,open,zb,c14);
	    GS4: logico port map(one,zero,one,c04,one,c14,
				zero,one,one,zero,one,zero,zero,one,
				c3x,zero,one,zero,one,zero,s(4));
	end generate;
	G5:if (result'length >= 6) generate
	    GS5: logico port map(c3x,zero,one,zero,one,zero,
				c04,zero,one,c04,one,c14,c14,zero,
				s5,zero,one,zero,one,zero,s(5));
	end generate;
	G6:if (result'length >= 7) generate
	    X7: ppalcell port map(zero,
				a(7),zero,b(7),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(7),one,b(7),one,zero,
				zero,zero,C07,S7,open,open,C17);
	    MC: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s7,zero,one,s7,c07,zero,one,c17,
				a(6),zero,b(6),zero,one,zero,
				zero,zero,open,yc,open,zc,c06);
	    MD: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,s7,s7,zero,one,c17,c07,zero,
				one,zero,one,a(6),one,b(6),
				zero,zero,open,yd,open,zd,c16);
	    GS6: logico port map(c3x,zero,one,zero,one,zero,
				ya,zero,one,ya,yb,zero,one,yb,
				one,C06,one,C16,one,zero,s(6));
	end generate;
	G7:if (result'length >= 8) generate
	    GS7: logico port map(ya,zero,one,zero,one,zero,
				yc,zero,yd,zero,yd,zero,yd,zero,
				c3x,zero,yb,zero,one,zero,s(7));
	end generate;
	G8:if (result'length >= 9) generate
	    GC7: ppalcell port map(zero,
				za,zero,one,zero,one,zero,
				zc,zero,zd,zero,zd,zero,zd,zero,
				c3,zero,zb,zero,one,zero,
				zero,zero,open,c7,open,open,open);
	    GD7: ppalcell port map(zero,
				za,zero,one,zero,one,zero,
				zc,zero,zd,zero,zd,zero,zd,zero,
				c3,zero,zb,zero,one,zero,
				zero,zero,open,c7x,open,open,open);
	    X9: ppalcell port map(zero,
				a(9),zero,b(9),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(9),one,b(9),one,zero,
				zero,zero,C09,S9,open,open,C19);
	    ME: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s9,zero,one,s9,c09,zero,one,c19,
				a(8),zero,b(8),zero,one,zero,
				zero,zero,open,ye,open,ze,c08);
	    MF: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,s9,s9,zero,one,c19,c09,zero,
				one,zero,one,a(8),one,b(8),
				zero,zero,open,yf,open,zf,c18);
	    GS8: logico port map(one,zero,one,C08,one,C18,
				zero,one,one,zero,one,zero,zero,one,
				c7,zero,one,zero,one,zero,s(8));
	end generate;
	G9:if (result'length >= 10) generate
	    GS9: logico port map(one,zero,one,zero,one,zero,
				one,zero,one,zero,ye,zero,yf,zero,
				c7,zero,one,zero,one,zero,s(9));
	end generate;
	G10:if (result'length >= 11) generate
	    MN: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				zg,zero,zh,zero,zg,zero,zh,zero,
				ze,zero,one,zero,one,zero,
				zero,zero,open,yn,open,zn,zex);
	    MO: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				zg,zero,zh,zero,zg,zero,zh,zero,
				zf,zero,one,zero,one,zero,
				zero,zero,open,yo,open,zo,zfx);
	    XB: ppalcell port map(zero,
				a(11),zero,b(11),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(11),one,b(11),one,zero,
				zero,zero,C0B,SB,open,open,C1B);
	    MG: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				sb,zero,one,sb,c0b,zero,one,c1b,
				a(10),zero,b(10),zero,one,zero,
				zero,zero,open,yg,open,zg,c0a);
	    MH: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,sb,sb,zero,one,c1b,c0b,zero,
				one,zero,one,a(10),one,b(10),
				zero,zero,open,yh,open,zh,c1a);
	    GSA: logico port map(c7,zero,one,zero,one,zero,
				zex,zero,one,zex,zfx,zero,one,zfx,
				one,C0A,one,C1A,one,zero,s(10));
	end generate;
	G11:if (result'length >= 12) generate
	    GSB: logico port map(zex,zero,one,zero,one,zero,
				yg,zero,yh,zero,yh,zero,yh,zero,
				c7,zero,zfx,zero,one,zero,s(11));
	end generate;
	G12:if (result'length >= 13) generate
	    XD: ppalcell port map(zero,
				a(13),zero,b(13),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(13),one,b(13),one,zero,
				zero,zero,C0D,SD,open,open,C1D);
	    MI: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				sd,zero,one,sd,c0d,zero,one,c1d,
				a(12),zero,b(12),zero,one,zero,
				zero,zero,open,yi,open,zi,c0c);
	    MJ: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,sd,sd,zero,one,c1d,c0d,zero,
				one,zero,one,a(12),one,b(12),
				zero,zero,open,yj,open,zj,c1c);
	    GSC: logico port map(c7,zero,one,zero,one,zero,
				zn,zero,one,zn,zo,zero,one,zo,
				one,C0C,one,C1C,one,zero,s(12));
	end generate;
	G13:if (result'length >= 14) generate
	    GSD: logico port map(zn,zero,one,zero,one,zero,
				yi,zero,yj,zero,yj,zero,yj,zero,
				c7,zero,zo,zero,one,zero,s(13));
	end generate;
	G14:if (result'length >= 15) generate
	    MK: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				sf,zero,one,sf,c0f,zero,one,c1f,
				a(14),zero,b(14),zero,one,zero,
				zero,zero,open,yk,open,yp,open);
	    MP: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,sf,sf,zero,one,c1f,c0f,zero,
				one,zero,one,a(14),one,b(14),
				zero,zero,open,zk,open,zp,open);
	    MM: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				se,zero,one,se,yk,zero,zk,zero,
				zi,zero,one,zero,one,zero,
				zero,zero,open,ym,open,zm,zix);
	    ML: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				se,zero,one,se,yk,zero,zk,zero,
				zj,zero,one,zero,one,zero,
				zero,zero,open,yl,open,zl,zjx);
	    XE: ppalcell port map(zero,
				a(14),zero,b(14),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(14),one,b(14),one,zero,
				zero,zero,C0E,SE,open,open,C1E);
	    XF: ppalcell port map(zero,
				a(15),zero,b(15),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(15),one,b(15),one,zero,
				zero,zero,C0F,SF,open,open,C1F);
	    GSE: logico port map(zn,zero,one,zero,one,zero,
				ym,zero,yl,zero,yl,zero,yl,zero,
				c7x,zero,zo,zero,one,zero,s(14));
	end generate;
	G15:if (result'length >= 16) generate
	    GSF: logico port map(zn,zero,one,zero,one,zero,
				zm,zero,zl,zero,zl,zero,zl,zero,
				c7x,zero,zo,zero,one,zero,s(15));
	end generate;
	G16:if (result'length >= 17) generate
	    MQ: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				yp,zero,zp,zero,yp,zero,zp,zero,
				zi,zero,one,zero,one,zero,
				zero,zero,open,yq,open,zq,open);
	    MR: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				yp,zero,zp,zero,yp,zero,zp,zero,
				zj,zero,one,zero,one,zero,
				zero,zero,open,yr,open,zr,open);
	    GC1: logico port map(yn,zero,one,zero,one,zero,
				yq,zero,yr,zero,yr,zero,yr,zero,
				c7x,zero,yo,zero,one,zero,cf1);
	    X17: ppalcell port map(zero,
				a(17),zero,b(17),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(17),one,b(17),one,zero,
				zero,zero,C017,S17,open,open,C117);
	    M16: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s17,zero,one,s17,c017,zero,one,c117,
				a(16),zero,b(16),zero,one,zero,
				zero,zero,open,y16,open,z16,c016);
	    M17: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,s17,s17,zero,one,c117,c017,zero,
				one,zero,one,a(16),one,b(16),
				zero,zero,open,y17,open,z17,c116);
	    GS16: logico port map(one,zero,one,C016,one,C116,
				zero,one,one,zero,one,zero,zero,one,
				cf1,zero,one,zero,one,zero,s(16));
	end generate;
	G17:if (result'length >= 18) generate
	    GS17: logico port map(one,zero,one,zero,one,zero,
				one,zero,one,zero,y16,zero,y17,zero,
				cf1,zero,one,zero,one,zero,s(17));
	end generate;
	G18:if (result'length >= 19) generate
	    N16: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				z18,zero,z19,zero,z18,zero,z19,zero,
				z16,zero,one,zero,one,zero,
				zero,zero,open,v16,open,w16,z16x);
	    N17: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				z18,zero,z19,zero,z18,zero,z19,zero,
				z17,zero,one,zero,one,zero,
				zero,zero,open,v17,open,w17,z17x);
	    X19: ppalcell port map(zero,
				a(19),zero,b(19),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(19),one,b(19),one,zero,
				zero,zero,C019,s19,open,open,C119);
	    M18: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s19,zero,one,s19,c019,zero,one,c119,
				a(18),zero,b(18),zero,one,zero,
				zero,zero,open,y18,open,z18,c018);
	    M19: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,s19,s19,zero,one,c119,c019,zero,
				one,zero,one,a(18),one,b(18),
				zero,zero,open,y19,open,z19,c118);
	    GS18: logico port map(cf1,zero,one,zero,one,zero,
				z16x,zero,one,z16x,z17x,zero,one,z17x,
				one,C018,one,C118,one,zero,s(18));
	end generate;
	G19:if (result'length >= 20) generate
	    GS19: logico port map(z16x,zero,one,zero,one,zero,
				y18,zero,y19,zero,y19,zero,y19,zero,
				cf1,zero,z17x,zero,one,zero,s(19));
	end generate;
	G20:if (result'length >= 21) generate
	    GC2: logico port map(yn,zero,one,zero,one,zero,
				yq,zero,yr,zero,yr,zero,yr,zero,
				c7x,zero,yo,zero,one,zero,cf2);
	    X21: ppalcell port map(zero,
				a(21),zero,b(21),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(21),one,b(21),one,zero,
				zero,zero,C021,S21,open,open,C121);
	    M20: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s21,zero,one,s21,c021,zero,one,c121,
				a(20),zero,b(20),zero,one,zero,
				zero,zero,open,y20,open,z20,c020);
	    M21: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,s21,s21,zero,one,c121,c021,zero,
				one,zero,one,a(20),one,b(20),
				zero,zero,open,y21,open,z21,c120);
	    GS20: logico port map(cf2,zero,one,zero,one,zero,
				v16,zero,one,v16,v17,zero,one,v17,
				one,C020,one,C120,one,zero,s(20));
	end generate;
	G21:if (result'length >= 22) generate
	    GS21: logico port map(v16,zero,one,zero,one,zero,
				y20,zero,y21,zero,y21,zero,y21,zero,
				cf2,zero,v17,zero,one,zero,s(21));
	end generate;
	G22:if (result'length >= 23) generate
	    M22: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s23,zero,one,s23,c023,zero,one,c123,
				a(22),zero,b(22),zero,one,zero,
				zero,zero,open,y22,open,z22,open);
	    M23: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,s23,s23,zero,one,c123,c023,zero,
				one,zero,one,a(22),one,b(22),
				zero,zero,open,y23,open,z23,open);
	    N20: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s22,zero,one,s22,y22,zero,y23,zero,
				z20,zero,one,zero,one,zero,
				zero,zero,open,v20,open,w20,z20x);
	    N21: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s22,zero,one,s22,y22,zero,y23,zero,
				z21,zero,one,zero,one,zero,
				zero,zero,open,v21,open,w21,z21x);
	    X22: ppalcell port map(zero,
				a(22),zero,b(22),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(22),one,b(22),one,zero,
				zero,zero,C022,S22,open,open,C122);
	    X23: ppalcell port map(zero,
				a(23),zero,b(23),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(23),one,b(23),one,zero,
				zero,zero,c023,s23,open,open,c123);
	    GS22: logico port map(v16,zero,one,zero,one,zero,
				v20,zero,v21,zero,v21,zero,v21,zero,
				cf2,zero,v17,zero,one,zero,s(22));
	end generate;
	G23:if (result'length >= 24) generate
	    GS23: logico port map(v16,zero,one,zero,one,zero,
				w20,zero,w21,zero,w21,zero,w21,zero,
				cf2,zero,v17,zero,one,zero,s(23));
	end generate;
	G24:if (result'length >= 25) generate
	    GC3: logico port map(yn,zero,one,zero,one,zero,
				zq,zero,zr,zero,zr,zero,zr,zero,
				c7x,zero,yo,zero,one,zero,cf3);
	    P25: logico port map(w16,zero,one,zero,one,zero,
				z22,zero,z23,zero,z23,zero,z23,zero,
				z20,zero,w17,zero,one,zero,q25);
	    P26: logico port map(w16,zero,one,zero,one,zero,
				z22,zero,z23,zero,z23,zero,z23,zero,
				z21,zero,w17,zero,one,zero,q26);
	    X25: ppalcell port map(zero,
				a(25),zero,b(25),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(25),one,b(25),one,zero,
				zero,zero,C025,S25,open,open,C125);
	    M24: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s25,zero,one,s25,c025,zero,one,c125,
				a(24),zero,b(24),zero,one,zero,
				zero,zero,open,y24,open,z24,c024);
	    M25: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,s25,s25,zero,one,c125,c025,zero,
				one,zero,one,a(24),one,b(24),
				zero,zero,open,y25,open,z25,c124);
	    GS24: logico port map(cf3,zero,one,zero,one,zero,
				q25,zero,one,q25,q26,zero,one,q26,
				one,C024,one,C124,one,zero,s(24));
	end generate;
	G25:if (result'length >= 26) generate
	    GS27: logico port map(q25,zero,one,zero,one,zero,
				y24,zero,y25,zero,y25,zero,y25,zero,
				cf3,zero,q26,zero,one,zero,s(25));
	end generate;
	G26:if (result'length >= 27) generate
	    N24: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				y26,zero,y27,zero,z26,zero,z27,zero,
				z24,zero,one,zero,one,zero,
				zero,zero,open,v24,open,w24,z24x);
	    N25: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				y26,zero,y27,zero,z26,zero,z27,zero,
				z25,zero,one,zero,one,zero,
				zero,zero,open,v25,open,w25,z25x);
	    X27: ppalcell port map(zero,
				a(27),zero,b(27),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(27),one,b(27),one,zero,
				zero,zero,C027,s27,open,open,C127);
	    M26: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s27,zero,one,s27,c027,zero,one,c127,
				a(26),zero,b(26),zero,one,zero,
				zero,zero,open,y26,open,z26,c026);
	    M27: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,s27,s27,zero,one,c127,c027,zero,
				one,zero,one,a(26),one,b(26),
				zero,zero,open,y27,open,z27,c126);
	    P27: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				z24x,zero,one,z24x,z25x,zero,one,z25x,
				one,c026,one,c126,one,zero,
				zero,zero,open,z24y,open,z25y,open);
	    GS26: logico port map(q25,zero,one,zero,one,zero,
				z24y,zero,z25y,zero,z25y,zero,z25y,zero,
				cf3,zero,q26,zero,one,zero,s(26));
	end generate;
	G27:if (result'length >= 28) generate
	    GS27: logico port map(q25,zero,one,zero,one,zero,
				v24,zero,v25,zero,v25,zero,v25,zero,
				cf3,zero,q26,zero,one,zero,s(27));
	end generate;
	G28:if (result'length >= 29) generate
	    GC4: logico port map(yn,zero,one,zero,one,zero,
				zq,zero,zr,zero,zr,zero,zr,zero,
				c7x,zero,yo,zero,one,zero,cf4);
	    P28: logico port map(w16,zero,one,zero,one,zero,
				z22,zero,z23,zero,z23,zero,z23,zero,
				z20,zero,w17,zero,one,zero,q28);
	    P29: logico port map(w16,zero,one,zero,one,zero,
				z22,zero,z23,zero,z23,zero,z23,zero,
				z21,zero,w17,zero,one,zero,q29);
	    X28: ppalcell port map(zero,
				a(28),zero,b(28),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(28),one,b(28),one,zero,
				zero,zero,c028,s28,open,open,c128);
	    X29: ppalcell port map(zero,
				a(29),zero,b(29),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(29),one,b(29),one,zero,
				zero,zero,c029,s29,open,open,c129);
	    M28: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s29,zero,one,s29,c029,zero,one,c129,
				a(28),zero,b(28),zero,one,zero,
				zero,zero,open,y28,open,z28,open);
	    M29: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,s29,s29,zero,one,c129,c029,zero,
				one,zero,one,a(28),one,b(28),
				zero,zero,open,y29,open,z29,open);
	    P30: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				s28,zero,one,s28,y28,zero,y29,zero,
				w24,zero,one,zero,one,zero,
				zero,zero,open,q30,open,q31,open);
	    P32: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				s28,zero,one,s28,y28,zero,y29,zero,
				w25,zero,one,zero,one,zero,
				zero,zero,open,q32,open,q33,open);
	    GS28: logico port map(q28,zero,one,zero,one,zero,
				q30,zero,q32,zero,q32,zero,q32,zero,
				cf4,zero,q29,zero,one,zero,s(28));
	end generate;
	G29:if (result'length >= 30) generate
	    GS29: logico port map(q28,zero,one,zero,one,zero,
				q31,zero,q33,zero,q33,zero,q33,zero,
				cf4,zero,q29,zero,one,zero,s(29));
	end generate;
	G30:if (result'length >= 31) generate
	    X30: ppalcell port map(zero,
				a(30),zero,b(30),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(30),one,b(30),one,zero,
				zero,zero,c030,s30,open,open,c130);
	    X31: ppalcell port map(zero,
				a(31),zero,b(31),zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(31),one,b(31),one,zero,
				zero,zero,open,s31,open,open,open);
	    M30: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				c030,zero,one,c030,one,c130,c130,zero,
				s31,zero,one,zero,one,zero,
				zero,zero,open,y31,open,z31,open);
	    N28: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				s30,zero,one,s30,y31,zero,z31,zero,
				z28,zero,one,zero,one,zero,
				zero,zero,open,v28,open,w28,open);
	    N29: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				s30,zero,one,s30,y31,zero,z31,zero,
				z29,zero,one,zero,one,zero,
				zero,zero,open,v29,open,w29,open);
	    P34: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				v28,zero,v29,zero,w28,zero,w29,zero,
				w24,zero,one,zero,one,zero,
				zero,zero,open,q34,open,q35,open);
	    P36: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				v28,zero,v29,zero,w28,zero,w29,zero,
				w25,zero,one,zero,one,zero,
				zero,zero,open,q36,open,q37,open);
	    GS30: logico port map(q28,zero,one,zero,one,zero,
				q34,zero,q36,zero,q36,zero,q36,zero,
				cf4,zero,q29,zero,one,zero,s(30));
	end generate;
	G31:if (result'length = 32) generate
	    GS31: logico port map(q28,zero,one,zero,one,zero,
				q35,zero,q37,zero,q37,zero,q37,zero,
				cf4,zero,q29,zero,one,zero,s(31));
	end generate;
	GCO: logico port map(b(result'high),zero,one,zero,one,zero,
		    zero,one,a(result'high),zero,a(result'high),zero,one,zero,
		    one,s(result'high),one,zero,one,zero,cout);

	GOF: logico port map(one,a(result'high),one,b(result'high),
		    s(result'high),zero,  zero,one,one,zero,one,zero,one,zero,
		    a(result'high),zero,b(result'high),zero,one,s(result'high),
		    overflow);
    end generate;

    -- SUBTRACT

    G02:if (((port_state(add_sub) = ps_tied_low)
			 AND (dir = lpm_no_typ))
			  OR (dir = lpm_sub)) generate
	GC:if (result'length >= 1) generate
	    GNC:if (port_state(cin) = ps_tied_high) generate
		SC: ppalcell port map(zero, 
				    one,a(0),b(0),zero,one,zero,
				    one,b(1),one,zero,zero,one,one,b(1),
				    a(1),zero,one,zero,one,zero,
				    zero,zero,c0,c1,open,open,a1t);
		SS: logico port map(a(0),zero,one,b(0),one,zero,
				    one,zero,zero,one,one,zero,one,zero,
				    one,c0,one,zero,one,zero,s(0));
	    end generate;
	    GCI:if NOT(port_state(cin) = ps_tied_high) generate
		X0C: logico port map(one,a(0),one,zero,one,zero,
				    zero,one,one,cin,one,cin,one,zero,
				    b(0),zero,one,zero,one,zero,c0);
		X1: ppalcell port map(zero, 
				    one,c0,one,zero,one,zero,
				    zero,one,one,b(1),one,b(1),one,zero,
				    a(1),zero,one,zero,one,zero,
				    zero,zero,open,c1,open,open,a1t);
		X0S: logico port map(one,a(0),one,zero,one,zero,
				    cin,zero,one,cin,one,cin,cin,zero,
				    b(0),zero,one,zero,one,zero,s(0));
	    end generate;
	end generate;

	G1:if (result'length >= 2) generate
	    S1: logico port map(one,b(1),one,zero,one,zero,
				one,c0,c0,zero,c0,zero,one,c0,
				a1t,zero,one,zero,one,zero,s(1));
	end generate;
	G2:if (result'length >= 3) generate
	    B1: pafrag_a port map (c1,zero,one,zero,one,zero,c1x);
	    X2: ppalcell port map(zero,
				a(2),b(2),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(2),b(2),zero,one,zero,
				zero,zero,c02,s2,open,open,c12);
	    GS2: logico port map(c1x,zero,one,zero,one,zero,
				zero,one,one,zero,one,zero,zero,one,
				s2,zero,one,zero,one,zero,s(2));
	end generate;
	G3:if (result'length >= 4) generate
	    X3: ppalcell port map(zero,
				a(3),b(3),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(3),b(3),zero,one,zero,
				zero,zero,c03,s3,open,open,c13);
	    GS3: logico port map(c1x,zero,one,zero,one,zero,
				c02,zero,one,c02,one,c12,c12,zero,
				s3,zero,one,zero,one,zero,s(3));
	end generate;
	G4:if (result'length >= 5) generate
	    GC3: ppalcell port map(zero,
				c02,zero,one,zero,one,zero,
				c03,zero,one,c13,one,c13,one,c13,
				c1,c12,one,zero,one,zero,
				zero,zero,open,c3,open,open,open);
	    B3: pafrag_f port map (c3,zero,one,zero,one,zero,c3x);
	    X5: ppalcell port map(zero,
				a(5),b(5),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(5),b(5),zero,one,zero,
				zero,zero,c05,s5,open,open,c15);
	    MA: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				c05,zero,one,c15,c05,zero,one,c15,
				a(4),b(4),one,zero,one,zero,
				zero,zero,open,ya,open,za,c04);
	    MB: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,c15,c05,zero,one,c15,c05,zero,
				one,zero,one,a(4),b(4),zero,
				zero,zero,open,yb,open,zb,c14);
	    GS4: logico port map(one,zero,one,c04,one,c14,
				zero,one,one,zero,one,zero,zero,one,
				c3x,zero,one,zero,one,zero,s(4));
	end generate;
	G5:if (result'length >= 6) generate
	    GS5: logico port map(c3x,zero,one,zero,one,zero,
				c04,zero,one,c04,one,c14,c14,zero,
				s5,zero,one,zero,one,zero,s(5));
	end generate;
	G6:if (result'length >= 7) generate
	    X7: ppalcell port map(zero,
				a(7),b(7),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(7),b(7),zero,one,zero,
				zero,zero,C07,S7,open,open,C17);
	    MC: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s7,zero,one,s7,c07,zero,one,c17,
				a(6),b(6),one,zero,one,zero,
				zero,zero,open,yc,open,zc,c06);
	    MD: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,s7,s7,zero,one,c17,c07,zero,
				one,zero,one,a(6),b(6),zero,
				zero,zero,open,yd,open,zd,c16);
	    GS6: logico port map(c3x,zero,one,zero,one,zero,
				ya,zero,one,ya,yb,zero,one,yb,
				one,C06,one,C16,one,zero,s(6));
	end generate;
	G7:if (result'length >= 8) generate
	    GS7: logico port map(ya,zero,one,zero,one,zero,
				yc,zero,yd,zero,yd,zero,yd,zero,
				c3x,zero,yb,zero,one,zero,s(7));
	end generate;
	G8:if (result'length >= 9) generate
	    GC7: ppalcell port map(zero,
				za,zero,one,zero,one,zero,
				zc,zero,zd,zero,zd,zero,zd,zero,
				c3,zero,zb,zero,one,zero,
				zero,zero,open,c7,open,open,open);
	    GD7: ppalcell port map(zero,
				za,zero,one,zero,one,zero,
				zc,zero,zd,zero,zd,zero,zd,zero,
				c3,zero,zb,zero,one,zero,
				zero,zero,open,c7x,open,open,open);
	    X9: ppalcell port map(zero,
				a(9),b(9),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(9),b(9),zero,one,zero,
				zero,zero,C09,S9,open,open,C19);
	    ME: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s9,zero,one,s9,c09,zero,one,c19,
				a(8),b(8),one,zero,one,zero,
				zero,zero,open,ye,open,ze,c08);
	    MF: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,s9,s9,zero,one,c19,c09,zero,
				one,zero,one,a(8),b(8),zero,
				zero,zero,open,yf,open,zf,c18);
	    GS8: logico port map(one,zero,one,C08,one,C18,
				zero,one,one,zero,one,zero,zero,one,
				c7,zero,one,zero,one,zero,s(8));
	end generate;
	G9:if (result'length >= 10) generate
	    GS9: logico port map(one,zero,one,zero,one,zero,
				one,zero,one,zero,ye,zero,yf,zero,
				c7,zero,one,zero,one,zero,s(9));
	end generate;
	G10:if (result'length >= 11) generate
	    MN: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				zg,zero,zh,zero,zg,zero,zh,zero,
				ze,zero,one,zero,one,zero,
				zero,zero,open,yn,open,zn,zex);
	    MO: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				zg,zero,zh,zero,zg,zero,zh,zero,
				zf,zero,one,zero,one,zero,
				zero,zero,open,yo,open,zo,zfx);
	    XB: ppalcell port map(zero,
				a(11),b(11),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(11),b(11),zero,one,zero,
				zero,zero,C0B,SB,open,open,C1B);
	    MG: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				sb,zero,one,sb,c0b,zero,one,c1b,
				a(10),b(10),one,zero,one,zero,
				zero,zero,open,yg,open,zg,c0a);
	    MH: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,sb,sb,zero,one,c1b,c0b,zero,
				one,zero,one,a(10),b(10),zero,
				zero,zero,open,yh,open,zh,c1a);
	    GSA: logico port map(c7,zero,one,zero,one,zero,
				zex,zero,one,zex,zfx,zero,one,zfx,
				one,C0A,one,C1A,one,zero,s(10));
	end generate;
	G11:if (result'length >= 12) generate
	    GSB: logico port map(zex,zero,one,zero,one,zero,
				yg,zero,yh,zero,yh,zero,yh,zero,
				c7,zero,zfx,zero,one,zero,s(11));
	end generate;
	G12:if (result'length >= 13) generate
	    XD: ppalcell port map(zero,
				a(13),b(13),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(13),b(13),zero,one,zero,
				zero,zero,C0D,SD,open,open,C1D);
	    MI: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				sd,zero,one,sd,c0d,zero,one,c1d,
				a(12),b(12),one,zero,one,zero,
				zero,zero,open,yi,open,zi,c0c);
	    MJ: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,sd,sd,zero,one,c1d,c0d,zero,
				one,zero,one,a(12),b(12),zero,
				zero,zero,open,yj,open,zj,c1c);
	    GSC: logico port map(c7,zero,one,zero,one,zero,
				zn,zero,one,zn,zo,zero,one,zo,
				one,C0C,one,C1C,one,zero,s(12));
	end generate;
	G13:if (result'length >= 14) generate
	    GSD: logico port map(zn,zero,one,zero,one,zero,
				yi,zero,yj,zero,yj,zero,yj,zero,
				c7,zero,zo,zero,one,zero,s(13));
	end generate;
	G14:if (result'length >= 15) generate
	    MK: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				sf,zero,one,sf,c0f,zero,one,c1f,
				a(14),b(14),one,zero,one,zero,
				zero,zero,open,yk,open,yp,open);
	    MP: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,sf,sf,zero,one,c1f,c0f,zero,
				one,zero,one,a(14),b(14),zero,
				zero,zero,open,zk,open,zp,open);
	    MM: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				se,zero,one,se,yk,zero,zk,zero,
				zi,zero,one,zero,one,zero,
				zero,zero,open,ym,open,zm,zix);
	    ML: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				se,zero,one,se,yk,zero,zk,zero,
				zj,zero,one,zero,one,zero,
				zero,zero,open,yl,open,zl,zjx);
	    XE: ppalcell port map(zero,
				a(14),b(14),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(14),b(14),zero,one,zero,
				zero,zero,C0E,SE,open,open,C1E);
	    XF: ppalcell port map(zero,
				a(15),b(15),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(15),b(15),zero,one,zero,
				zero,zero,C0F,SF,open,open,C1F);
	    GSE: logico port map(zn,zero,one,zero,one,zero,
				ym,zero,yl,zero,yl,zero,yl,zero,
				c7x,zero,zo,zero,one,zero,s(14));
	end generate;
	G15:if (result'length >= 16) generate
	    GSF: logico port map(zn,zero,one,zero,one,zero,
				zm,zero,zl,zero,zl,zero,zl,zero,
				c7x,zero,zo,zero,one,zero,s(15));
	end generate;
	G16:if (result'length >= 17) generate
	    MQ: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				yp,zero,zp,zero,yp,zero,zp,zero,
				zi,zero,one,zero,one,zero,
				zero,zero,open,yq,open,zq,open);
	    MR: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				yp,zero,zp,zero,yp,zero,zp,zero,
				zj,zero,one,zero,one,zero,
				zero,zero,open,yr,open,zr,open);
	    GC1: logico port map(yn,zero,one,zero,one,zero,
				yq,zero,yr,zero,yr,zero,yr,zero,
				c7x,zero,yo,zero,one,zero,cf1);
	    X17: ppalcell port map(zero,
				a(17),b(17),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(17),b(17),zero,one,zero,
				zero,zero,C017,S17,open,open,C117);
	    M16: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s17,zero,one,s17,c017,zero,one,c117,
				a(16),b(16),one,zero,one,zero,
				zero,zero,open,y16,open,z16,c016);
	    M17: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,s17,s17,zero,one,c117,c017,zero,
				one,zero,one,a(16),b(16),zero,
				zero,zero,open,y17,open,z17,c116);
	    GS16: logico port map(one,zero,one,C016,one,C116,
				zero,one,one,zero,one,zero,zero,one,
				cf1,zero,one,zero,one,zero,s(16));
	end generate;
	G17:if (result'length >= 18) generate
	    GS17: logico port map(one,zero,one,zero,one,zero,
				one,zero,one,zero,y16,zero,y17,zero,
				cf1,zero,one,zero,one,zero,s(17));
	end generate;
	G18:if (result'length >= 19) generate
	    N16: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				z18,zero,z19,zero,z18,zero,z19,zero,
				z16,zero,one,zero,one,zero,
				zero,zero,open,v16,open,w16,z16x);
	    N17: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				z18,zero,z19,zero,z18,zero,z19,zero,
				z17,zero,one,zero,one,zero,
				zero,zero,open,v17,open,w17,z17x);
	    X19: ppalcell port map(zero,
				a(19),b(19),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(19),b(19),zero,one,zero,
				zero,zero,C019,s19,open,open,C119);
	    M18: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s19,zero,one,s19,c019,zero,one,c119,
				a(18),b(18),one,zero,one,zero,
				zero,zero,open,y18,open,z18,c018);
	    M19: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,s19,s19,zero,one,c119,c019,zero,
				one,zero,one,a(18),b(18),zero,
				zero,zero,open,y19,open,z19,c118);
	    GS18: logico port map(cf1,zero,one,zero,one,zero,
				z16x,zero,one,z16x,z17x,zero,one,z17x,
				one,C018,one,C118,one,zero,s(18));
	end generate;
	G19:if (result'length >= 20) generate
	    GS19: logico port map(z16x,zero,one,zero,one,zero,
				y18,zero,y19,zero,y19,zero,y19,zero,
				cf1,zero,z17x,zero,one,zero,s(19));
	end generate;
	G20:if (result'length >= 21) generate
	    GC2: logico port map(yn,zero,one,zero,one,zero,
				yq,zero,yr,zero,yr,zero,yr,zero,
				c7x,zero,yo,zero,one,zero,cf2);
	    X21: ppalcell port map(zero,
				a(21),b(21),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(21),b(21),zero,one,zero,
				zero,zero,C021,S21,open,open,C121);
	    M20: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s21,zero,one,s21,c021,zero,one,c121,
				a(20),b(20),one,zero,one,zero,
				zero,zero,open,y20,open,z20,c020);
	    M21: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,s21,s21,zero,one,c121,c021,zero,
				one,zero,one,a(20),b(20),zero,
				zero,zero,open,y21,open,z21,c120);
	    GS20: logico port map(cf2,zero,one,zero,one,zero,
				v16,zero,one,v16,v17,zero,one,v17,
				one,C020,one,C120,one,zero,s(20));
	end generate;
	G21:if (result'length >= 22) generate
	    GS21: logico port map(v16,zero,one,zero,one,zero,
				y20,zero,y21,zero,y21,zero,y21,zero,
				cf2,zero,v17,zero,one,zero,s(21));
	end generate;
	G22:if (result'length >= 23) generate
	    M22: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s23,zero,one,s23,c023,zero,one,c123,
				a(22),b(22),one,zero,one,zero,
				zero,zero,open,y22,open,z22,open);
	    M23: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,s23,s23,zero,one,c123,c023,zero,
				one,zero,one,a(22),b(22),zero,
				zero,zero,open,y23,open,z23,open);
	    N20: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s22,zero,one,s22,y22,zero,y23,zero,
				z20,zero,one,zero,one,zero,
				zero,zero,open,v20,open,w20,z20x);
	    N21: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s22,zero,one,s22,y22,zero,y23,zero,
				z21,zero,one,zero,one,zero,
				zero,zero,open,v21,open,w21,z21x);
	    X22: ppalcell port map(zero,
				a(22),b(22),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(22),b(22),zero,one,zero,
				zero,zero,C022,S22,open,open,C122);
	    X23: ppalcell port map(zero,
				a(23),b(23),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(23),b(23),zero,one,zero,
				zero,zero,c023,s23,open,open,c123);
	    GS22: logico port map(v16,zero,one,zero,one,zero,
				v20,zero,v21,zero,v21,zero,v21,zero,
				cf2,zero,v17,zero,one,zero,s(22));
	end generate;
	G23:if (result'length >= 24) generate
	    GS23: logico port map(v16,zero,one,zero,one,zero,
				w20,zero,w21,zero,w21,zero,w21,zero,
				cf2,zero,v17,zero,one,zero,s(23));
	end generate;
	G24:if (result'length >= 25) generate
	    GC3: logico port map(yn,zero,one,zero,one,zero,
				zq,zero,zr,zero,zr,zero,zr,zero,
				c7x,zero,yo,zero,one,zero,cf3);
	    P25: logico port map(w16,zero,one,zero,one,zero,
				z22,zero,z23,zero,z23,zero,z23,zero,
				z20,zero,w17,zero,one,zero,q25);
	    P26: logico port map(w16,zero,one,zero,one,zero,
				z22,zero,z23,zero,z23,zero,z23,zero,
				z21,zero,w17,zero,one,zero,q26);
	    X25: ppalcell port map(zero,
				a(25),b(25),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(25),b(25),zero,one,zero,
				zero,zero,C025,S25,open,open,C125);
	    M24: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s25,zero,one,s25,c025,zero,one,c125,
				a(24),b(24),one,zero,one,zero,
				zero,zero,open,y24,open,z24,c024);
	    M25: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,s25,s25,zero,one,c125,c025,zero,
				one,zero,one,a(24),b(24),zero,
				zero,zero,open,y25,open,z25,c124);
	    GS24: logico port map(cf3,zero,one,zero,one,zero,
				q25,zero,one,q25,q26,zero,one,q26,
				one,C024,one,C124,one,zero,s(24));
	end generate;
	G25:if (result'length >= 26) generate
	    GS27: logico port map(q25,zero,one,zero,one,zero,
				y24,zero,y25,zero,y25,zero,y25,zero,
				cf3,zero,q26,zero,one,zero,s(25));
	end generate;
	G26:if (result'length >= 27) generate
	    N24: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				y26,zero,y27,zero,z26,zero,z27,zero,
				z24,zero,one,zero,one,zero,
				zero,zero,open,v24,open,w24,z24x);
	    N25: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				y26,zero,y27,zero,z26,zero,z27,zero,
				z25,zero,one,zero,one,zero,
				zero,zero,open,v25,open,w25,z25x);
	    X27: ppalcell port map(zero,
				a(27),b(27),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(27),b(27),zero,one,zero,
				zero,zero,C027,s27,open,open,C127);
	    M26: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s27,zero,one,s27,c027,zero,one,c127,
				a(26),b(26),one,zero,one,zero,
				zero,zero,open,y26,open,z26,c026);
	    M27: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,s27,s27,zero,one,c127,c027,zero,
				one,zero,one,a(26),b(26),zero,
				zero,zero,open,y27,open,z27,c126);
	    P27: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				z24x,zero,one,z24x,z25x,zero,one,z25x,
				one,c026,one,c126,one,zero,
				zero,zero,open,z24y,open,z25y,open);
	    GS26: logico port map(q25,zero,one,zero,one,zero,
				z24y,zero,z25y,zero,z25y,zero,z25y,zero,
				cf3,zero,q26,zero,one,zero,s(26));
	end generate;
	G27:if (result'length >= 28) generate
	    GS27: logico port map(q25,zero,one,zero,one,zero,
				v24,zero,v25,zero,v25,zero,v25,zero,
				cf3,zero,q26,zero,one,zero,s(27));
	end generate;
	G28:if (result'length >= 29) generate
	    GC4: logico port map(yn,zero,one,zero,one,zero,
				zq,zero,zr,zero,zr,zero,zr,zero,
				c7x,zero,yo,zero,one,zero,cf4);
	    P28: logico port map(w16,zero,one,zero,one,zero,
				z22,zero,z23,zero,z23,zero,z23,zero,
				z20,zero,w17,zero,one,zero,q28);
	    P29: logico port map(w16,zero,one,zero,one,zero,
				z22,zero,z23,zero,z23,zero,z23,zero,
				z21,zero,w17,zero,one,zero,q29);
	    X28: ppalcell port map(zero,
				a(28),b(28),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(28),b(28),zero,one,zero,
				zero,zero,c028,s28,open,open,c128);
	    X29: ppalcell port map(zero,
				a(29),b(29),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(29),b(29),zero,one,zero,
				zero,zero,c029,s29,open,open,c129);
	    M28: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				s29,zero,one,s29,c029,zero,one,c129,
				a(28),b(28),one,zero,one,zero,
				zero,zero,open,y28,open,z28,open);
	    M29: ppalcell port map(zero,
				zero,zero,one,zero,one,zero,
				one,s29,s29,zero,one,c129,c029,zero,
				one,zero,one,a(28),b(28),zero,
				zero,zero,open,y29,open,z29,open);
	    P30: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				s28,zero,one,s28,y28,zero,y29,zero,
				w24,zero,one,zero,one,zero,
				zero,zero,open,q30,open,q31,open);
	    P32: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				s28,zero,one,s28,y28,zero,y29,zero,
				w25,zero,one,zero,one,zero,
				zero,zero,open,q32,open,q33,open);
	    GS28: logico port map(q28,zero,one,zero,one,zero,
				q30,zero,q32,zero,q32,zero,q32,zero,
				cf4,zero,q29,zero,one,zero,s(28));
	end generate;
	G29:if (result'length >= 30) generate
	    GS29: logico port map(q28,zero,one,zero,one,zero,
				q31,zero,q33,zero,q33,zero,q33,zero,
				cf4,zero,q29,zero,one,zero,s(29));
	end generate;
	G30:if (result'length >= 31) generate
	    X30: ppalcell port map(zero,
				a(30),b(30),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(30),b(30),zero,one,zero,
				zero,zero,c030,s30,open,open,c130);
	    X31: ppalcell port map(zero,
				a(31),b(31),one,zero,one,zero,
				one,zero,zero,one,zero,one,zero,one,
				one,a(31),b(31),zero,one,zero,
				zero,zero,open,s31,open,open,open);
	    M30: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				c030,zero,one,c030,one,c130,c130,zero,
				s31,zero,one,zero,one,zero,
				zero,zero,open,y31,open,z31,open);
	    N28: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				s30,zero,one,s30,y31,zero,z31,zero,
				z28,zero,one,zero,one,zero,
				zero,zero,open,v28,open,w28,open);
	    N29: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				s30,zero,one,s30,y31,zero,z31,zero,
				z29,zero,one,zero,one,zero,
				zero,zero,open,v29,open,w29,open);
	    P34: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				v28,zero,v29,zero,w28,zero,w29,zero,
				w24,zero,one,zero,one,zero,
				zero,zero,open,q34,open,q35,open);
	    P36: ppalcell port map(zero,
				zero,one,zero,one,zero,one,
				v28,zero,v29,zero,w28,zero,w29,zero,
				w25,zero,one,zero,one,zero,
				zero,zero,open,q36,open,q37,open);
	    GS30: logico port map(q28,zero,one,zero,one,zero,
				q34,zero,q36,zero,q36,zero,q36,zero,
				cf4,zero,q29,zero,one,zero,s(30));
	end generate;
	G31:if (result'length = 32) generate
	    GS31: logico port map(q28,zero,one,zero,one,zero,
				q35,zero,q37,zero,q37,zero,q37,zero,
				cf4,zero,q29,zero,one,zero,s(31));
	end generate;
	GCO: logico port map(b(result'high),zero,one,zero,one,zero,
		    one,zero,a(result'high),zero,a(result'high),zero,zero,one,
			s(result'high),zero,one,zero,one,zero,cout);

	GOF: logico port map(one,a(result'high),one,b(result'high),
		    s(result'high),zero,  zero,one,one,zero,one,zero,one,zero,
		    a(result'high),zero,b(result'high),zero,one,s(result'high),
		    overflow);
    end generate;

    result <= s(result'high downto 0);

end archlpmstd;

--
--	add2wc
--

library ieee;
use ieee.std_logic_1164.all;

use work.Madd_sub_local.all;

entity add2wc is
    port(a0,b0,a1,b1,ci	: in    std_logic;
	 s0,s1,c0,c1	: inout std_logic);
end add2wc;

architecture archlpmstd of add2wc is

    signal x0, x1, bn : std_logic;
    signal one  : std_logic := '1';
    signal zero : std_logic := '0';

begin

    U0: ppalcell port map (zero,
			ci,zero,one,zero,one,zero,
			one,x0,one,x0,one,a0,a0,zero,
			b0,zero,one,zero,one,zero,
			zero,zero,open,s0,open,x0,open);

    U1: ppalcell port map (zero,
			one,zero,one,zero,one,c0,
			one,x1,one,x1,one,a1,a1,zero,
			one,zero,one,zero,one,b1,
			zero,zero,open,s1,open,x1,bn);

    U2: ppalcell port map (zero,
			x1,zero,one,zero,one,zero,
			a1,bn,a1,bn,ci,x0,one,zero,
			a0,zero,b0,zero,one,zero,
			zero,zero,open,c1,open,c0,open);
end archlpmstd;

--
--	sub2wc
--

library ieee;
use ieee.std_logic_1164.all;

use work.Madd_sub_local.all;

entity sub2wc is
    port(a0,b0,a1,b1,ci	: in    std_logic;
	 s0,s1,c0,c1	: inout std_logic);
end sub2wc;

architecture archlpmstd of sub2wc is

    signal x0,x1,bn,cn : std_logic;
    signal one  : std_logic := '1';
    signal zero : std_logic := '0';

begin

    U0: ppalcell port map (zero,
			ci,zero,one,zero,one,zero,
			one,x0,one,x0,one,a0,a0,zero,
			one,zero,one,zero,one,b0,
			zero,zero,open,s0,open,x0,open);

    U1: ppalcell port map (zero,
			one,zero,one,zero,one,c0,
			one,x1,one,x1,one,a1,a1,zero,
			b1,zero,one,zero,one,zero,
			zero,zero,open,s1,open,x1,bn);

    U2: ppalcell port map (zero,
			x1,zero,one,zero,one,zero,
			a1,bn,a1,bn,ci,x0,one,zero,
			a0,zero,one,zero,one,b0,
			zero,zero,open,c1,open,c0,open);
end archlpmstd;

library ieee;
use ieee.std_logic_1164.all;

use work.Madd_sub_local.all ;

entity madd_sub is
    generic(lpm_width	: positive := 1;
	    lpm_representation	: repre_type;
	    lpm_direction	: arith_type;
	    lpm_hint	: goal_type);
    port(   dataA	: in	std_logic_vector((lpm_width-1) downto 0);
	    dataB	: in	std_logic_vector((lpm_width-1) downto 0);
	    cin		: in	std_logic;
	    add_sub	: in	std_logic;
	    result	: out	std_logic_vector((lpm_width-1) downto 0);
	    cout	: out	std_logic;
	    overflow	: out   std_logic);
end madd_sub;

architecture archlpmstd of madd_sub is

    constant L   : integer := ((lpm_width+1)/2) - 1;
    constant K   : integer := 32;
    signal c     : std_logic_vector ((lpm_width/K) downto 0);
    signal c1,c0 : std_logic_vector ((L)           downto 0);
    signal a,b,r : std_logic_vector ((lpm_width)   downto 0);
begin

    a(result'high downto 0) <= dataa;
    G00: if NOT((port_state(add_sub) = ps_connected)
			    AND (lpm_direction = lpm_no_typ)) generate
	b(result'high downto 0) <= datab;
    end generate;

    G00B: if ((port_state(add_sub) = ps_connected)
			  AND (lpm_direction = lpm_no_typ)) generate
	b(result'high downto 0) <= datab when (add_sub = '1') else NOT datab;
    end generate;

    a(result'length) <= '0';
    b(result'length) <= '0';

    GQA: if (lpm_hint = area) generate
	GA: for i in 0 to L generate
	    G0: if i  = 0 generate
		GA: if NOT((lpm_direction = lpm_sub) OR
			    ((port_state(add_sub) = ps_tied_low) AND
			    (lpm_direction = lpm_no_typ))) generate
		    UI:add2wc port map(a(i*2),b(i*2),a((i*2)+1),b((i*2)+1),
					cin,r(i*2),r((i*2)+1),c0(i),c1(i));
		end generate;
		GS: if    ((lpm_direction = lpm_sub) OR
			    ((port_state(add_sub) = ps_tied_low) AND
			    (lpm_direction = lpm_no_typ))) generate
		    UI:sub2wc port map(a(i*2),b(i*2),a((i*2)+1),b((i*2)+1),
					cin,r(i*2),r((i*2)+1),c0(i),c1(i));
		end generate;
	    end generate;
	    G1: if i /= 0 generate
		GA: if NOT((lpm_direction = lpm_sub) OR
			    ((port_state(add_sub) = ps_tied_low) AND
			    (lpm_direction = lpm_no_typ))) generate
		    UI:add2wc port map(a(i*2),b(i*2),a((i*2)+1),b((i*2)+1),
				    c1(i-1),r(i*2),r((i*2)+1),c0(i),c1(i));
		end generate;
		GS: if    ((lpm_direction = lpm_sub) OR
			    ((port_state(add_sub) = ps_tied_low) AND
			    (lpm_direction = lpm_no_typ))) generate
		    UI:sub2wc port map(a(i*2),b(i*2),a((i*2)+1),b((i*2)+1),
				    c1(i-1),r(i*2),r((i*2)+1),c0(i),c1(i));
		end generate;
	    end generate;
	end generate;

	result <= r(result'high downto 0);
	CO: if ((result'length rem 2) /= 0) generate
	    cout <= c0(c1'high);
	    CX:if (c1'high > 0) generate
		overflow <= c1(c1'high - 1) XOR c0(c1'high);
	    end generate;
	    CY:if (c1'high <= 0) generate
		overflow <= c0(c1'high);
	    end generate;
	end generate;
	CE: if ((result'length rem 2)  = 0) generate
	    cout <= c1(c1'high);
	    overflow <= c1(c1'high) XOR c0(c1'high);
	end generate;

    end generate;

    GQS: if (lpm_hint = speed) generate
	GS: if (result'length <= 32) generate
	    U0: as32 generic map (lpm_width,lpm_direction)
			port map (dataA,dataB,cin,add_sub,result,cout,overflow);
	end generate;
	GB: if (result'length >  32) generate
	    ASSERT (result'length <= K)
	    REPORT "Widths greater than 32 may result in reduced performance."
	    SEVERITY WARNING;

	    c(0) <= cin;
	    G0: for i in 0 to (result'high/K) generate
		G1: if (((i*K)+K) <  result'length) generate
		    U0: as32 generic map (K,lpm_direction)
			    port map (dataA(((i*K)+(K-1)) downto (i*K)),
			       dataB(((i*K)+(K-1)) downto (i*K)),
			       c(i), add_sub,
			       result(((i*K)+(K-1)) downto (i*K)),c(i+1));
		end generate;
		G2: if (((i*K)+K) >= result'length) generate
		    U0: as32 generic map (result'length-(i*K),lpm_direction)
			    port map (dataA((result'high) downto (i*K)),
			       dataB((result'high) downto (i*K)),
			       c(i),add_sub,result((result'high) downto (i*K)),
			       cout,overflow);
		end generate;
	    end generate;
	end generate;
    end generate;
end archlpmstd;
