--
--      Copyright Cypress Semiconductor Corporation, 1995,1996
--	as an unpublished work.
--

--
--	COMPare 2
--

library ieee;
use ieee.std_logic_1164.all;

package Mcompare_local is
    type bvm		is array (integer range <>,integer range <>) of std_logic;
    type Shift_Type	is (LPM_LOGICAL, LPM_ROTATE, LPM_ARITHMETIC);
    type Repre_Type	is (LPM_SIGNED, LPM_UNSIGNED, LPM_GRAY);
    type Truth_Type	is (LPM_F, LPM_FD, LPM_FR, LPM_FDR);
    type CtDir_Type	is (LPM_NO_DIR, LPM_UP, LPM_DOWN);
    type Arith_Type	is (LPM_NO_TYP, LPM_ADD, LPM_SUB);
    type Fflop_Type	is (LPM_DFF,LPM_TFF);
    type ShDir_Type	is (LPM_LEFT,LPM_RIGHT);
    type goal_type	is (speed,area,combinatorial);
    type port_state_type is (ps_tied_low,ps_tied_high,ps_open,ps_connected);

    signal zero : std_logic := '0' ;
    signal one : std_logic := '1' ;

COMPONENT ppalcell
	PORT (
	    qs	: IN std_logic;	-- Async Set on FF(Q).

	    a1	: IN std_logic;	-- Non-invert to AND A.
	    a2	: IN std_logic;	-- Invert to AND A.
	    a3	: IN std_logic;	-- Non-invert to AND A.
	    a4	: IN std_logic;	-- Invert to AND A.
	    a5	: IN std_logic;	-- Non-invert to AND A.
	    a6	: IN std_logic;	-- Invert to AND A.

	    b1	: IN std_logic;	-- Non-invert to AND B.
	    b2	: IN std_logic;	-- Invert to AND B.

	    c1	: IN std_logic;	-- Non-invert to AND C.
	    c2	: IN std_logic;	-- Invert to AND C.

	    d1	: IN std_logic;	-- Non-invert to AND D.
	    d2	: IN std_logic;	-- Invert to AND D.

	    e1	: IN std_logic;	-- Non-invert to AND E.
	    e2	: IN std_logic;	-- Invert to AND E.

	    f1	: IN std_logic;	-- Non-invert to AND F.
	    f2	: IN std_logic;	-- Invert to AND F.
	    f3	: IN std_logic;	-- Non-invert to AND F.
	    f4	: IN std_logic;	-- Invert to AND F.
	    f5	: IN std_logic;	-- Non-invert to AND F.
	    f6	: IN std_logic;	-- Invert to AND F.

	    qc	: IN std_logic;	-- Clock to FF(Q).
	    qr	: IN std_logic;	-- Async Reset on FF(Q).

	    az	: OUT std_logic;	-- Output of AND A.
	    oz	: OUT std_logic;	-- Output of MUX O.
	    qz	: OUT std_logic;	-- Output of FF  Q.
	    nz	: OUT std_logic;	-- Output of MUX N.
	    fz	: OUT std_logic	-- Output of AND F.
	);
    END COMPONENT;

    ATTRIBUTE dont_touch of ppalcell:component is true ;
    COMPONENT logico		-- Master Logic Cell.
	PORT (
	    a1	: IN std_logic;	-- Non-invert to AND A.
	    a2	: IN std_logic;	-- Invert to AND A.
	    a3	: IN std_logic;	-- Non-invert to AND A.
	    a4	: IN std_logic;	-- Invert to AND A.
	    a5	: IN std_logic;	-- Non-invert to AND A.
	    a6	: IN std_logic;	-- Invert to AND A.

	    b1	: IN std_logic;	-- Non-invert to AND B.
	    b2	: IN std_logic;	-- Invert to AND B.

	    c1	: IN std_logic;	-- Non-invert to AND C.
	    c2	: IN std_logic;	-- Invert to AND C.

	    d1	: IN std_logic;	-- Non-invert to AND D.
	    d2	: IN std_logic;	-- Invert to AND D.

	    e1	: IN std_logic;	-- Non-invert to AND E.
	    e2	: IN std_logic;	-- Invert to AND E.

	    f1	: IN std_logic;	-- Non-invert to AND F.
	    f2	: IN std_logic;	-- Invert to AND F.
	    f3	: IN std_logic;	-- Non-invert to AND F.
	    f4	: IN std_logic;	-- Invert to AND F.
	    f5	: IN std_logic;	-- Non-invert to AND F.
	    f6	: IN std_logic;	-- Invert to AND F.

	    oz	: OUT std_logic	-- Output of MUX O.
	);
    END COMPONENT;
function andvec ( x : std_logic_vector ) return std_logic ;
end Mcompare_local ;
 
package body Mcompare_local is
function andvec ( x : std_logic_vector ) return std_logic is
    variable result : std_logic ;
begin
    result := '1' ;
    for i in x'range loop
	result := result AND x(i) ;
    end loop ;
    return result ;
end andvec ;
end Mcompare_local ;

library ieee;
use ieee.std_logic_1164.all;

use work.Mcompare_local.all;

entity comp2 is
    PORT (A0,B0,A1,B1	: in	std_logic;
	EQ0,EQ1		: out	std_logic);
END comp2;

architecture archlpmstd of comp2 is

begin

    U1:ppalcell port map(zero,A0,zero,one,zero,one,B0,
			B0,A0,B0,A0,one,A1,A1,zero,
			B1,zero,one,zero,one,zero,
			zero,zero,open,EQ0,open,EQ1,open);

end archlpmstd;

--
--	BIG AND
--

library ieee;
use ieee.std_logic_1164.all;

use work.Mcompare_local.all;

entity bigand is
port (	a	: in  std_logic_vector(13 downto 0);
	q	: out std_logic);
end bigand;

architecture archlpmstd of bigand is

begin

    U1:logico port map (a(11),a(10),a(9),a(8),a(7),a(6),
			zero,one,zero,one,zero,one,a(13),a(12),
			a(5),a(4),a(3),a(2),a(1),a(0),q);

end archlpmstd;

--
--	Greater Than and Less Than
--

library ieee;
use ieee.std_logic_1164.all;

use work.Mcompare_local.all;
entity gtlt is
    port (a,b,g,l	: in	std_logic;
	gt,lt		: out	std_logic);
end gtlt;

architecture archlpmstd of gtlt is

begin

    U1:ppalcell port map(zero,b,zero,one,g,one,a,
			l,zero,l,zero,g,zero,one,zero,
			a,zero,l,zero,one,b,
			zero,zero,open,lt,open,gt,open);

end archlpmstd;

--
--	Less Than Cell
--

library ieee;
use ieee.std_logic_1164.all;

use work.Mcompare_local.all;
entity ltcell is
    port (a1,b1,a0,b0	: in	std_logic;
		lt	: out	std_logic);
end ltcell;

architecture archlpmstd of ltcell is

begin

    U1:logico port map(b1,zero,one,zero,one,a1,
			b0,a0,zero,one,one,zero,one,zero,
			a1,zero,one,zero,one,b1,lt);

end archlpmstd;

--
--	Greater Than Cell
--

library ieee;
use ieee.std_logic_1164.all;

use work.Mcompare_local.all;
entity gtcell is
    port (a1,b1,a0,b0	: in	std_logic;
		gt	: out	std_logic);
end gtcell;

architecture archlpmstd of gtcell is

begin

    U1:logico port map(a1,zero,one,zero,one,b1,
			a0,b0,zero,one,one,zero,one,zero,
			b1,zero,one,zero,one,a1,gt);

end archlpmstd;

--
--	PRiority Cell
--

library ieee;
use ieee.std_logic_1164.all;

use work.Mcompare_local.all;

entity prcell is
    port (ht2,ht1,ht0,lt2,lt1  : in  std_logic;
			    pr : out std_logic);
end prcell;

architecture archlpmstd of prcell is

begin

    U1:logico port map(ht0,zero,one,lt1,one,lt2,
			ht1,lt2,one,zero,one,zero,one,zero,
			ht2,zero,one,zero,one,zero,pr);

end archlpmstd;

--
--	MCOMPARE
--

library ieee;
use ieee.std_logic_1164.all;

use work.Mcompare_local.all;

entity Mcompare is
    generic(lpm_width	: positive;
	    lpm_representation : repre_type;
	    lpm_hint	: goal_type);
    port(   dataA	: in	std_logic_vector((lpm_width-1) downto 0);
	    dataB	: in	std_logic_vector((lpm_width-1) downto 0);
	    alb		: out	std_logic;
	    aeb		: out	std_logic;
	    agb		: out	std_logic;
	    ageb	: out   std_logic;
	    aleb	: out   std_logic;
	    aneb	: out   std_logic);
end Mcompare;

architecture archlpmstd of Mcompare is

    constant C : integer := 14;
    constant K : integer := 2;
    constant L : integer := 3;
    signal andin : std_logic_vector (((((dataA'high/C)+1)*C)-1) downto 0);
    signal aebi  : std_logic_vector (((dataA'high/C)+1) downto 0);
    signal aebo  : std_logic_vector ( 0 downto 0);
    signal eq	 : std_logic_vector (dataA'high downto 0);
    signal unused: std_logic_vector (dataA'high downto 0);

    signal lt,gt  : std_logic_vector ((dataA'high/K) downto 0);
    signal albi,albn,agbi : std_logic_vector (dataA'high downto 0);
    signal alb2, agb2 : std_logic_vector (((dataA'high/K)/L) downto 0);
    signal alb3, agb3 : std_logic_vector (2 downto 0);

begin

    -- Generate the comparator slices.
    GEQ: for j in 0 to (dataA'high / 2) generate
	E: if ((j*2+1) <= dataA'high) generate
	    U0: comp2 port map
	    (dataA(j*2),dataB(j*2),dataA(j*2+1),dataB(j*2+1),eq(j*2),eq(j*2+1));
	end generate;
	O: if ((j*2+1) > dataA'high) generate
	    U0: comp2 port map
	    (dataA(j*2),dataB(j*2),zero,zero,eq(j*2),open);
	end generate;
    end generate;

    -- Generate the constant values for the unused inputs on the BIGAND.
    AG0: if ((dataA'length rem C) /= 0) generate
	F1: for k in ((((dataA'high/C)+1)*C)-1) downto dataA'length generate
	    andin(k) <= '0' when ((k rem 2) = 0) else '1';
	end generate;
    end generate;
    andin(dataA'high downto 0) <= eq;

    -- Generate the 14-input AND gates (7 are low true).
    AG1: for l in 0 to (dataA'high/C) generate
	U1: bigand port map (andin(13+(l*C) downto l*C),aebi(l));
    end generate;
    aebi(aebi'high) <= '1';

    aebo(0) <= andvec(aebi) ;
--    U2: mand generic map (1,((dataA'high/C)+2),speed) port map (aebi, aebo);
    aeb  <= aebo(0);
    aneb <= NOT aebo(0);

    -- Generate the greater than and less than.
    GA: if (lpm_hint = AREA) generate
	GNE3: for j in dataA'high downto ((dataA'length*2)/3) generate
	    GM: if (j  = dataA'high) generate
		U1: gtlt port map
		    (dataA(j),dataB(j),zero,one,agbi(j),albn(j));
	    end generate;
	    GI: if (j /= dataA'high) generate
		U2: gtlt port map
		    (dataA(j),dataB(j),agbi(j+1),albn(j+1),agbi(j),albn(j));
	    end generate;
	end generate;
	GNE2:for j in (((dataA'length*2)/3)-1) downto (dataA'length/3) generate
	    GM: if (j  = (((dataA'length*2)/3)-1)) generate
		U1: gtlt port map
		    (dataA(j),dataB(j),zero,one,agbi(j),albn(j));
	    end generate;
	    GI: if (j /= (((dataA'length*2)/3)-1)) generate
		U2: gtlt port map
		    (dataA(j),dataB(j),agbi(j+1),albn(j+1),agbi(j),albn(j));
	    end generate;
	end generate;
	GNE1:for j in ((dataA'length/3)-1) downto 0 generate
	    GM: if (j  = ((dataA'length/3)-1)) generate
		U1: gtlt port map
		    (dataA(j),dataB(j),zero,one,agbi(j),albn(j));
	    end generate;
	    GI: if (j /= ((dataA'length/3)-1)) generate
		U2: gtlt port map
		    (dataA(j),dataB(j),agbi(j+1),albn(j+1),agbi(j),albn(j));
	    end generate;
	end generate;

	albi <= NOT albn;
	U3: prcell port map
	    (agbi((dataA'length*2)/3),agbi((dataA'length)/3),agbi(0),
		albi((dataA'length*2)/3),albi((dataA'length)/3),agb3(0));
	U4: prcell port map
	    (albi((dataA'length*2)/3),albi((dataA'length)/3),albi(0),
		agbi((dataA'length*2)/3),agbi((dataA'length)/3),alb3(0));

	agb  <= agb3(0);
	ageb <= NOT alb3(0);
	alb  <= alb3(0);
	aleb <= NOT agb3(0);
    end generate;

    GS: if (lpm_hint = SPEED) generate

	ASSERT (dataA'length <= 54)
	REPORT "Widths greater than 54 are not supported for speed goals."
	SEVERITY FAILURE;

	-- Generate the greater/less than cells for the first level.
	GNE: for j in (dataA'high / K) downto 0 generate
	    E: if ((j*2+1) <= dataA'high) generate
		U0: gtcell port map
		    (dataA(j*K+1),dataB(j*K+1),dataA(j*K),dataB(j*K),gt(j));
		U1: ltcell port map
		    (dataA(j*K+1),dataB(j*K+1),dataA(j*K),dataB(j*K),lt(j));
	    end generate;
	    O: if ((j*2+1) > dataA'high) generate
		U0: gtcell port map
		    (zero,zero,dataA(j*k),dataB(j*K),gt(j));
		U1: ltcell port map
		    (zero,zero,dataA(j*k),dataB(j*K),lt(j));
	    end generate;
	end generate;

	-- Generate the priority cells for the second level.
	GG: for i in (gt'high/L) downto 0 generate
	    F3: if (((gt'high rem L) = 2) OR ((gt'high/L) /= i))  generate
		U0: prcell port map
		    (gt(i*L+2),gt(i*L+1),gt(i*L),lt(i*L+2),lt(i*L+1),agb2(i));
		U1: prcell port map
		    (lt(i*L+2),lt(i*L+1),lt(i*L),gt(i*L+2),gt(i*L+1),alb2(i));
	    end generate;
	    F2: if (((gt'high rem L) = 1) AND ((gt'high/L) = i)) generate
		U0: prcell port map
		    (gt(i*L+1),gt(i*L),zero,lt(i*L+1),one,agb2(i));
		U1: prcell port map
		    (lt(i*L+1),lt(i*L),zero,gt(i*L+1),one,alb2(i));
	    end generate;
	    F1: if (((gt'high rem L) = 0) AND ((gt'high/L) = i)) generate
		agb2(i) <= gt(i*L);
		alb2(i) <= lt(i*L);
	    end generate;
	end generate;

	-- Generate the priority cells for the third level.
	GTL: for i in (dataA'high/18) downto 0 generate
	    F3: if (dataA'length > i*18+12) generate
		U0: prcell port map (agb2(i*L+2),agb2(i*L+1),agb2(i*L),
					alb2(i*L+2),alb2(i*L+1),agb3(i));
		U1: prcell port map (alb2(i*L+2),alb2(i*L+1),alb2(i*L),
					agb2(i*L+2),agb2(i*L+1),alb3(i));
	    end generate;
	    F2: if ((dataA'length > i*18+6) AND (dataA'length <= i*18+12))
	    generate
		U0: prcell port map
		    (agb2(i*L+1),agb2(i*L),zero,alb2(i*L+1),one,agb3(i));
		U1: prcell port map
		    (alb2(i*L+1),alb2(i*L),zero,agb2(i*L+1),one,alb3(i));
	    end generate;
	    F1: if (dataA'length <= i*18+6) generate
		agb3(i) <= agb2(i*L);
		alb3(i) <= alb2(i*L);
	    end generate;
	end generate;

	-- Generate the priority cells for the forth level.
	F3: if (dataA'length > 18*2) generate
	    U0: prcell port map
		(agb3(2),agb3(1),agb3(0),alb3(2),alb3(1),agbi(0));
	    U1: prcell port map
		(alb3(2),alb3(1),alb3(0),agb3(2),agb3(1),albi(0));
	end generate;
	F2: if ((dataA'length > 18) AND (dataA'length <= 18*2)) generate
	    U0: prcell port map
		(agb3(1),agb3(0),zero,alb3(1),one,agbi(0));
	    U1: prcell port map
		(alb3(1),alb3(0),zero,agb3(1),one,albi(0));
	end generate;
	F1: if (dataA'length <= 18) generate
	    agbi(0) <= agb3(0);
	    albi(0) <= alb3(0);
	end generate;
    agb  <= agbi(0);
    alb  <= albi(0);
    aleb <= NOT agbi(0);
    ageb <= NOT albi(0);
    end generate;

end archlpmstd;
