--
--	Copyright Cypress Semiconductor Corporation, 1995,1996
--	  as an unpublished work.
--

library ieee;
use ieee.std_logic_1164.all;

PACKAGE iopkg IS

    COMPONENT CKPAD
	PORT ( 
	    p		: in std_logic;
	    q		: out std_logic);
    END COMPONENT;

    COMPONENT HDPAD
	PORT (
	    p		: in std_logic;
	    q		: out std_logic);
    END COMPONENT;

    COMPONENT HDIPAD
	PORT (
	    p		: in std_logic;
	    qn		: out std_logic);
    END COMPONENT;

    COMPONENT HD2PAD
	PORT (
	    p0		: in std_logic;
	    p1		: in std_logic;
	    q		: out std_logic);
    END COMPONENT;

    COMPONENT HD3PAD
	PORT (
	    p0		: in std_logic;
	    p1		: in std_logic;
	    p2		: in std_logic;
	    q		: out std_logic);
    END COMPONENT;

    COMPONENT HD4PAD
	PORT (
	    p0		: in std_logic;
	    p1		: in std_logic;
	    p2		: in std_logic;
	    p3		: in std_logic;
	    q		: out std_logic);
    END COMPONENT;

    COMPONENT INPAD
	PORT ( 
	    p		: inout std_logic;
	    q		: out std_logic);
    END COMPONENT;

    COMPONENT OUTPAD
	PORT ( 
	    a		: in std_logic;
	    p		: inout std_logic);
    END COMPONENT;

END iopkg;

------

library ieee;
use ieee.std_logic_1164.all;

package iopkg_local is
    COMPONENT PAckcell		-- Master Clock Cell.
	PORT (
	    ip	: IN std_logic;	-- Input pad.
	    ini	: OUT std_logic;	-- Inverted.
	    iz	: OUT std_logic;	-- Non-inverted.
	    ic	: OUT std_logic	-- To "Dedicated clock network".
	);
    END COMPONENT;
    COMPONENT PAincell		-- Master Input Cell.
	PORT (
	    ip	: IN std_logic;	-- Input pad.
	    ini	: OUT std_logic;	-- Inverting output of cell.
	    iz	: OUT std_logic	-- Non-inverting output of cell.
	);
    END COMPONENT;
    COMPONENT PAbicell		-- Master Bi-directional Cell.
	PORT (
	    ie	: IN std_logic;	-- Output Enable.
	    i1	: IN std_logic;	-- Inverting input.
	    i2	: IN std_logic;	-- Non-inverting input.
	    iz	: OUT std_logic;	-- Path into chip.
	    ip	: inout std_logic	-- I/O pad.
	);
    END COMPONENT;

end iopkg_local ;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.iopkg_local.ALL;
ENTITY CKPAD IS
    PORT ( 
	p	: in std_logic;
	q	: out std_logic);
END CKPAD;

ARCHITECTURE archCKPAD OF CKPAD IS
BEGIN
    u1	: PAckcell
	port map (
	    ip	=> P,
	    ini	=> OPEN,
	    iz	=> OPEN,
	    ic	=> Q);
END archCKPAD;

------

library ieee;
use ieee.std_logic_1164.all;

USE WORK.iopkg_local.ALL;
ENTITY HDPAD IS
    PORT (
	p	: in std_logic;
	q	: out std_logic);
END HDPAD;

ARCHITECTURE archHDPAD OF HDPAD IS
BEGIN
    u1	: PAincell
	port map (
	    ip	=> p,
	    ini	=> OPEN,
	    iz	=> q);
END archHDPAD;

------

library ieee;
use ieee.std_logic_1164.all;

USE WORK.iopkg_local.ALL;
ENTITY HDIPAD IS
    PORT (
	p	: in std_logic;
	qn	: out std_logic);
END HDIPAD;

ARCHITECTURE archHDIPAD OF HDIPAD IS
BEGIN
    u1	: PAincell
	port map (
	    ip	=> p,
	    ini	=> qn,
	    iz	=> OPEN);
END archHDIPAD;

------

library ieee;
use ieee.std_logic_1164.all;

USE WORK.iopkg_local.ALL;
ENTITY HD2PAD IS
    PORT (
	p0	: IN  std_logic;
	p1	: IN  std_logic;
	q	: out std_logic);
END HD2PAD;

ARCHITECTURE archHD2PAD OF HD2PAD IS

	signal o : std_logic;

BEGIN
    u0	: PAincell port map ( ip => p0, ini => OPEN, iz => o);
    u1	: PAincell port map ( ip => p1, ini => OPEN, iz => o);
    q <= o;

END archHD2PAD;

------

library ieee;
use ieee.std_logic_1164.all;

USE WORK.iopkg_local.ALL;
ENTITY HD3PAD IS
    PORT (
	p0	: IN  std_logic;
	p1	: IN  std_logic;
	p2	: IN  std_logic;
	q	: out std_logic);
END HD3PAD;

ARCHITECTURE archHD3PAD OF HD3PAD IS

	signal o : std_logic;

BEGIN
    u0	: PAincell port map ( ip => p0, ini => OPEN, iz => o);
    u1	: PAincell port map ( ip => p1, ini => OPEN, iz => o);
    u2	: PAincell port map ( ip => p2, ini => OPEN, iz => o);
    q <= o;

END archHD3PAD;

------

library ieee;
use ieee.std_logic_1164.all;

USE WORK.iopkg_local.ALL;
ENTITY HD4PAD IS
    PORT (
	p0	: IN  std_logic;
	p1	: IN  std_logic;
	p2	: IN  std_logic;
	p3	: IN  std_logic;
	q	: out std_logic);
END HD4PAD;

ARCHITECTURE archHD4PAD OF HD4PAD IS

	signal o : std_logic;

BEGIN
    u0	: PAincell port map ( ip => p0, ini => OPEN, iz => o);
    u1	: PAincell port map ( ip => p1, ini => OPEN, iz => o);
    u2	: PAincell port map ( ip => p2, ini => OPEN, iz => o);
    u3	: PAckcell port map ( ip => p3, ini => OPEN, iz	=> o, ic => OPEN);
    q <= o;

END archHD4PAD;

------

library ieee;
use ieee.std_logic_1164.all;

USE WORK.iopkg_local.ALL;
ENTITY INPAD IS
    PORT ( 
	p		: inout std_logic;
	q		: out std_logic);
END INPAD;

ARCHITECTURE archINPAD OF INPAD IS
    signal zero : std_logic := '0';
    signal one : std_logic := '1';
BEGIN
	U0 : PAbicell port map (zero,one,zero,q,p);

END archINPAD;

------

library ieee;
use ieee.std_logic_1164.all;

USE WORK.iopkg_local.ALL;
ENTITY OUTPAD IS
    PORT ( 
	a		: in std_logic;
	p		: inout std_logic);
END OUTPAD;

ARCHITECTURE archOUTPAD OF OUTPAD IS
    signal one : std_logic := '1';
BEGIN
	U0 : PAbicell port map (one,one,a,OPEN,p);

END archOUTPAD;
