--
--	Copyright Cypress Semiconductor Corporation, 1995,1996
--	  as an unpublished work.
--

use work.cypress.x01z;
use work.cypress.x01z_vector;

PACKAGE iopkg IS

    COMPONENT CKPAD
	PORT ( 
	    p		: in bit;
	    q		: out bit);
    END COMPONENT;

    COMPONENT HDPAD
	PORT (
	    p		: in bit;
	    q		: out bit);
    END COMPONENT;

    COMPONENT HDIPAD
	PORT (
	    p		: in bit;
	    qn		: out bit);
    END COMPONENT;

    COMPONENT HD2PAD
	PORT (
	    p0,p1	: in bit;
	    q		: out bit);
    END COMPONENT;

    COMPONENT HD3PAD
	PORT (
	    p0,p1,p2	: in bit;
	    q		: out bit);
    END COMPONENT;

    COMPONENT HD4PAD
	PORT (
	    p0,p1,p2,p3	: in bit;
	    q		: out bit);
    END COMPONENT;

    COMPONENT INPAD
	PORT ( 
	    p		: inout x01z;
	    q		: out bit);
    END COMPONENT;

    COMPONENT OUTPAD
	PORT ( 
	    a		: in bit;
	    p		: inout x01z);
    END COMPONENT;

END iopkg;

------

ENTITY CKPAD IS
    PORT ( 
	p	: in bit;
	q	: out bit);
END CKPAD;

ARCHITECTURE archCKPAD OF CKPAD IS
BEGIN
    q <= p;
END archCKPAD;

------

ENTITY HDPAD IS
    PORT (
	p	: in bit;
	q	: out bit);
END HDPAD;

ARCHITECTURE archHDPAD OF HDPAD IS
BEGIN
    q <= p;
END archHDPAD;

------

ENTITY HDIPAD IS
    PORT (
	p	: in bit;
	qn	: out bit);
END HDIPAD;

ARCHITECTURE archHDIPAD OF HDIPAD IS
BEGIN
    qn <= not p;
END archHDIPAD;

------

ENTITY HD2PAD IS
    PORT (
	p0,p1	: in bit;
	q	: out bit);
END HD2PAD;

ARCHITECTURE archHD2PAD OF HD2PAD IS
BEGIN
    q <= p0 or p1;
END archHD2PAD;

------

ENTITY HD3PAD IS
    PORT (
	p0,p1,p2: in bit;
	q	: out bit);
END HD3PAD;

ARCHITECTURE archHD3PAD OF HD3PAD IS
BEGIN
    q <= p0 or p1 or p2;
END archHD3PAD;

------

ENTITY HD4PAD IS
    PORT (
	p0,p1,p2,p3 : in bit;
	q	: out bit);
END HD4PAD;

ARCHITECTURE archHD4PAD OF HD4PAD IS
BEGIN
    q <= p0 or p1 or p2 or p3;
END archHD4PAD;

------

use work.cypress.x01z;
use work.cypress.x01z_vector;

ENTITY INPAD IS
    PORT (
	p	: inout x01z;
	q	: out bit);
END INPAD;

ARCHITECTURE archINPAD OF INPAD IS
    signal zero : bit := '0';
component bufoe
    port(
	x	: in bit;		-- Logic input to Buffer.
	oe	: in bit;		-- Output Enable input.
	y	: inout x01z;		-- Output pin.
	yfb	: out bit		-- Output feed back as bit.
    );
end component;
BEGIN
    U1: bufoe port map (zero,zero,p,q);
END archINPAD;

------

use work.cypress.x01z;
use work.cypress.x01z_vector;

ENTITY OUTPAD IS
    PORT (
	a	: in bit;
	p	: inout x01z);
END OUTPAD;

ARCHITECTURE archOUTPAD OF OUTPAD IS
    signal one : bit := '1';
component bufoe
    port(
	x	: in bit;		-- Logic input to Buffer.
	oe	: in bit;		-- Output Enable input.
	y	: inout x01z;		-- Output pin.
	yfb	: out bit		-- Output feed back as bit.
    );
end component;
BEGIN
    U1: bufoe port map (a,one,p,open);
END archOUTPAD;

