--
--      Copyright Cypress Semiconductor Corporation, 1995,1996
--	as an unpublished work.
--

--
--	MADD_SUB
--

use work.lpmpkg.all;

entity Madd_sub is
    generic(lpm_width	: positive;
	    lpm_representation	: repre_type;
	    lpm_direction	: arith_type;
	    lpm_hint	: goal_type);
    port(   dataA	: in	bit_vector((lpm_width-1) downto 0);
	    dataB	: in	bit_vector((lpm_width-1) downto 0);
	    cin		: in	bit;
	    add_sub	: in	bit;
	    result	: out	bit_vector((lpm_width-1) downto 0);
	    cout	: out	bit;
	    overflow	: out   bit);
end Madd_sub;

architecture SIM of Madd_sub is

    signal a,b,c,r : bit_vector (lpm_width downto 0) := (others => '0');

begin

    a((lpm_width - 1) downto 0)  <= dataA;
    b((lpm_width - 1) downto 0)  <= dataB when (lpm_direction = lpm_add) else
				NOT dataB when (lpm_direction = lpm_sub) else
				NOT dataB when (add_sub = '0') else
				    dataB;
    c(0) <= cin;
    A0: for i in result'low to result'high generate
	r(i) <= c(i) XOR a(i) XOR b(i);
	c(i+1) <= (a(i) AND b(i)) OR (a(i) AND c(i)) OR (b(i) AND c(i));
    end generate;
    result <= r((lpm_width - 1) downto 0);
    cout <= c(lpm_width);
    overflow <= ((r(result'high) AND
	      NOT a(result'high) AND NOT b(result'high)) OR
	     (NOT r(result'high) AND
		  a(result'high) AND     b(result'high)));

end SIM;

--
--	MAND
--

use work.lpmpkg.all;

entity Mand is
    generic(lpm_width	: positive;
	    lpm_size	: positive;
	    lpm_hint	: goal_type;
	    lpm_data_pol	: string;
	    lpm_result_pol	: string);
    port(   data	: in	bit_vector(((lpm_width*lpm_size)-1) downto 0);
	    result	: out	bit_vector((lpm_width-1) downto 0));
end Mand;

architecture SIM of Mand is

    constant d_pol : bit_vector(((lpm_width*lpm_size)-1) downto 0)
				:= restring(lpm_data_pol,lpm_width*lpm_size);
    constant r_pol : bit_vector(result'range)
				:= restring(lpm_result_pol,lpm_width);

begin

    process (data)
	variable tmp : bit;
    begin
	for n in 0 to lpm_width-1 loop
	    tmp := '1';
	    for m in 0 to lpm_size-1 loop
	        tmp := (tmp AND data((m*lpm_width)+n))
			XOR NOT d_pol((m*lpm_width)+n);
	    end loop;
	    result(n) <= tmp XOR NOT r_pol(n);
	end loop;
    end process;

end SIM;

--
--	MOR
--

use work.lpmpkg.all;

entity Mor is
    generic(lpm_width	: positive;
	    lpm_size	: positive;
	    lpm_hint	: goal_type;
	    lpm_data_pol	: string;
	    lpm_result_pol	: string);
    port(   data	: in	bit_vector(((lpm_width*lpm_size)-1) downto 0);
	    result	: out	bit_vector((lpm_width-1) downto 0));
end Mor;

architecture SIM of Mor is

    constant d_pol : bit_vector(((lpm_width*lpm_size)-1) downto 0)
				:= restring(lpm_data_pol,lpm_width*lpm_size);
    constant r_pol : bit_vector(result'range)
				:= restring(lpm_result_pol,lpm_width);

begin

    process (data)
	variable tmp : bit;
    begin
	for n in 0 to lpm_width-1 loop
	    tmp := '0';
	    for m in 0 to lpm_size-1 loop
	        tmp := (tmp OR data((m*lpm_width)+n))
			XOR NOT d_pol((m*lpm_width)+n);
	    end loop;
	    result(n) <= tmp XOR NOT r_pol(n);
	end loop;
    end process;

end SIM;

--
--	MBUF
--

use work.lpmpkg.all;

entity Mbuf is
    generic(	lpm_width	: positive;
		lpm_hint	: goal_type);
    port(	data		: in	bit_vector((lpm_width-1) downto 0);
    		result		: out	bit_vector((lpm_width-1) downto 0));
end Mbuf;

architecture SIM of Mbuf is
begin

    result <= data;

end SIM;

--
--	MBUSTRI
--

use work.cypress.x01z;
use work.cypress.x01z_vector;

use work.lpmpkg.all;

entity Mbustri is
    generic(lpm_width	: positive;
	    lpm_hint	: goal_type);
    port(   tridata	: inout	x01z_vector((lpm_width-1) downto 0);
	    data	: in	bit_vector((lpm_width-1) downto 0);
	    enabletr	: in	bit;
	    enabledt	: in	bit;
	    result	: out	x01z_vector((lpm_width-1) downto 0));
end Mbustri;

architecture SIM of Mbustri is

    signal yfb : bit_vector (result'range);

    signal unk : x01z := 'x';
    signal hiz : x01z := 'z';

begin

    L0: for i in data'range generate
	tridata(i) <= hiz when (enabledt = '0') else
		      '0' when (data(i)  = '0') else
		      '1' when (data(i)  = '1') else
		      unk;

	result(i) <=  hiz when (enabletr   = '0') else
		      '0' when (tridata(i) = '0') else
		      '1' when (tridata(i) = '1') else
		      unk;
    end generate;

end SIM;

--
--	MCOMPARE
--

use work.lpmpkg.all;

entity Mcompare is
    generic(lpm_width	: positive;
	    lpm_representation	: repre_type;
	    lpm_hint	: goal_type);
    port(   dataA	: in	bit_vector((lpm_width-1) downto 0);
	    dataB	: in	bit_vector((lpm_width-1) downto 0);
	    alb		: out	bit;
	    aeb		: out	bit;
	    agb		: out	bit;
	    ageb	: out   bit;
	    aleb	: out   bit;
	    aneb	: out   bit);
end Mcompare;

architecture SIM of Mcompare is
begin

    alb  <= '1' when (dataA < dataB) else '0';
    aeb  <= '1' when (dataA = dataB) else '0';
    agb  <= '1' when (dataA > dataB) else '0';
    ageb <= '1' when NOT(dataA < dataB) else '0';
    aleb <= '1' when NOT(dataA > dataB) else '0';
    aneb <= '1' when NOT(dataA = dataB) else '0';

end SIM;

--
--	MMUX
--

use work.lpmpkg.all;

entity Mmux is
    generic(lpm_width	: positive;
	    lpm_size	: positive;
	    lpm_widths	: positive;
	    lpm_hint	: goal_type);
    port(   data	: in	bit_vector(((lpm_width*lpm_size)-1) downto 0);
	    sel		: in	bit_vector((lpm_widths-1) downto 0);
	    result	: out	bit_vector((lpm_width-1) downto 0));
end Mmux;

architecture SIM of Mmux is

begin

    process (data,sel)
	variable tmp : bit_vector((lpm_width-1) downto 0);
    begin
	tmp := (others => '0');
	for selval in 0 to ((2 ** lpm_widths)-1) loop
	    if (vectorize(selval, lpm_widths) = sel) then
		tmp:=data(((lpm_width*(selval+1))-1) downto (lpm_width*selval));
	    end if;
	end loop;
	result <= tmp;
    end process;

end SIM;

--
--	MCLSHIFT
--

use work.lpmpkg.all;

entity Mclshift is
    generic(lpm_width		: positive;
	    lpm_widthdist	: natural;
	    lpm_shifttype	: shift_type;
	    lpm_hint		: goal_type);
    port(   data		: in  bit_vector((lpm_width-1) downto 0);
	    distance		: in  bit_vector((lpm_widthdist-1) downto 0);
	    direction 		: in  bit;
	    result		: out bit_vector((lpm_width-1) downto 0);
	    overflow		: out bit;
	    underflow		: out bit);
end Mclshift;

architecture SIM of Mclshift is

    FUNCTION log2 (val : natural) RETURN natural IS
	variable tmp  : natural := val;
	variable sval : natural;
    BEGIN
	if (tmp REM 2) = 0 then tmp := tmp - 1; end if;
	sval := 0;
	while tmp > 0 loop
	    sval := sval + 1;
	    tmp := tmp / 2;
	end loop;
	return sval;
    END log2;

    constant movement : natural := log2(lpm_width);
    constant shift : natural := max(movement,lpm_widthdist);
    signal def : bit;
    signal OK : boolean := true;
    signal widthv : bit_vector(distance'range);
    signal shifter,ors,dist : bit_vector((shift-1) downto 0) := (others => '0');
    signal barrel:bit_vector(((result'length*result'length)-1) downto 0);
    signal left,right,sign,mask : bit_vector(result'high downto 0);
    signal output,loutput,aoutput : bit_vector(result'high downto 0);
    signal lunderflow,loverflow,aunderflow,aoverflow : bit;
    signal un,ov : bit_vector(result'high downto 0);

    for all: mmux use entity work.mmux (sim);
    for all: mcompare use entity work.mcompare (sim);

begin

    CP: if (2**lpm_widthdist > lpm_width) generate
	widthv <= vectorize(lpm_width,lpm_widthdist);
	C1: mcompare generic map (lpm_widthdist,lpm_unsigned,lpm_hint)
			port map (distance,widthv,def,open,open,open,open,open);
	OK <= true when (def = '1') else false;
    end generate;

    dist(distance'range) <= distance;
    shifter(0) <= dist(0);
    ors(0) <= dist(0);
    GS: for i in (shifter'low + 1) to shifter'high generate
	ors(i) <= ors(i-1) OR dist(i);
	shifter(i) <= dist(i) when (direction = '1')
				    else dist(i) XOR (ors(i-1));
    end generate;

    GB: for j in data'low to data'high generate
	GJ1: for k in data'low to data'high generate
	    barrel((j*data'length)+k) <= data((j+k) rem result'length);
	end generate;
    end generate;

    M1:mmux generic map(result'length,result'length,shift,lpm_hint)
		port map(barrel,shifter,output);

    GM:if (lpm_shifttype /= lpm_rotate) generate
	GML: for i in result'low to result'high generate
	    D0:if (i <  lpm_widthdist**2) generate
		left(i) <= '1' when
		    ((vectorize(i,lpm_widthdist)) < distance) else '0';
		right(result'high-i) <= '1' when
		    ((vectorize(i,lpm_widthdist)) < distance) else '0';
	    end generate;
	    D1:if (i >= lpm_widthdist**2) generate
		left(i) <= '0';
		right(result'high-i) <= '0';
	    end generate;
	end generate;
	mask <= left when (direction = '0') else right;
    end generate;

    GL:if (lpm_shifttype = lpm_logical) generate
	loutput <= (output AND NOT mask);

	ov(0) <= output(0) AND mask(0);
	GLO: for i in (result'low + 1) to result'high generate
	    ov(i) <= ov(i-1) OR (output(i) AND mask(i));
	end generate;
	loverflow <= NOT direction AND ov(ov'high);

	un(0) <= loutput(0);
	GLU: for i in (result'low + 1) to result'high generate
	    un(i) <= un(i-1) OR loutput(i);
	end generate;
	lunderflow <= direction AND NOT un(un'high);
    end generate;

    GA:if (lpm_shifttype = lpm_arithmetic) generate
	sign <= (others => data(data'high));
	L0: for i in result'low to (result'high - 1) generate
	    aoutput(i) <= output(i) AND NOT mask(i) when (direction = '0')
		else (output(i) AND NOT mask(i)) OR (right(i) AND sign(i));
	end generate;
	aoutput(result'high) <= data(result'high) when (direction = '0')
	    else (output(result'high) AND NOT mask(result'high))
	    OR (right(result'high) AND sign(result'high));

	ov(0) <= (output(0) XOR sign(0)) AND mask(0);
	GAO: for i in (result'low + 1) to result'high generate
	    ov(i) <= ov(i-1) OR ((output(i) XOR sign(i)) AND mask(i));
	end generate;
	aoverflow <= NOT direction AND
		(ov(ov'high) OR (output(output'high) XOR data(data'high)));

	un(0) <= aoutput(0) XOR NOT sign(0);
	GAU: for i in (result'low + 1) to result'high generate
	    un(i) <= un(i-1) AND (aoutput(i) XOR NOT sign(i));
	end generate;
	aunderflow <= direction AND un(un'high);
    end generate;

    result    <= aoutput when ((lpm_shifttype = lpm_arithmetic) AND OK) else
		 loutput when ((lpm_shifttype = lpm_logical) AND OK) else
		 output  when  (lpm_shifttype = lpm_rotate) else
		 (others => data(data'high))
		 when ((lpm_shifttype = lpm_arithmetic) AND NOT OK) else
		 (others => '0');
    overflow  <= aoverflow  when (lpm_shifttype = lpm_arithmetic) else
		 loverflow  when (lpm_shifttype = lpm_logical) else '0';
    underflow <= aunderflow when (lpm_shifttype = lpm_arithmetic) else
		 lunderflow when (lpm_shifttype = lpm_logical) else '0';

end SIM;

--
--	Mcnstnt
--

use work.lpmpkg.all;

entity Mcnstnt is
    generic(lpm_width	: positive;
	    lpm_cvalue	: string;
	    lpm_hint	: goal_type);
    port(   result	: out	bit_vector((lpm_width-1) downto 0));
end Mcnstnt;

architecture SIM of Mcnstnt is
begin

    result <= restring(lpm_cvalue,lpm_width);

end SIM;

--
--	MCOUNTER
--

use work.lpmpkg.all;

entity Mcounter is
    generic(lpm_width	: positive;
	    lpm_direction	: ctdir_type;
	    lpm_avalue	: string;
	    lpm_svalue	: string;
	    lpm_pvalue	: string;
	    lpm_hint	: goal_type);
    port(   data	: in	bit_vector((lpm_width-1) downto 0);
	    clock	: in	bit;
	    clk_en	: in	bit;
	    cnt_en	: in	bit;
	    updown	: in	bit;
	    q		: out	bit_vector((lpm_width-1) downto 0);
	    aset	: in	bit;
	    aclr	: in	bit;
	    aload	: in	bit;
	    sset	: in	bit;
	    sclr	: in	bit;
	    sload	: in	bit;
	    testenab	: in	bit;
	    testin	: in	bit;
	    testout	: out	bit);
end Mcounter;

architecture SIM of Mcounter is

    signal async,set,clr,qtmp,d,en_up,en_dn,en : bit_vector(q'range);

begin

    async <= restring(lpm_avalue,lpm_width);

    G0: for j in q'reverse_range generate
	G0:if (j  = 0) generate
	    en_up(0) <= '1';
	    en_dn(0) <= '1';
	end generate;
	G1:if (j /= 0) generate
	    en_up(j) <= (en(j-1) AND     updown AND     qtmp(j-1));
	    en_dn(j) <= (en(j-1) AND NOT updown AND NOT qtmp(j-1));
	end generate;
	
	TU: if (lpm_direction = lpm_up) generate
	    en(j) <= en_up(j);
	end generate;
	TD: if (lpm_direction = lpm_down) generate
	    en(j) <= en_dn(j);
	end generate;
	TB: if NOT((lpm_direction = lpm_up)
		OR (lpm_direction = lpm_down)) generate
	    en(j) <= en_up(j) OR en_dn(j);
	end generate;
    end generate;
	
    process (data,sset,sclr,sload,en,cnt_en,testin,testenab,qtmp)
    begin
	if (testenab = '1') then
	    d(0) <= testin;
	    if (lpm_width > 1) then
		d(q'high downto 1) <= qtmp((q'high - 1) downto 0);
	    end if;
	elsif (sclr = '1' AND clk_en = '1') then
	    d <= (others => '0');
	elsif (sset = '1' AND clk_en = '1') then
	    d <= restring(lpm_svalue,lpm_width);
	elsif (sload = '1' AND clk_en = '1') then
	    d <= data;
	elsif (cnt_en = '1' AND clk_en = '1') then
	    for j in q'range loop
		if (en(j) = '1') then
		    d(j) <= NOT qtmp(j);
		else
		    d(j) <= qtmp(j);
		end if;
	    end loop;
	else
	    d <= qtmp;
	end if;
    end process;

    process (aset, aclr, aload, clock)
    begin
	for j in q'range loop
	    set(j) <=(aset AND     async(j)) OR (aload AND     data(j));
	    clr(j) <=(aset AND NOT async(j)) OR (aload AND NOT data(j)) OR aclr;

	    if    (clr(j) = '1') then qtmp(j) <= '0';
	    elsif (set(j) = '1') then qtmp(j) <= '1';
	    elsif (clock  = '1' AND clock'event) then qtmp(j) <= d(j);
	    end if;
	end loop;
    end process;

    testout <= qtmp(q'high);
    q <= qtmp;

end SIM;

--
--	MDECODE
--

use work.lpmpkg.all;

entity Mdecode is
    generic(lpm_width	: positive;
	    lpm_decodes	: positive;
	    lpm_hint	: goal_type);
    port(   data	: in  bit_vector((lpm_width-1) downto 0);
	    enable	: in  bit;
	    eq		: out bit_vector((lpm_decodes-1) downto 0));
end Mdecode;

architecture SIM of Mdecode is
begin

    G0: for i in eq'low to eq'high generate
	eq(i) <= '1' when
	    ((vectorize(i,data'length) = data) AND (enable = '1'))
	else '0';
    end generate;

end SIM;

--
--	MFF
--

use work.lpmpkg.all;

entity Mff is
    generic(lpm_width	: positive;
	    lpm_fftype	: fflop_type;
	    lpm_avalue	: string;
	    lpm_svalue	: string;
	    lpm_pvalue	: string;
	    lpm_hint	: goal_type);
    port(   data	: in	bit_vector((lpm_width-1) downto 0);
	    clock	: in	bit;
	    enable	: in	bit;
	    q		: out	bit_vector((lpm_width-1) downto 0);
	    aset	: in	bit;
	    aclr	: in	bit;
	    aload	: in	bit;
	    sset	: in	bit;
	    sclr	: in	bit;
	    sload	: in	bit;
	    testenab	: in	bit;
	    testin	: in	bit;
	    testout	: out	bit);
end Mff;

architecture SIM of Mff is

    signal async,set,clr,qtmp,d : bit_vector(q'range);

begin

    async <= restring(lpm_avalue,lpm_width);

    process (data,sset,sclr,sload,enable,testin,testenab,qtmp)
    begin
	if (testenab = '1') then
	    d(0) <= testin;
	    if (lpm_width > 1) then
		d(q'high downto 1) <= qtmp((q'high - 1) downto 0);
	    end if;
	elsif (sclr = '1' AND enable = '1') then
	    d <= (others => '0');
	elsif (sset = '1' AND enable = '1') then
	    d <= restring(lpm_svalue,lpm_width);
	elsif (sload = '1' AND enable = '1') then
	    d <= data;
	elsif (enable = '1') then
	    if (lpm_fftype = lpm_tff) then
		d <= data XOR qtmp;
	    else
		d <= data;
	   end if;
	else
	    d <= qtmp;
	end if;
    end process;

    process (aset, aclr, aload, clock)
    begin
	for j in q'range loop
	    set(j) <=(aset AND     async(j)) OR (aload AND     data(j));
	    clr(j) <=(aset AND NOT async(j)) OR (aload AND NOT data(j)) OR aclr;

	    if    (clr(j) = '1') then qtmp(j) <= '0';
	    elsif (set(j) = '1') then qtmp(j) <= '1';
	    elsif (clock  = '1' AND clock'event) then qtmp(j) <= d(j);
	    end if;
	end loop;
    end process;

    testout <= qtmp(qtmp'high);
    q <= qtmp;

end SIM;

--
--	Expandable GND
--

use work.lpmpkg.all;
entity MGND is
    generic(lpm_width		: positive);
    port(	X		: out	bit_vector((lpm_width-1) downto 0));
end MGND;

architecture SIM of mgnd is
begin

    x <= (others => '0');

end SIM;

--
--	MINV
--

use work.lpmpkg.all;

entity Minv is
    generic(lpm_width	: positive;
	    lpm_hint	: goal_type);
    port(   data	: in	bit_vector((lpm_width-1) downto 0);
	    result	: out	bit_vector((lpm_width-1) downto 0));
end Minv;

architecture SIM of Minv is
begin

    result <= NOT data;

end SIM;

--
--	MLATCH
--

use work.lpmpkg.all;

entity Mlatch is
    generic(lpm_width	: positive;
	    lpm_avalue	: string;
	    lpm_pvalue	: string;
	    lpm_hint	: goal_type);
    port(   data	: in	bit_vector((lpm_width-1) downto 0);
	    gate	: in	bit;
	    q		: out	bit_vector((lpm_width-1) downto 0);
	    aset	: in	bit;
	    aclr	: in	bit);
end Mlatch;

architecture SIM of Mlatch is

    signal qtmp,async,set,clr : bit_vector(q'high downto 0);

begin

    async <= restring(lpm_avalue,lpm_width);

    G0: for j in q'range generate
	set(j)    <= (aset AND     async(j));
	clr(j)    <= (aset AND NOT async(j)) OR aclr;
	qtmp(j) <= NOT clr(j) AND (set(j) OR (data(j) AND gate)
		    OR (qtmp(j) AND NOT gate));
    end generate;

    q <= qtmp;

end SIM;

--
--	MMULT
--

use work.lpmpkg.all;

entity Mmult is
    generic(lpm_widthA	: positive;
	    lpm_widthB	: positive;
	    lpm_widthS	: natural;
	    lpm_widthP	: positive;
	    lpm_representation : repre_type;
	    lpm_hint	: goal_type);
    port(   dataA	: in	bit_vector((lpm_widthA-1) downto 0);
	    dataB	: in	bit_vector((lpm_widthB-1) downto 0);
	    sum		: in	bit_vector((lpm_widthS-1) downto 0);
	    result	: out	bit_vector((lpm_widthP-1) downto 0));
end Mmult;

architecture SIM of Mmult is

    constant psize : natural := max(lpm_widthA,lpm_widthB);
    constant asize : natural := max((lpm_widthA+lpm_widthB),lpm_widthS);
    signal a,b : bit_vector ((psize-1) downto 0);
    signal p   : bit_vector (((psize*2)-1) downto 0);
    signal a0,b0,x : bit_vector ((asize-1) downto 0);
    signal one  : bit := '1';
    signal zero : bit := '0';

    for all: madd_sub use entity work.madd_sub (sim);

    function plus (a,b : bit_vector) return bit_vector is
	variable result : bit_vector((a'length-1) downto 0);
	variable c : bit_vector (a'length downto 0);
    begin
	c(0) := '0';
	for i in result'low to result'high loop
	    result(i) := c(i) XOR a(i) XOR b(i);
	    c(i+1) := (a(i) AND b(i)) OR (a(i) AND c(i)) OR (b(i) AND c(i));
	end loop;
	return result;
    end plus;

    function multx2 (a : bit_vector) return bit_vector is
	variable result : bit_vector((a'length-1) downto 0) := (others => '0');
    begin
	result(result'high downto 1) := a((a'high-1) downto a'low);
	result(0) := '0';
	return result;
    end multx2;

    function multiply (a,b : bit_vector) return bit_vector is
	variable result,tmp : bit_vector ((a'length+b'length-1) downto 0) 
							:= (others => '0');
    begin
	tmp((b'length-1) downto 0) := b;
	tmp(tmp'high downto b'length) := (others => '0');
	for i in 0 to a'length-1 loop
	    if (a(a'low+i) = '1') then result := plus(result,tmp); end if;
	    tmp := multx2(tmp);
	end loop;
	return result;
    end multiply;
 
begin

    R0: if (lpm_widthA > lpm_widthB) generate
	b((b'high) downto lpm_widthB) <= (others => '0');
	b((lpm_widthB - 1) downto 0) <= dataB;
	a <= dataA;
    end generate;
    R1: if (lpm_widthA < lpm_widthB) generate
	a((psize-1) downto lpm_widthA) <= (others => '0');
	a((lpm_widthA - 1) downto 0) <= dataA;
	b <= dataB;
    end generate;
    R2: if (lpm_widthA = lpm_widthB) generate
	a <= dataA;
	b <= dataB;
    end generate;

    p <= multiply(a,b);

    -- Align the product and the partial sum.

    GA:if (lpm_widthS > 0) generate
	G4: if (lpm_widthA+lpm_widthB < lpm_widthS) generate
	    a0((asize-1) downto (lpm_widthA+lpm_widthB)) <= (OTHERS => '0');
	    a0((lpm_widthA+lpm_widthB-1) downto 0) <=
					p((lpm_widthA+lpm_widthB-1) downto 0);
	    b0 <= sum;
	end generate;
	G5: if (lpm_widthA+lpm_widthB > lpm_widthS) generate
	    a0 <= p;
	    b0((asize-1) downto (lpm_widthS)) <= (OTHERS => '0');
	    b0(sum'range) <= sum;
	end generate;
	G6: if (lpm_widthA+lpm_widthB = lpm_widthS) generate
	    a0 <= p((lpm_widthA+lpm_widthB-1) downto 0);
	    b0 <= sum;
	end generate;

	U0: Madd_sub generic map (asize,lpm_unsigned,lpm_add,lpm_hint)
			port map (a0,b0,zero,one,x,open,open);
    end generate;

    -- No partial sum.

    GN:if (lpm_widthS = 0) generate
	    x <= p((asize-1) downto 0);
    end generate;

    -- Align the result with the most significant bit of the product.

    G7: if (lpm_widthP > asize) generate
	result((lpm_widthP-1) downto asize) <= (OTHERS => '0');
	result((asize-1) downto 0) <= x;
    end generate;
    G8: if (lpm_widthP < asize) generate
	result <= x((asize-1) downto (asize-lpm_widthP));
    end generate;
    G9: if (lpm_widthP = asize) generate
	result <= x;
    end generate;

end SIM;

--
--	MSHIFTREG
--

use work.lpmpkg.all;

entity Mshiftreg is
    generic(lpm_width	: positive;
	    lpm_direction : shdir_type;
	    lpm_avalue	: string;
	    lpm_svalue	: string;
	    lpm_pvalue	: string;
	    lpm_hint	: goal_type);
    port(   data	: in	bit_vector((lpm_width-1) downto 0);
	    clock	: in	bit;
	    enable	: in	bit;
	    shiftin	: in	bit;
	    load	: in	bit;
	    q		: out	bit_vector((lpm_width-1) downto 0);
	    shiftout	: out	bit;
	    aset	: in	bit;
	    aclr	: in	bit;
	    sset	: in	bit;
	    sclr	: in	bit;
	    testenab	: in	bit;
	    testin	: in	bit;
	    testout	: out	bit);
end Mshiftreg;

architecture SIM of Mshiftreg is

    signal qtmp,d,async,set,clr : bit_vector(q'high downto 0);

begin

    async <= restring(lpm_avalue,lpm_width);

    process (data,sset,sclr,load,enable,shiftin,testin,testenab,qtmp)
    begin
	if (testenab = '1') then
	    d(0) <= testin;
	    if (lpm_width > 1) then
		d(q'high downto 1) <= qtmp((qtmp'high - 1) downto 0);
	    end if;
	elsif (sclr = '1' AND enable = '1') then
	    d <= (others => '0');
	elsif (sset = '1' AND enable = '1') then
	    d <= restring(lpm_svalue,lpm_width);
	elsif (load = '1' AND enable = '1') then
	    d <= data;
	elsif (enable = '1') then
	    if (lpm_direction = lpm_left) then
		d(0) <= shiftin;
		d(q'high downto 1) <= qtmp((qtmp'high - 1) downto 0);
	    else
		d(q'high - 1 downto 0) <= qtmp(qtmp'high downto 1);
		d(q'high) <= shiftin;
	    end if;
	else
	    d <= qtmp;
	end if;
    end process;

    process (aset, aclr, clock)
    begin
	for j in q'range loop
	    set(j) <= (aset AND     async(j));
	    clr(j) <= (aset AND NOT async(j)) OR aclr;

	    if    (clr(j) = '1') then qtmp(j) <= '0';
	    elsif (set(j) = '1') then qtmp(j) <= '1';
	    elsif (clock  = '1' AND clock'event) then qtmp(j) <= d(j);
	    end if;
	end loop;
    end process;

    shiftout <= qtmp(qtmp'high) AND NOT load AND NOT testenab
	when (lpm_direction = lpm_left) else
		qtmp(qtmp'low)  AND NOT load AND NOT testenab;
    testout <= qtmp(qtmp'high);
    q <= qtmp;

end SIM;
 
--
--	Expandable VCC
--

use work.lpmpkg.all;
entity MVCC is
    generic(lpm_width		: positive);
    port(	X		: out	bit_vector((lpm_width-1) downto 0));
end MVCC;

architecture SIM of mvcc is
begin

    x <= (others => '1');

end SIM;

--
--	MXOR
--

use work.lpmpkg.all;

entity Mxor is
    generic(lpm_width	: positive;
	    lpm_size	: positive;
	    lpm_hint	: goal_type;
	    lpm_data_pol	: string;
	    lpm_result_pol	: string);
    port(   data	: in	bit_vector(((lpm_width*lpm_size)-1) downto 0);
	    result	: out	bit_vector((lpm_width-1) downto 0));
end Mxor;

architecture SIM of Mxor is

    constant d_pol : bit_vector(((lpm_width*lpm_size)-1) downto 0)
				:= restring(lpm_data_pol,lpm_width*lpm_size);
    constant r_pol : bit_vector(result'range)
				:= restring(lpm_result_pol,lpm_width);

begin

    process (data)
	variable tmp : bit;
    begin
	for n in 0 to lpm_width-1 loop
	    tmp := '0';
	    for m in 0 to lpm_size-1 loop
	        tmp := (tmp XOR data((m*lpm_width)+n))
			XOR NOT d_pol((m*lpm_width)+n);
	    end loop;
	    result(n) <= tmp XOR NOT r_pol(n);
	end loop;
    end process;

end SIM;
